@@ -28,6 +28,7 @@ styling_within_preprocessor=1;0;false;false
 [keywords]
 # all items must be in one line
 primary=and and_eq asm auto bitand bitor bool break case catch char class compl const const_cast continue default delete do double dynamic_cast else enum explicit export extern false float for friend goto if inline int long mutable namespace new not not_eq operator or or_eq private protected public register reinterpret_cast return short signed sizeof static static_cast struct switch template this throw true try typedef typeid typename union unsigned using virtual void volatile wchar_t while xor xor_eq
+secondary=
 # these are some doxygen keywords (incomplete)
 docComment=attention author brief bug class code date def enum example exception file fn namespace note param remarks return see since struct throw todo typedef var version warning union
 
@@ -43,13 +44,16 @@ comment_close=
 #comment_close=*/
 
 # set to false if a comment character/string should start at column 0 of a line, true uses any
-# indention of the line, e.g. setting to true causes the following on pressing CTRL+d
+# indentation of the line, e.g. setting to true causes the following on pressing CTRL+d
 	#command_example();
 # setting to false would generate this
 #	command_example();
 # This setting works only for single line comments
 comment_use_indent=true
 
+# context action command (please see Geany's main documentation for details)
+context_action_cmd=
+
 [build_settings]
 # %f will be replaced by the complete filename
 # %e will be replaced by the filename without extension
