@@ -13,13 +13,16 @@ comment_open=/*
 comment_close=*/
 
 # set to false if a comment character/string should start at column 0 of a line, true uses any
-# indention of the line, e.g. setting to true causes the following on pressing CTRL+d
+# indentation of the line, e.g. setting to true causes the following on pressing CTRL+d
 	#command_example();
 # setting to false would generate this
 #	command_example();
 # This setting works only for single line comments
 comment_use_indent=true
 
+# context action command (please see Geany's main documentation for details)
+context_action_cmd=
+
 [build_settings]
 # %f will be replaced by the complete filename
 # %e will be replaced by the filename without extension
