@@ -2,7 +2,7 @@
 [styling]
 # foreground;background;bold;italic
 default=0x000000;0xffffff;false;false
-commentline=0xff0000;0xffffff;false;false
+commentline=0xd00000;0xffffff;false;false
 number=0x007f00;0xffffff;false;false
 word=0x111199;0xffffff;true;false
 string=0xff901e;0xffffff;false;false
@@ -14,25 +14,28 @@ param=0x009f00;0xffffff;false;false
 scalar=0x105090;0xffffff;false;false
 
 [keywords]
-primary=break case continue do done elif else esac eval exit export fi for function goto if in integer return set shift then while
+primary=break case continue do done elif else esac eval exit export fi for function goto if in integer return set shift then until while
 
 
 [settings]
 # the following characters are these which a "word" can contains, see documentation
-wordchars=_#&abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789
+#wordchars=_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789
 
 # if only single comment char is supported like # in this file, leave comment_close blank
 comment_open=#
 comment_close=
 
 # set to false if a comment character/string should start a column 0 of a line, true uses any
-# indention of the line, e.g. setting to true causes the following on pressing CTRL+d
+# indentation of the line, e.g. setting to true causes the following on pressing CTRL+d
 	#command_example();
 # setting to false would generate this
 #	command_example();
 # This setting works only for single line comments
 comment_use_indent=true
 
+# context action command (please see Geany's main documentation for details)
+context_action_cmd=
+
 [build_settings]
 # %f will be replaced by the complete filename
 # %e will be replaced by the filename without extension
