@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: about.c 1540 2007-05-16 17:08:22Z eht16 $
+ * $Id: about.c 1878 2007-09-12 16:25:19Z ntrel $
  */
 
 /*
@@ -40,58 +40,47 @@ static GtkWidget *gb_window = NULL;
 #define CODENAME "<span weight=\"bold\">\"" GEANY_CODENAME "\"</span>"
 #define BUILDDATE "<span size=\"smaller\">%s</span>"
 #define COPYRIGHT "Copyright (c)  2005-2007\nEnrico Tröger\nNick Treleaven\nFrank Lanitz\nAll rights reserved."
-#define CREDITS \
-"<span size=\"larger\" weight=\"bold\">%s</span>\n\t\
-Enrico Tröger - %s\n\t\
-&lt;enrico.troeger@uvena.de&gt;\n\n\t\
-Nick Treleaven - %s\n\t\
-&lt;nick.treleaven@btinternet.com&gt;\n\n\t\
-Frank Lanitz - %s\n\t\
-&lt;frank@frank.uvena.de&gt;\n\n\t\
-\n<span size=\"larger\" weight=\"bold\">%s</span>\n\t\
-\
-Yura Semashko - <language> be_BY\n\t\
-&lt;yurand2@gmail.com&gt;\n\n\t\
-Dilyan Rusev - <language> bg\n\t\
-&lt;dilyanrusev@gmail.com&gt;\n\n\t\
-Toni Garcia-Navarro - <language> ca_ES\n\t\
-&lt;topi@elpiset.net&gt;\n\n\t\
-Petr Messner - <language> cs_CZ \n\t\
-&lt;messa@messa.cz&gt;\n\n\t\
-Frank Lanitz - <language> de_DE\n\t\
-&lt;frank@frank.uvena.de&gt;\n\n\t\
-Damián Viano - <language> es\n\t\
-&lt;debian@damianv.com.ar&gt;\n\n\t\
-Harri Koskinen  - <language> fi_FI\n\t\
-&lt;harri@fastmonkey.org&gt;\n\n\t\
-Jean-Philippe Moal - <language> fr\n\t\
-&lt;skateinmars@skateinmars.net&gt;\n\n\t\
-Gabor Kmetyko - <language> hu\n\t\
-&lt;kg_kilo@freemail.hu&gt;\n\n\t\
-M. Baldinelli - <language> it\n\t\
-&lt;m.baldinelli@agora.it&gt;\n\n\t\
-Dario Santomarco - <language> it\n\t\
-&lt;dariello@yahoo.it&gt;\n\n\t\
-Kurt De Bree - <language> nl\n\t\
-&lt;kdebree@telenet.be&gt;\n\n\t\
-Jacek Wolszczak - <language> pl_PL\n\t\
-&lt;shutdownrunner@o2.pl&gt;\n\n\t\
-Alexandre Moreira - <language> pt_BR\n\t\
-&lt;alexandream@gmail.com&gt;\n\n\t\
-brahmann_ - <language> ru_RU\n\t\
-&lt;brahmann@mthr.net.ru&gt;\n\n\t\
-Clytie Siddall - <language> vi_VN\n\t\
-&lt;clytie@riverland.net.au&gt;\n\n\t\
-Dormouse Young - <language> zh_CN\n\t\
-&lt;dormouse.young@gmail.com&gt;\n\n\t\
-KoViCH - <language> zh_TW\n\t\
-&lt;kovich.ian@gmail.com&gt;\n\n\t"
+
+const gchar *translators[][2] = {
+	{ "be_BY", "Yura Semashko &lt;yurand2@gmail.com&gt;" },
+	{ "bg", "Dilyan Rusev &lt;dilyanrusev@gmail.com&gt;" },
+	{ "ca_ES", "Toni Garcia-Navarro &lt;topi@elpiset.net&gt;" },
+	{ "cs_CZ", "Petr Messner &lt;messa@messa.cz&gt;" },
+	{ "de_DE", "Enrico Tröger &lt;enrico.troeger@uvena.de&gt;\nFrank Lanitz &lt;frank@frank.uvena.de&gt;" },
+	{ "es", "Damián Viano &lt;debian@damianv.com.ar&gt;\nNacho Cabanes &lt;ncabanes@gmail.com&gt;" },
+	{ "fi_FI", "Harri Koskinen &lt;harri@fastmonkey.org&gt;" },
+	{ "fr", "Jean-Philippe Moal &lt;skateinmars@skateinmars.net&gt;" },
+	{ "hu", "Gabor Kmetyko &lt;kg_kilo@freemail.hu&gt;" },
+	{ "it", "Max Baldinelli &lt;m.baldinelli@agora.it&gt;,\nDario Santomarco &lt;dariello@yahoo.it&gt;" },
+	{ "nl", "Kurt De Bree &lt;kdebree@telenet.be&gt;" },
+	{ "pl_PL", "Jarosław Foksa &lt;jfoksa@gmail.com&gt;" },
+	{ "pt_BR", "Alexandra Moreire &lt;alexandream@gmail.com&gt;\nAdrovane Marques Kade &lt;adrovane@gmail.com&gt;" },
+	{ "ru_RU", "brahmann_ &lt;brahmann@mthr.net.ru&gt;" },
+	{ "vi_VN", "Clytie Siddall &lt;clytie@riverland.net.au&gt;" },
+	{ "zh_CN", "Dormouse Young &lt;dormouse.young@gmail.com&gt;" },
+	{ "zh_TW", "KoViCH &lt;kovich.ian@gmail.com&gt;" }
+};
+static const gint translators_len = G_N_ELEMENTS(translators);
+
+const gchar *prev_translators[][2] = {
+	{ "pl_PL", "Jacek Wolszczak &lt;shutdownrunner@o2.pl&gt;" },
+};
+static const gint prev_translators_len = G_N_ELEMENTS(prev_translators);
+
 
 static void header_eventbox_style_set(GtkWidget *widget);
 static void header_label_style_set(GtkWidget *widget);
 static void homepage_clicked(GtkButton *button, gpointer data);
 
 
+#define ROW(text, row, col, x_align, y_padding, col_span) \
+	label = gtk_label_new((text)); \
+	gtk_table_attach(GTK_TABLE(table), label, (col), (col) + (col_span) + 1, (row), (row) + 1, \
+			(GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0, (y_padding)); \
+	gtk_label_set_use_markup(GTK_LABEL(label), TRUE); \
+	gtk_misc_set_alignment(GTK_MISC(label), (x_align), 0);
+
+
 static GtkWidget *create_dialog(void)
 {
 	GtkWidget *dialog;
@@ -103,11 +92,11 @@ static GtkWidget *create_dialog(void)
 	GtkWidget *url_button;
 	GtkWidget *cop_label;
 	GtkWidget *label;
-	GtkWidget *credits_label;
 	GtkWidget *license_textview;
 	GtkWidget *notebook;
 	GtkWidget *box;
 	GtkWidget *credits_scrollwin;
+	GtkWidget *table;
 	GtkWidget *license_scrollwin;
 	GtkWidget *info_box;
 	GtkWidget *button;
@@ -118,11 +107,11 @@ static GtkWidget *create_dialog(void)
 	gchar *license_text = NULL;
 	gchar buffer[512];
 	gchar buffer2[128];
-	gchar *credits;
+	gint i, row = 0;
 
 	dialog = gtk_dialog_new();
 
-	//configure dialog
+	// configure dialog
 	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(app->window));
 	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER_ON_PARENT);
 	gtk_window_set_title(GTK_WINDOW(dialog), _("About Geany"));
@@ -143,8 +132,9 @@ static GtkWidget *create_dialog(void)
 	gtk_box_pack_start(GTK_BOX(header_hbox), header_image, FALSE,FALSE,0);
 	header_label = gtk_label_new(NULL);
 	gtk_label_set_use_markup(GTK_LABEL(header_label), TRUE);
-	// print the subversion revision if it is available
-	g_snprintf(buffer, sizeof(buffer), HEADER, (utils_str_equal(REVISION, "-1")) ? "" : " (" REVISION ")");
+	// print the subversion revision generated by ./configure if it is available
+	g_snprintf(buffer, sizeof(buffer), HEADER,
+		(utils_str_equal(REVISION, "-1")) ? "" : "svn >= " REVISION);
 	gtk_label_set_markup(GTK_LABEL(header_label), buffer);
 	gtk_widget_show(header_label);
 	gtk_box_pack_start(GTK_BOX(header_hbox), header_label, FALSE,FALSE,0);
@@ -168,7 +158,7 @@ static GtkWidget *create_dialog(void)
 
 	// create "Info" tab
 	info_box = gtk_vbox_new(FALSE, 0);
-	//gtk_container_set_border_width(GTK_CONTAINER(info_box), 6);
+	gtk_container_set_border_width(GTK_CONTAINER(info_box), 6);
 	gtk_widget_show(info_box);
 
 	label_info = gtk_label_new(NULL);
@@ -196,7 +186,7 @@ static GtkWidget *create_dialog(void)
 	gtk_label_set_justify(GTK_LABEL(builddate_label), GTK_JUSTIFY_CENTER);
 	gtk_label_set_selectable(GTK_LABEL(builddate_label), TRUE);
 	gtk_label_set_use_markup(GTK_LABEL(builddate_label), TRUE);
-	g_snprintf(buffer2, sizeof(buffer2), _("(built on %s)"), __DATE__);
+	g_snprintf(buffer2, sizeof(buffer2), _("(built on or after %s)"), __DATE__);
 	g_snprintf(buffer, sizeof(buffer), BUILDDATE, buffer2);
 	gtk_label_set_markup(GTK_LABEL(builddate_label), buffer);
 	gtk_misc_set_padding(GTK_MISC(builddate_label), 2, 2);
@@ -234,19 +224,88 @@ static GtkWidget *create_dialog(void)
 
 	// create "Credits" tab
 	credits_scrollwin = gtk_scrolled_window_new(NULL, NULL);
-	gtk_container_set_border_width(GTK_CONTAINER(credits_scrollwin), 6);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(credits_scrollwin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-	credits_label = gtk_label_new(NULL);
-	gtk_label_set_selectable(GTK_LABEL(credits_label), TRUE);
-	gtk_label_set_use_markup(GTK_LABEL(credits_label), TRUE);
-	credits = g_strdup_printf(CREDITS, _("Developers"), _("Maintainer"), _("developer"),
-							 _("translation maintainer"), _("Translators"));
-	credits = utils_str_replace(credits, "<language>", _("language"));
-	gtk_label_set_markup(GTK_LABEL(credits_label), credits);
-	gtk_misc_set_alignment(GTK_MISC(credits_label), 0.0, 0.0);
-	gtk_widget_show(credits_label);
-	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(credits_scrollwin), credits_label);
-	gtk_viewport_set_shadow_type(GTK_VIEWPORT(gtk_widget_get_parent(credits_label)), GTK_SHADOW_NONE);
+	gtk_container_set_border_width(GTK_CONTAINER(credits_scrollwin), 10);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(credits_scrollwin),
+		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+
+	table = gtk_table_new(14 + translators_len + prev_translators_len, 3, FALSE);
+	gtk_table_set_col_spacings(GTK_TABLE(table), 10);
+
+	row = 0;
+	g_snprintf(buffer, sizeof(buffer),
+		"<span size=\"larger\" weight=\"bold\">%s</span>", _("Developers"));
+	label = gtk_label_new(buffer);
+	gtk_table_attach(GTK_TABLE(table), label, 0, 2, row, row + 1,
+					(GtkAttachOptions) (GTK_FILL),
+					(GtkAttachOptions) (0), 0, 5);
+	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
+	row++;
+
+	g_snprintf(buffer, sizeof(buffer), "Enrico Tröger - %s", _("maintainer"));
+	ROW(buffer, row, 0, 0, 0, 1);
+	row++;
+	ROW("&lt;enrico.troeger@uvena.de&gt;", row, 0, 0, 0, 1);
+	row++;
+	ROW("", row, 0, 0, 0, 0);
+	row++;
+
+	g_snprintf(buffer, sizeof(buffer), "Nick Treleaven - %s", _("developer"));
+	ROW(buffer, row, 0, 0, 0, 1);
+	row++;
+	ROW("&lt;nick.treleaven@btinternet.com&gt;", row, 0, 0, 0, 1);
+	row++;
+	ROW("", row, 0, 0, 0, 0);
+	row++;
+
+	g_snprintf(buffer, sizeof(buffer), "Frank Lanitz - %s", _("translation maintainer"));
+	ROW(buffer, 7, 0, 0, 0, 1);
+	row++;
+	ROW("&lt;frank@frank.uvena.de&gt;", row, 0, 0, 0, 1);
+	row++;
+	ROW("", row, 0, 0, 0, 0);
+	row++;
+
+	g_snprintf(buffer, sizeof(buffer),
+		"<span size=\"larger\" weight=\"bold\">%s</span>", _("Translators"));
+	label = gtk_label_new(buffer);
+	gtk_table_attach(GTK_TABLE(table), label, 0, 2, row, row + 1,
+					(GtkAttachOptions) (GTK_FILL),
+					(GtkAttachOptions) (0), 0, 5);
+	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+	row++;
+
+	for (i = 0; i < translators_len; i++)
+	{
+		ROW(translators[i][0], row, 0, 1, 4, 0);
+		ROW(translators[i][1], row, 1, 0, 4, 0);
+		row++;
+	}
+
+	ROW("", row, 0, 0, 0, 0);
+	row++;
+
+	g_snprintf(buffer, sizeof(buffer),
+		"<span size=\"larger\" weight=\"bold\">%s</span>", _("Previous Translators"));
+	label = gtk_label_new(buffer);
+	gtk_table_attach(GTK_TABLE(table), label, 0, 2, row, row + 1,
+					(GtkAttachOptions) (GTK_FILL),
+					(GtkAttachOptions) (0), 0, 5);
+	gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+	row++;
+
+	for (i = 0; i < prev_translators_len; i++)
+	{
+		ROW(prev_translators[i][0], row, 0, 1, 4, 0);
+		ROW(prev_translators[i][1], row, 1, 0, 4, 0);
+		row++;
+	}
+
+	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(credits_scrollwin), table);
+	gtk_viewport_set_shadow_type(GTK_VIEWPORT(gtk_widget_get_parent(table)), GTK_SHADOW_NONE);
+	gtk_widget_show_all(table);
 	label = gtk_label_new(_("Credits"));
 	gtk_widget_show(label);
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), credits_scrollwin, label);
@@ -254,7 +313,8 @@ static GtkWidget *create_dialog(void)
 	// create "License" tab
 	license_scrollwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_container_set_border_width(GTK_CONTAINER(license_scrollwin), 6);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(license_scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(license_scrollwin),
+		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	license_textview = gtk_text_view_new();
 	gtk_text_view_set_editable(GTK_TEXT_VIEW(license_textview), FALSE);
 	gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(license_textview), FALSE);
@@ -268,12 +328,12 @@ static GtkWidget *create_dialog(void)
 	g_file_get_contents(buffer, &license_text, NULL, NULL);
 	if (license_text == NULL)
 	{
-		license_text = g_strdup("License text could not be found, please google for GPLv2");
+		license_text = g_strdup(
+			_("License text could not be found, please visit http://www.gnu.org/licenses/gpl-2.0.txt to view it online."));
 	}
 	tb = gtk_text_view_get_buffer(GTK_TEXT_VIEW(license_textview));
 	gtk_text_buffer_set_text(tb, license_text, strlen(license_text));
 
-	g_free(credits);
 	g_free(license_text);
 
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), license_scrollwin, label);
