@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: build.c 1535 2007-05-15 15:16:16Z eht16 $
+ * $Id: build.c 1915 2007-09-27 15:10:22Z ntrel $
  */
 
 /*
@@ -41,19 +41,24 @@
 # include <signal.h>
 #endif
 
+#include "prefs.h"
 #include "support.h"
 #include "utils.h"
 #include "ui_utils.h"
 #include "dialogs.h"
 #include "msgwindow.h"
 #include "document.h"
+#include "filetypes.h"
 #include "keybindings.h"
 #include "vte.h"
 #include "project.h"
+#include "editor.h"
 
 
 BuildInfo build_info = {GBO_COMPILE, 0, NULL, GEANY_FILETYPES_ALL, NULL};
 
+static gchar *current_dir_entered = NULL;
+
 static struct
 {
 	GPid pid;
@@ -80,6 +85,14 @@ static BuildMenuItems latex_menu_items =
 	{NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
 
 
+static struct
+{
+	GtkWidget	*run_button;
+	GtkWidget	*compile_button;
+}
+widgets;
+
+
 static gboolean build_iofunc(GIOChannel *ioc, GIOCondition cond, gpointer data);
 static gboolean build_create_shellscript(const gchar *fname, const gchar *cmd, gboolean autoclose);
 static GPid build_spawn_cmd(gint idx, const gchar *cmd, const gchar *dir);
@@ -88,9 +101,15 @@ static void on_make_target_entry_activate(GtkEntry *entry, gpointer user_data);
 static void set_stop_button(gboolean stop);
 static void build_exit_cb(GPid child_pid, gint status, gpointer user_data);
 static void run_exit_cb(GPid child_pid, gint status, gpointer user_data);
-static void
-on_build_arguments_activate            (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
+static void on_build_arguments_activate(GtkMenuItem *menuitem, gpointer user_data);
+
+static void on_build_compile_activate(GtkMenuItem *menuitem, gpointer user_data);
+static void on_build_tex_activate(GtkMenuItem *menuitem, gpointer user_data);
+static void on_build_build_activate(GtkMenuItem *menuitem, gpointer user_data);
+static void on_build_make_activate(GtkMenuItem *menuitem, gpointer user_data);
+static void on_build_execute_activate(GtkMenuItem *menuitem, gpointer user_data);
+static void on_build_next_error(GtkMenuItem *menuitem, gpointer user_data);
+
 
 #ifndef G_OS_WIN32
 static void kill_process(GPid *pid);
@@ -173,7 +192,7 @@ static GPid build_view_tex_file(gint idx, gint mode)
 	locale_cmd_string = utils_get_locale_from_utf8(cmd_string);
 
 	/* get the terminal path */
-	locale_term_cmd = utils_get_locale_from_utf8(app->tools_term_cmd);
+	locale_term_cmd = utils_get_locale_from_utf8(prefs.tools_term_cmd);
 	// split the term_cmd, so arguments will work too
 	term_argv = g_strsplit(locale_term_cmd, " ", -1);
 	term_argv_len = g_strv_length(term_argv);
@@ -189,8 +208,8 @@ static GPid build_view_tex_file(gint idx, gint mode)
 	if (term_argv[0] == NULL)
 	{
 		msgwin_status_add(
-			_("Could not find terminal '%s' "
-				"(check path for Terminal tool setting in Preferences)"), app->tools_term_cmd);
+			_("Could not find terminal \"%s\" "
+				"(check path for Terminal tool setting in Preferences)"), prefs.tools_term_cmd);
 
 		utils_free_pointers(executable, view_file, locale_filename, cmd_string, locale_cmd_string,
 										locale_term_cmd, NULL);
@@ -286,7 +305,7 @@ static GPid build_make_file(gint idx, gint build_opts)
 
 	if (idx < 0 || doc_list[idx].file_name == NULL) return (GPid) 1;
 
-	cmdstr = g_string_new(app->tools_make_cmd);
+	cmdstr = g_string_new(prefs.tools_make_cmd);
 	g_string_append_c(cmdstr, ' ');
 
 	if (build_opts == GBO_MAKE_OBJECT)
@@ -454,6 +473,7 @@ static GPid build_spawn_cmd(gint idx, const gchar *cmd, const gchar *dir)
 	g_return_val_if_fail(DOC_IDX_VALID(idx), (GPid) 1);
 
 	clear_errors(idx);
+	setptr(current_dir_entered, NULL);
 
 	locale_filename = utils_get_locale_from_utf8(doc_list[idx].file_name);
 	executable = utils_remove_ext_from_filename(locale_filename);
@@ -523,35 +543,26 @@ static GPid build_spawn_cmd(gint idx, const gchar *cmd, const gchar *dir)
 }
 
 
-// Returns: NULL if there was an error, or the working directory the script was created in.
-static gchar *prepare_run_script(gint idx)
+/* Checks if the executable file corresponding to document idx exists.
+ * Returns the name part of the filename, without extension.
+ * Returns NULL if executable file doesn't exist. */
+static gchar *get_build_executable(const gchar *locale_filename, gboolean check_exists,
+		filetype_id ft_id)
 {
-	gchar	*long_executable = NULL;
-	gchar	*check_executable = NULL;
-	gchar	*utf8_check_executable = NULL;
-	gchar	*locale_filename = NULL;
-	gchar	*cmd = NULL;
-	gchar	*executable = NULL;
-	gchar	*working_dir = NULL;
-	gboolean autoclose = FALSE;
+	gchar *long_executable = NULL;
 	struct stat st;
-	gboolean result = FALSE;
-	gchar	*tmp;
-
-	locale_filename = utils_get_locale_from_utf8(doc_list[idx].file_name);
 
 	long_executable = utils_remove_ext_from_filename(locale_filename);
 #ifdef G_OS_WIN32
-	tmp = long_executable;
-	long_executable = g_strconcat(long_executable, ".exe", NULL);
-	g_free(tmp);
+	setptr(long_executable, g_strconcat(long_executable, ".exe", NULL));
 #endif
 
-	// only check for existing executable, if executable is required by %e
-	if (strstr(doc_list[idx].file_type->programs->run_cmd, "%e") != NULL)
+	if (check_exists)
 	{
+		gchar *check_executable = NULL;
+
 		// add .class extension for JAVA source files (only for stat)
-		if (doc_list[idx].file_type->id == GEANY_FILETYPES_JAVA)
+		if (ft_id == GEANY_FILETYPES_JAVA)
 		{
 #ifdef G_OS_WIN32
 			gchar *tmp;
@@ -571,34 +582,79 @@ static gchar *prepare_run_script(gint idx)
 		{
 			msgwin_status_add(_("Command stopped because the current file has no extension."));
 			utils_beep();
-			goto free_strings;
+			g_free(check_executable);
+			return NULL;
 		}
 
 		// check whether executable exists
 		if (g_stat(check_executable, &st) != 0)
 		{
-			utf8_check_executable = utils_get_utf8_from_locale(check_executable);
-			msgwin_status_add(_("Failed to execute %s (make sure it is already built)"),
+			gchar *utf8_check_executable = utils_get_utf8_from_locale(check_executable);
+
+			msgwin_status_add(_("Failed to execute \"%s\" (make sure it is already built)"),
 														utf8_check_executable);
-			goto free_strings;
+			g_free(utf8_check_executable);
+			g_free(check_executable);
+			return NULL;
 		}
 	}
 
-	executable = g_path_get_basename(long_executable);
+	// remove path
+	setptr(long_executable, g_path_get_basename(long_executable));
+	return long_executable;
+}
+
+
+// Returns: NULL if there was an error, or the working directory the script was created in.
+static gchar *prepare_run_script(gint idx)
+{
+	gchar	*locale_filename = NULL;
+	gboolean have_project;
+	GeanyProject *project = app->project;
+	filetype *ft = doc_list[idx].file_type;
+	gboolean check_exists;
+	gchar	*cmd = NULL;
+	gchar	*executable = NULL;
+	gchar	*working_dir = NULL;
+	gboolean autoclose = FALSE;
+	gboolean result = FALSE;
+	gchar	*tmp;
+
+	locale_filename = utils_get_locale_from_utf8(doc_list[idx].file_name);
+
+	have_project = (project != NULL && NZV(project->run_cmd));
+	cmd = (have_project) ?
+		project->run_cmd :
+		ft->programs->run_cmd;
+
+	// only check for existing executable, if executable is required by %e
+	check_exists = (strstr(cmd, "%e") != NULL);
+	executable = get_build_executable(locale_filename, check_exists, FILETYPE_ID(ft));
+	if (executable == NULL)
+	{
+		g_free(locale_filename);
+		return NULL;
+	}
+
+	working_dir = (have_project) ?
+		utils_get_locale_from_utf8(project->base_path) :
+		g_path_get_dirname(locale_filename);
 
-	working_dir = g_path_get_dirname(locale_filename);
 	if (chdir(working_dir) != 0)
 	{
-		gchar *utf8_working_dir = NULL;
-		utf8_working_dir = utils_get_utf8_from_locale(working_dir);
+		gchar *utf8_working_dir =
+			utils_get_utf8_from_locale(working_dir);
 
-		msgwin_status_add(_("Failed to change the working directory to %s"), utf8_working_dir);
+		msgwin_status_add(_("Failed to change the working directory to \"%s\""), utf8_working_dir);
 		g_free(utf8_working_dir);
-		goto free_strings;
+		g_free(working_dir);
+		g_free(executable);
+		g_free(locale_filename);
+		return NULL;
 	}
 
 	// replace %f and %e in the run_cmd string
-	cmd = g_strdup(doc_list[idx].file_type->programs->run_cmd);
+	cmd = g_strdup(cmd);
 	tmp = g_path_get_basename(locale_filename);
 	cmd = utils_str_replace(cmd, "%f", tmp);
 	g_free(tmp);
@@ -610,24 +666,19 @@ static gchar *prepare_run_script(gint idx)
 #endif
 
 	// (RUN_SCRIPT_CMD should be ok in UTF8 without converting in locale because it contains no umlauts)
-	if (! build_create_shellscript(RUN_SCRIPT_CMD, cmd, autoclose))
+	result = build_create_shellscript(RUN_SCRIPT_CMD, cmd, autoclose);
+	if (! result)
 	{
-		utf8_check_executable = utils_get_utf8_from_locale(check_executable);
+		gchar *utf8_cmd = utils_get_utf8_from_locale(cmd);
+
 		msgwin_status_add(_("Failed to execute \"%s\" (start-script could not be created)"),
-													utf8_check_executable);
-	}
-	else
-	{
-		result = TRUE;
+			utf8_cmd);
+		g_free(utf8_cmd);
 	}
 
-	free_strings:
-	g_free(executable);
 	g_free(cmd);
+	g_free(executable);
 	g_free(locale_filename);
-	g_free(utf8_check_executable);
-	g_free(check_executable);
-	g_free(long_executable);
 
 	if (result)
 		return working_dir;
@@ -637,57 +688,15 @@ static gchar *prepare_run_script(gint idx)
 }
 
 
-static gchar *prepare_project_run_script()
-{
-	GeanyProject *project = app->project;
-	gboolean autoclose = FALSE;
-	gchar *working_dir;
-	gchar *cmd;
-
-	if (project == NULL || project->run_cmd == NULL) return NULL;
-	g_return_val_if_fail(project->base_path != NULL, NULL);
-
-	working_dir = utils_get_locale_from_utf8(project->base_path);
-	if (chdir(working_dir) != 0)
-	{
-		msgwin_status_add(_("Failed to change the working directory to %s"), project->base_path);
-		g_free(working_dir);
-		return NULL;
-	}
-
-#ifdef HAVE_VTE
-	if (vte_info.load_vte && vc != NULL && vc->run_in_vte)
-		autoclose = TRUE; // don't wait for user input at the end of script when we are running in VTE
-#endif
-	cmd = utils_get_locale_from_utf8(project->run_cmd);
-
-	// (RUN_SCRIPT_CMD should be ok in UTF8 without converting in locale because it contains no umlauts)
-	if (! build_create_shellscript(RUN_SCRIPT_CMD, cmd, autoclose))
-	{
-		msgwin_status_add(_("Failed to execute \"%s\" (start-script could not be created)"),
-													project->run_cmd);
-		g_free(working_dir);
-		g_free(cmd);
-		return NULL;
-	}
-	g_free(cmd);
-	return working_dir;
-}
-
-
 static GPid build_run_cmd(gint idx)
 {
-	GeanyProject *project = app->project;
 	gchar	*working_dir;
 	GError	*error = NULL;
 
-	if (! DOC_IDX_VALID(idx) || doc_list[idx].file_name == NULL) return (GPid) 1;
-
-	if (project != NULL && project->run_cmd != NULL && *project->run_cmd != 0)
-		working_dir = prepare_project_run_script();
-	else
-		working_dir = prepare_run_script(idx);
+	if (! DOC_IDX_VALID(idx) || doc_list[idx].file_name == NULL)
+		return (GPid) 1;
 
+	working_dir = prepare_run_script(idx);
 	if (working_dir == NULL)
 	{
 		return (GPid) 1;
@@ -721,7 +730,7 @@ static GPid build_run_cmd(gint idx)
 		gchar  **argv = NULL;
 
 		/* get the terminal path */
-		locale_term_cmd = utils_get_locale_from_utf8(app->tools_term_cmd);
+		locale_term_cmd = utils_get_locale_from_utf8(prefs.tools_term_cmd);
 		// split the term_cmd, so arguments will work too
 		term_argv = g_strsplit(locale_term_cmd, " ", -1);
 		term_argv_len = g_strv_length(term_argv);
@@ -737,8 +746,8 @@ static GPid build_run_cmd(gint idx)
 		if (term_argv[0] == NULL)
 		{
 			msgwin_status_add(
-				_("Could not find terminal '%s' "
-					"(check path for Terminal tool setting in Preferences)"), app->tools_term_cmd);
+				_("Could not find terminal \"%s\" "
+					"(check path for Terminal tool setting in Preferences)"), prefs.tools_term_cmd);
 			run_info.pid = (GPid) 1;
 			goto free_strings;
 		}
@@ -790,7 +799,7 @@ static gboolean build_iofunc(GIOChannel *ioc, GIOCondition cond, gpointer data)
 	if (cond & (G_IO_IN | G_IO_PRI))
 	{
 		//GIOStatus s;
-		gchar *msg, *dir = NULL;
+		gchar *msg;
 
 		while (g_io_channel_read_line(ioc, &msg, NULL, NULL, NULL) && msg)
 		{
@@ -803,20 +812,20 @@ static gboolean build_iofunc(GIOChannel *ioc, GIOCondition cond, gpointer data)
 
 			if (build_parse_make_dir(msg, &tmp))
 			{
-				if (dir != NULL)
-					g_free(dir);
-				dir = tmp;
+				setptr(current_dir_entered, tmp);
 			}
 
-			if (app->pref_editor_use_indicators)
+			if (editor_prefs.use_indicators)
 			{
 				gchar *filename;
 				gint line;
 
-				msgwin_parse_compiler_error_line(msg, dir, &filename, &line);
+				msgwin_parse_compiler_error_line(msg, current_dir_entered,
+					&filename, &line);
 				if (line != -1 && filename != NULL)
 				{
 					gint idx = document_find_by_filename(filename, FALSE);
+
 					document_set_indicator(idx, line - 1);	// will check valid idx
 					color = COLOR_RED;	// error message parsed on the line
 				}
@@ -826,9 +835,6 @@ static gboolean build_iofunc(GIOChannel *ioc, GIOCondition cond, gpointer data)
 
 			g_free(msg);
 		}
-
-		if (dir != NULL)
-			g_free(dir);
 	}
 	if (cond & (G_IO_ERR | G_IO_HUP | G_IO_NVAL))
 		return FALSE;
@@ -882,7 +888,7 @@ static void show_build_result_message(gboolean failure)
 		msg = _("Compilation failed.");
 		msgwin_compiler_add(COLOR_DARK_RED, msg);
 		// If msgwindow is hidden, user will want to display it to see the error
-		if (! app->msgwindow_visible)
+		if (! ui_prefs.msgwindow_visible)
 		{
 			gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_COMPILER);
 			msgwin_show_hide(TRUE);
@@ -895,7 +901,7 @@ static void show_build_result_message(gboolean failure)
 	{
 		msg = _("Compilation finished successfully.");
 		msgwin_compiler_add(COLOR_BLUE, msg);
-		if (! app->msgwindow_visible ||
+		if (! ui_prefs.msgwindow_visible ||
 			gtk_notebook_get_current_page(GTK_NOTEBOOK(msgwindow.notebook)) != MSG_COMPILER)
 				ui_set_statusbar("%s", msg);
 	}
@@ -953,7 +959,7 @@ static gboolean build_create_shellscript(const gchar *fname, const gchar *cmd, g
 	gchar *tmp;
 #endif
 
-	fp = fopen(fname, "w");
+	fp = g_fopen(fname, "w");
 	if (! fp) return FALSE;
 
 #ifdef G_OS_WIN32
@@ -1055,7 +1061,6 @@ static void create_build_menu_gen(BuildMenuItems *menu_items)
 	g_signal_connect((gpointer) item, "activate", G_CALLBACK(on_build_make_activate),
 		GINT_TO_POINTER(GBO_MAKE_OBJECT));
 	menu_items->item_make_object = item;
-#endif
 
 	item = gtk_separator_menu_item_new();
 	gtk_widget_show(item);
@@ -1072,6 +1077,7 @@ static void create_build_menu_gen(BuildMenuItems *menu_items)
 	item = gtk_separator_menu_item_new();
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
+#endif
 
 	// execute the code
 	item = gtk_image_menu_item_new_from_stock("gtk-execute", accel_group);
@@ -1170,7 +1176,6 @@ static void create_build_menu_tex(BuildMenuItems *menu_items)
 	item = gtk_separator_menu_item_new();
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-#endif
 
 	// next error
 	item = gtk_image_menu_item_new_with_mnemonic(_("_Next Error"));
@@ -1183,6 +1188,7 @@ static void create_build_menu_tex(BuildMenuItems *menu_items)
 	item = gtk_separator_menu_item_new();
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
+#endif
 
 	// DVI view
 #define LATEX_VIEW_DVI_LABEL _("View DVI File") // used later again
@@ -1313,7 +1319,7 @@ static void show_includes_arguments_tex()
 	gtk_container_add(GTK_CONTAINER(vbox), table);
 
 	// LaTeX -> DVI args
-	if (ft->programs->compiler != NULL)
+	if (ft->programs->compiler != NULL && ft->actions->can_compile)
 	{
 		label = gtk_label_new(_("DVI creation:"));
 		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
@@ -1332,7 +1338,7 @@ static void show_includes_arguments_tex()
 	}
 
 	// LaTeX -> PDF args
-	if (ft->programs->linker != NULL)
+	if (ft->programs->linker != NULL && ft->actions->can_link)
 	{
 		label = gtk_label_new(_("PDF creation:"));
 		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
@@ -1480,7 +1486,8 @@ static void show_includes_arguments_gen()
 	if (ft->actions->can_compile || ft->actions->can_link || ft->actions->can_exec)
 	{
 		GtkWidget *align, *frame;
-		gchar *frame_title = g_strconcat(ft->title, _(" commands"), NULL);
+		// in-dialog heading for the "Set Includes and Arguments" dialog
+		gchar *frame_title = g_strdup_printf(_("%s commands"), ft->title);
 
 		frame = ui_frame_new_with_alignment(frame_title, &align);
 		gtk_container_add(GTK_CONTAINER(vbox), frame);
@@ -1613,8 +1620,8 @@ void build_menu_update(gint idx)
 	{
 		gtk_widget_set_sensitive(lookup_widget(app->window, "menu_build1"), FALSE);
 		gtk_menu_item_remove_submenu(GTK_MENU_ITEM(lookup_widget(app->window, "menu_build1")));
-		gtk_widget_set_sensitive(app->compile_button, FALSE);
-		gtk_widget_set_sensitive(app->run_button, FALSE);
+		gtk_widget_set_sensitive(widgets.compile_button, FALSE);
+		gtk_widget_set_sensitive(widgets.run_button, FALSE);
 		return;
 	}
 	else
@@ -1679,8 +1686,8 @@ void build_menu_update(gint idx)
 	if (menu_items->item_set_args)
 		gtk_widget_set_sensitive(menu_items->item_set_args, can_set_args);
 
-	gtk_widget_set_sensitive(app->compile_button, can_build && ft->actions->can_compile);
-	gtk_widget_set_sensitive(app->run_button, can_run || can_stop);
+	gtk_widget_set_sensitive(widgets.compile_button, can_build && ft->actions->can_compile);
+	gtk_widget_set_sensitive(widgets.run_button, can_run || can_stop);
 
 	// show the stop command if a program is running, otherwise show run command
 	set_stop_button(can_stop);
@@ -1688,7 +1695,8 @@ void build_menu_update(gint idx)
 	// simply enable next error command if the compiler window has any items
 	have_errors = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(msgwindow.store_compiler),
 		NULL) > 0;
-	gtk_widget_set_sensitive(menu_items->item_next_error, have_errors);
+	if (menu_items->item_next_error)
+		gtk_widget_set_sensitive(menu_items->item_next_error, have_errors);
 }
 
 
@@ -1700,14 +1708,14 @@ static void set_stop_button(gboolean stop)
 		build_get_menu_items(run_info.file_type_id)->item_exec;
 
 	if (stop && utils_str_equal(
-		gtk_tool_button_get_stock_id(GTK_TOOL_BUTTON(app->run_button)), "gtk-stop")) return;
+		gtk_tool_button_get_stock_id(GTK_TOOL_BUTTON(widgets.run_button)), "gtk-stop")) return;
 	if (! stop && utils_str_equal(
-		gtk_tool_button_get_stock_id(GTK_TOOL_BUTTON(app->run_button)), "gtk-execute")) return;
+		gtk_tool_button_get_stock_id(GTK_TOOL_BUTTON(widgets.run_button)), "gtk-execute")) return;
 
 	// use the run button also as stop button
 	if (stop)
 	{
-		gtk_tool_button_set_stock_id(GTK_TOOL_BUTTON(app->run_button), "gtk-stop");
+		gtk_tool_button_set_stock_id(GTK_TOOL_BUTTON(widgets.run_button), "gtk-stop");
 
 		if (menuitem != NULL)
 		{
@@ -1720,7 +1728,7 @@ static void set_stop_button(gboolean stop)
 	}
 	else
 	{
-		gtk_tool_button_set_stock_id(GTK_TOOL_BUTTON(app->run_button), "gtk-execute");
+		gtk_tool_button_set_stock_id(GTK_TOOL_BUTTON(widgets.run_button), "gtk-execute");
 
 		if (menuitem != NULL)
 		{
@@ -1778,7 +1786,7 @@ BuildMenuItems *build_get_menu_items(gint filetype_idx)
 }
 
 
-void
+static void
 on_build_compile_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
@@ -1795,7 +1803,7 @@ on_build_compile_activate              (GtkMenuItem     *menuitem,
 }
 
 
-void
+static void
 on_build_tex_activate                  (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
@@ -1815,7 +1823,7 @@ on_build_tex_activate                  (GtkMenuItem     *menuitem,
 }
 
 
-void
+static void
 on_build_build_activate                (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
@@ -1832,7 +1840,7 @@ on_build_build_activate                (GtkMenuItem     *menuitem,
 }
 
 
-void
+static void
 on_build_make_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
@@ -1865,7 +1873,7 @@ on_build_make_activate                 (GtkMenuItem     *menuitem,
 }
 
 
-void
+static void
 on_build_execute_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
@@ -1962,7 +1970,7 @@ static void kill_process(GPid *pid)
 #endif
 
 
-void
+static void
 on_build_next_error                    (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
@@ -1976,3 +1984,19 @@ on_build_next_error                    (GtkMenuItem     *menuitem,
 }
 
 
+void build_init()
+{
+	widgets.compile_button = lookup_widget(app->window, "toolbutton13");
+	widgets.run_button = lookup_widget(app->window, "toolbutton26");
+
+#ifdef G_OS_WIN32
+	// hide build support items, at least until they are available for Windows
+	gtk_widget_hide(widgets.compile_button);
+	{
+		GtkWidget *compiler_tab;
+		compiler_tab = gtk_notebook_get_tab_label(GTK_NOTEBOOK(msgwindow.notebook),
+			gtk_notebook_get_nth_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_COMPILER));
+		gtk_widget_set_sensitive(compiler_tab, FALSE);
+	}
+#endif
+}
