@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: build.h 1418 2007-03-24 12:40:20Z ntrel $
+ * $Id: build.h 1884 2007-09-17 11:16:48Z ntrel $
  */
 
 
@@ -34,7 +34,7 @@ typedef enum	// Geany Build Options
 	GBO_MAKE_OBJECT
 } build_type;
 
-typedef struct
+typedef struct BuildInfo
 {
 	build_type	type;	// current action(one of the above enumeration)
 	GPid		pid;	// process id of the spawned process
@@ -45,7 +45,8 @@ typedef struct
 
 extern BuildInfo build_info;
 
-typedef struct
+
+typedef struct BuildMenuItems
 {
 	GtkWidget		*menu;
 	GtkWidget		*item_compile;
@@ -61,8 +62,11 @@ typedef struct
 
 
 
+void build_init();
+
 void build_finalize();
 
+
 gboolean build_parse_make_dir(const gchar *string, gchar **prefix);
 
 void build_menu_update(gint idx);
@@ -70,28 +74,4 @@ void build_menu_update(gint idx);
 BuildMenuItems *build_get_menu_items(gint filetype_idx);
 
 
-void
-on_build_compile_activate              (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_build_tex_activate                  (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_build_build_activate                (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_build_make_activate                 (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_build_execute_activate              (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_build_next_error                    (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 #endif
