@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: callbacks.h 1511 2007-05-06 14:49:51Z eht16 $
+ * $Id: callbacks.h 1860 2007-09-07 19:01:55Z eht16 $
  */
 
 typedef struct
@@ -29,9 +29,6 @@ typedef struct
 extern CallbacksData callbacks_data;
 
 
-gint
-destroyapp                             (GtkWidget *widget, gpointer gdata);
-
 gboolean
 on_exit_clicked                        (GtkWidget *widget, gpointer gdata);
 
@@ -91,34 +88,12 @@ void
 on_toolbutton23_clicked                (GtkToolButton   *toolbutton,
                                         gpointer         user_data);
 
-void
-on_file_open_dialog_response           (GtkDialog *dialog,
-                                        gint response,
-                                        gpointer user_data);
-
 void
 on_notebook1_switch_page               (GtkNotebook     *notebook,
                                         GtkNotebookPage *page,
                                         guint            page_num,
                                         gpointer         user_data);
 
-void
-on_file_save_dialog_response           (GtkDialog *dialog,
-                                        gint response,
-                                        gpointer user_data);
-
-void
-on_color_ok_button_clicked             (GtkButton       *button,
-                                        gpointer         user_data);
-
-void
-on_color_cancel_button_clicked         (GtkButton       *button,
-                                        gpointer         user_data);
-
-void
-on_font_cancel_button_clicked          (GtkButton       *button,
-                                        gpointer         user_data);
-
 void
 on_save_all1_activate                  (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -127,14 +102,6 @@ void
 on_close1_activate                     (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_font_ok_button_clicked              (GtkButton       *button,
-                                        gpointer         user_data);
-
-void
-on_font_apply_button_clicked           (GtkButton       *button,
-                                        gpointer         user_data);
-
 gboolean
 on_close_all1_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -229,22 +196,13 @@ on_entry1_changed                      (GtkEditable     *editable,
                                         gpointer         user_data);
 
 void
-on_to_lower_case1_activate             (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_to_upper_case1_activate             (GtkMenuItem     *menuitem,
+on_toggle_case1_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
 on_fullscreen1_activate                (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-gboolean
-on_window_configure_event              (GtkWidget *widget,
-                                        GdkEventConfigure *event,
-                                        gpointer user_data);
-
 void
 on_show_toolbar1_toggled               (GtkCheckMenuItem *checkmenuitem,
                                         gpointer         user_data);
@@ -398,14 +356,6 @@ void
 on_insert_date_activate                (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_file_open_selection_changed         (GtkFileChooser  *filechooser,
-                                        gpointer         user_data);
-
-void
-on_file_open_entry_activate            (GtkEntry        *entry,
-                                        gpointer         user_data);
-
 void
 on_toolbar_large_icons1_activate       (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -418,21 +368,12 @@ void
 on_set_file_readonly1_toggled          (GtkCheckMenuItem *checkmenuitem,
                                         gpointer         user_data);
 
-void
-on_file_open_check_hidden_toggled      (GtkToggleButton *togglebutton,
-                                        gpointer         user_data);
-
 void
 on_tv_notebook_switch_page             (GtkNotebook     *notebook,
                                         GtkNotebookPage *page,
                                         guint            page_num,
                                         gpointer         user_data);
 
-gboolean
-on_window_key_press_event              (GtkWidget *widget,
-                                        GdkEventKey *event,
-                                        gpointer user_data);
-
 void
 on_help_shortcuts1_activate            (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -447,7 +388,7 @@ on_show_line_numbers1_toggled          (GtkCheckMenuItem *checkmenuitem,
                                         gpointer         user_data);
 
 void
-on_use_auto_indention1_toggled         (GtkCheckMenuItem *checkmenuitem,
+on_use_auto_indentation1_toggled       (GtkCheckMenuItem *checkmenuitem,
                                         gpointer         user_data);
 
 
@@ -540,10 +481,6 @@ on_next_message1_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 
-void
-on_menu_insert_special_chars1_activate (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 void
 on_menu_comments_multiline_activate    (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -608,3 +545,30 @@ void
 on_menu_toggle_all_additional_widgets1_activate
                                         (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
+void
+on_back_activate                       (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_forward_activate                    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_file1_activate                      (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+gboolean
+on_motion_event                        (GtkWidget       *widget,
+                                        GdkEventMotion  *event,
+                                        gpointer         user_data);
+
+
+gboolean
+on_entry1_key_press_event              (GtkWidget       *widget,
+                                        GdkEventKey     *event,
+                                        gpointer         user_data);
+
+gboolean
+on_entry_goto_line_key_press_event     (GtkWidget       *widget,
+                                        GdkEventKey     *event,
+                                        gpointer         user_data);
