@@ -18,11 +18,11 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: dialogs.c 1535 2007-05-15 15:16:16Z eht16 $
+ * $Id: dialogs.c 1818 2007-08-24 11:19:26Z ntrel $
  */
 
 /*
- * File related dialogs, miscellaneous dialogs.
+ * File related dialogs, miscellaneous dialogs, font dialog.
  */
 
 #include "geany.h"
@@ -43,8 +43,10 @@
 
 #include "dialogs.h"
 
+#include "prefs.h"
 #include "callbacks.h"
 #include "document.h"
+#include "filetypes.h"
 #include "win32.h"
 #include "sciwrappers.h"
 #include "support.h"
@@ -52,6 +54,7 @@
 #include "ui_utils.h"
 #include "keybindings.h"
 #include "encodings.h"
+#include "build.h"
 
 
 #if ! GEANY_USE_WIN32_DIALOG
@@ -59,6 +62,101 @@ static GtkWidget *add_file_open_extra_widget();
 #endif
 
 
+#if ! GEANY_USE_WIN32_DIALOG
+static void
+on_file_open_dialog_response           (GtkDialog *dialog,
+                                        gint response,
+                                        gpointer user_data)
+{
+	gtk_widget_hide(ui_widgets.open_filesel);
+
+	if (response == GTK_RESPONSE_ACCEPT || response == GTK_RESPONSE_APPLY)
+	{
+		GSList *filelist;
+		gint filetype_idx = gtk_combo_box_get_active(GTK_COMBO_BOX(
+						lookup_widget(GTK_WIDGET(dialog), "filetype_combo")));
+		gint encoding_idx = gtk_combo_box_get_active(GTK_COMBO_BOX(
+						lookup_widget(GTK_WIDGET(dialog), "encoding_combo")));
+		filetype *ft = NULL;
+		gchar *charset = NULL;
+		gboolean ro = (response == GTK_RESPONSE_APPLY);	// View clicked
+
+		if (filetype_idx >= 0 && filetype_idx < GEANY_FILETYPES_ALL) ft = filetypes[filetype_idx];
+		if (encoding_idx >= 0 && encoding_idx < GEANY_ENCODINGS_MAX)
+			charset = encodings[encoding_idx].charset;
+
+		filelist = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(ui_widgets.open_filesel));
+		if (filelist != NULL)
+		{
+			document_open_files(filelist, ro, ft, charset);
+			g_slist_foreach(filelist, (GFunc) g_free, NULL);	// free filenames
+		}
+		g_slist_free(filelist);
+	}
+}
+#endif
+
+
+#if ! GEANY_USE_WIN32_DIALOG
+// callback for the text entry for typing in filename
+static void
+on_file_open_entry_activate            (GtkEntry        *entry,
+                                        gpointer         user_data)
+{
+	gchar *locale_filename = utils_get_locale_from_utf8(gtk_entry_get_text(entry));
+
+	if (g_file_test(locale_filename, G_FILE_TEST_IS_DIR))
+	{
+		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(ui_widgets.open_filesel), locale_filename);
+	}
+	else if (g_file_test(locale_filename, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
+	{
+		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(ui_widgets.open_filesel), locale_filename);
+		on_file_open_dialog_response(GTK_DIALOG(ui_widgets.open_filesel), GTK_RESPONSE_ACCEPT, NULL);
+	}
+
+	g_free(locale_filename);
+}
+#endif
+
+
+#if ! GEANY_USE_WIN32_DIALOG
+static void
+on_file_open_selection_changed         (GtkFileChooser  *filechooser,
+                                        gpointer         user_data)
+{
+	gchar *filename = gtk_file_chooser_get_filename(filechooser);
+	gboolean is_on = gtk_file_chooser_get_show_hidden(filechooser);
+
+	if (filename)
+	{
+		// try to get the UTF-8 equivalent for the filename, fallback to filename if error
+		gchar *utf8_filename = utils_get_utf8_from_locale(filename);
+
+		gtk_entry_set_text(GTK_ENTRY(lookup_widget(
+				GTK_WIDGET(filechooser), "file_entry")), utf8_filename);
+		g_free(utf8_filename);
+		g_free(filename);
+	}
+
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(
+			lookup_widget(GTK_WIDGET(filechooser), "check_hidden")), is_on);
+}
+#endif
+
+
+#if ! GEANY_USE_WIN32_DIALOG
+static void
+on_file_open_check_hidden_toggled      (GtkToggleButton *togglebutton,
+                                        gpointer         user_data)
+{
+	gboolean is_on = gtk_toggle_button_get_active(togglebutton);
+
+	gtk_file_chooser_set_show_hidden(GTK_FILE_CHOOSER(ui_widgets.open_filesel), is_on);
+}
+#endif
+
+
 /* This shows the file selection dialog to open a file. */
 void dialogs_show_open_file ()
 {
@@ -69,7 +167,7 @@ void dialogs_show_open_file ()
 
 	/* We use the same file selection widget each time, so first
 		of all we create it if it hasn't already been created. */
-	if (app->open_filesel == NULL)
+	if (ui_widgets.open_filesel == NULL)
 	{
 		GtkWidget *filetype_combo, *encoding_combo;
 		GtkWidget *viewbtn;
@@ -77,53 +175,53 @@ void dialogs_show_open_file ()
 		gint i;
 		gchar *encoding_string;
 
-		app->open_filesel = gtk_file_chooser_dialog_new(_("Open File"), GTK_WINDOW(app->window),
+		ui_widgets.open_filesel = gtk_file_chooser_dialog_new(_("Open File"), GTK_WINDOW(app->window),
 				GTK_FILE_CHOOSER_ACTION_OPEN, NULL, NULL);
+		gtk_widget_set_name(ui_widgets.open_filesel, "GeanyDialog");
 
 		viewbtn = gtk_button_new_with_mnemonic(_("_View"));
 		gtk_tooltips_set_tip(tooltips, viewbtn,
 			_("Opens the file in read-only mode. If you choose more than one file to open, all files will be opened read-only."), NULL);
 		gtk_widget_show(viewbtn);
-		gtk_widget_set_name(app->open_filesel, "GeanyDialog");
-		gtk_dialog_add_action_widget(GTK_DIALOG(app->open_filesel),
+		gtk_dialog_add_action_widget(GTK_DIALOG(ui_widgets.open_filesel),
 			viewbtn, GTK_RESPONSE_APPLY);
-		gtk_dialog_add_buttons(GTK_DIALOG(app->open_filesel),
+
+		gtk_dialog_add_buttons(GTK_DIALOG(ui_widgets.open_filesel),
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
-		// set default Open, so pressing enter can open multiple files
-		gtk_dialog_set_default_response(GTK_DIALOG(app->open_filesel),
+		gtk_dialog_set_default_response(GTK_DIALOG(ui_widgets.open_filesel),
 			GTK_RESPONSE_ACCEPT);
 
-		gtk_widget_set_size_request(app->open_filesel, -1, 460);
-		gtk_window_set_modal(GTK_WINDOW(app->open_filesel), TRUE);
-		gtk_window_set_destroy_with_parent(GTK_WINDOW(app->open_filesel), TRUE);
-		gtk_window_set_skip_taskbar_hint(GTK_WINDOW(app->open_filesel), TRUE);
-		gtk_window_set_type_hint(GTK_WINDOW(app->open_filesel), GDK_WINDOW_TYPE_HINT_DIALOG);
-		gtk_window_set_transient_for(GTK_WINDOW(app->open_filesel), GTK_WINDOW(app->window));
-		gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(app->open_filesel), TRUE);
+		gtk_widget_set_size_request(ui_widgets.open_filesel, -1, 460);
+		gtk_window_set_modal(GTK_WINDOW(ui_widgets.open_filesel), TRUE);
+		gtk_window_set_destroy_with_parent(GTK_WINDOW(ui_widgets.open_filesel), TRUE);
+		gtk_window_set_skip_taskbar_hint(GTK_WINDOW(ui_widgets.open_filesel), TRUE);
+		gtk_window_set_type_hint(GTK_WINDOW(ui_widgets.open_filesel), GDK_WINDOW_TYPE_HINT_DIALOG);
+		gtk_window_set_transient_for(GTK_WINDOW(ui_widgets.open_filesel), GTK_WINDOW(app->window));
+		gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(ui_widgets.open_filesel), TRUE);
 
 		// add checkboxes and filename entry
-		gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(app->open_filesel),
+		gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(ui_widgets.open_filesel),
 			add_file_open_extra_widget());
-		filetype_combo = lookup_widget(app->open_filesel, "filetype_combo");
+		filetype_combo = lookup_widget(ui_widgets.open_filesel, "filetype_combo");
 
 		// add FileFilters(start with "All Files")
-		gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(app->open_filesel),
+		gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(ui_widgets.open_filesel),
 					filetypes_create_file_filter(filetypes[GEANY_FILETYPES_ALL]));
 		// now create meta filter "All Source"
-		gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(app->open_filesel),
+		gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(ui_widgets.open_filesel),
 					filetypes_create_file_filter_all_source());
 		for (i = 0; i < GEANY_MAX_FILE_TYPES - 1; i++)
 		{
 			gtk_combo_box_append_text(GTK_COMBO_BOX(filetype_combo), filetypes[i]->title);
-			gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(app->open_filesel),
+			gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(ui_widgets.open_filesel),
 					filetypes_create_file_filter(filetypes[i]));
 		}
 		gtk_combo_box_append_text(GTK_COMBO_BOX(filetype_combo), _("Detect by file extension"));
 		gtk_combo_box_set_active(GTK_COMBO_BOX(filetype_combo), GEANY_MAX_FILE_TYPES - 1);
 
 		// fill encoding combo box
-		encoding_combo = lookup_widget(app->open_filesel, "encoding_combo");
+		encoding_combo = lookup_widget(ui_widgets.open_filesel, "encoding_combo");
 		for (i = 0; i < GEANY_ENCODINGS_MAX; i++)
 		{
 			encoding_string = encodings_to_string(&encodings[i]);
@@ -133,11 +231,11 @@ void dialogs_show_open_file ()
 		gtk_combo_box_append_text(GTK_COMBO_BOX(encoding_combo), _("Detect from file"));
 		gtk_combo_box_set_active(GTK_COMBO_BOX(encoding_combo), GEANY_ENCODINGS_MAX);
 
-		g_signal_connect((gpointer) app->open_filesel, "selection-changed",
+		g_signal_connect((gpointer) ui_widgets.open_filesel, "selection-changed",
 					G_CALLBACK(on_file_open_selection_changed), NULL);
-		g_signal_connect ((gpointer) app->open_filesel, "delete_event",
+		g_signal_connect ((gpointer) ui_widgets.open_filesel, "delete_event",
 					G_CALLBACK(gtk_widget_hide_on_delete), NULL);
-		g_signal_connect((gpointer) app->open_filesel, "response",
+		g_signal_connect((gpointer) ui_widgets.open_filesel, "response",
 					G_CALLBACK(on_file_open_dialog_response), NULL);
 	}
 
@@ -151,19 +249,27 @@ void dialogs_show_open_file ()
 
 		if (g_path_is_absolute(locale_filename))
 			gtk_file_chooser_set_current_folder(
-				GTK_FILE_CHOOSER(app->open_filesel), locale_filename);
+				GTK_FILE_CHOOSER(ui_widgets.open_filesel), locale_filename);
 
 		g_free(initdir);
 		g_free(locale_filename);
 	}
+	// use default startup directory(if set) if no files are open
+	/// TODO should it only be used when initally open the dialog and not on every show?
+	else if (prefs.default_open_path != NULL && *prefs.default_open_path != '\0')
+	{
+		if (g_path_is_absolute(prefs.default_open_path))
+			gtk_file_chooser_set_current_folder(
+				GTK_FILE_CHOOSER(ui_widgets.open_filesel), prefs.default_open_path);
+	}
 
-	gtk_file_chooser_unselect_all(GTK_FILE_CHOOSER(app->open_filesel));
-	gtk_widget_show(app->open_filesel);
+	gtk_file_chooser_unselect_all(GTK_FILE_CHOOSER(ui_widgets.open_filesel));
+	gtk_widget_show(ui_widgets.open_filesel);
 #endif
 }
 
 
-#ifndef USE_WIN32_DIALOG
+#if ! GEANY_USE_WIN32_DIALOG
 static GtkWidget *add_file_open_extra_widget()
 {
 	GtkWidget *vbox, *table, *file_entry, *check_hidden;
@@ -242,13 +348,13 @@ static GtkWidget *add_file_open_extra_widget()
 	g_signal_connect((gpointer) check_hidden, "toggled",
 				G_CALLBACK(on_file_open_check_hidden_toggled), NULL);
 
-	g_object_set_data_full(G_OBJECT(app->open_filesel), "file_entry",
+	g_object_set_data_full(G_OBJECT(ui_widgets.open_filesel), "file_entry",
 				gtk_widget_ref(file_entry), (GDestroyNotify)gtk_widget_unref);
-	g_object_set_data_full(G_OBJECT(app->open_filesel), "check_hidden",
+	g_object_set_data_full(G_OBJECT(ui_widgets.open_filesel), "check_hidden",
 				gtk_widget_ref(check_hidden), (GDestroyNotify)gtk_widget_unref);
-	g_object_set_data_full(G_OBJECT(app->open_filesel), "filetype_combo",
+	g_object_set_data_full(G_OBJECT(ui_widgets.open_filesel), "filetype_combo",
 				gtk_widget_ref(filetype_combo), (GDestroyNotify)gtk_widget_unref);
-	g_object_set_data_full(G_OBJECT(app->open_filesel), "encoding_combo",
+	g_object_set_data_full(G_OBJECT(ui_widgets.open_filesel), "encoding_combo",
 				gtk_widget_ref(encoding_combo), (GDestroyNotify)gtk_widget_unref);
 
 	return vbox;
@@ -256,45 +362,168 @@ static GtkWidget *add_file_open_extra_widget()
 #endif
 
 
-/* This shows the file selection dialog to save a file, returning TRUE if
- * the file was saved. */
+#if ! GEANY_USE_WIN32_DIALOG
+static void on_save_as_new_tab_toggled(GtkToggleButton *togglebutton, gpointer user_data)
+{
+	gtk_widget_set_sensitive(GTK_WIDGET(user_data),
+		! gtk_toggle_button_get_active(togglebutton));
+}
+#endif
+
+
+#if ! GEANY_USE_WIN32_DIALOG
+static void
+on_file_save_dialog_response           (GtkDialog *dialog,
+                                        gint response,
+                                        gpointer user_data)
+{
+	gboolean rename_file = FALSE;
+
+	switch (response)
+	{
+		case GTK_RESPONSE_APPLY:
+			rename_file = TRUE;
+			// fall through
+
+		case GTK_RESPONSE_ACCEPT:
+		{
+			gint idx = document_get_cur_idx();
+			gchar *new_filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(ui_widgets.save_filesel));
+			gchar *utf8_filename;
+			gboolean open_new_tab = gtk_toggle_button_get_active(
+					GTK_TOGGLE_BUTTON(lookup_widget(ui_widgets.save_filesel, "check_open_new_tab")));
+
+#ifdef G_OS_WIN32
+			utf8_filename = g_strdup(new_filename);
+#else
+			utf8_filename = utils_get_utf8_from_locale(new_filename);
+#endif
+			// check if file exists and ask whether to overwrite or not
+			if (g_file_test(new_filename, G_FILE_TEST_EXISTS))
+			{
+				if (dialogs_show_question(
+					_("The file '%s' already exists. Do you want to overwrite it?"),
+					utf8_filename) == FALSE)
+					return;
+			}
+
+			if (open_new_tab)
+			{	// "open" the saved file in a new tab
+				idx = document_clone(idx, utf8_filename);
+				g_free(utf8_filename);
+			}
+			else
+			{
+				if (doc_list[idx].file_name != NULL)
+				{
+					if (rename_file)
+					{	// delete the previous file name
+#ifdef G_OS_WIN32
+						g_unlink(doc_list[idx].file_name);
+#else
+						gchar *old_filename = utils_get_locale_from_utf8(doc_list[idx].file_name);
+
+						g_unlink(old_filename);
+						g_free(old_filename);
+#endif
+					}
+					// create a new tm_source_file object otherwise tagmanager won't work correctly
+					tm_workspace_remove_object(doc_list[idx].tm_file, TRUE);
+					doc_list[idx].tm_file = NULL;
+					g_free(doc_list[idx].file_name);
+				}
+				doc_list[idx].file_name = utf8_filename;
+			}
+			utils_replace_filename(idx);
+			document_save_file(idx, TRUE);
+
+			if (! open_new_tab)
+				build_menu_update(idx);
+
+			// finally add current file to recent files menu
+			ui_add_recent_file(doc_list[idx].file_name);
+
+			g_free(new_filename);
+		}
+	}
+	gtk_widget_hide(ui_widgets.save_filesel);
+}
+#endif
+
+
+/* Show the Save As dialog for the current notebook page.
+ * Returns: TRUE if the file was saved. */
 gboolean dialogs_show_save_as()
 {
-#ifdef G_OS_WIN32
+#if GEANY_USE_WIN32_DIALOG
 	return win32_show_file_dialog(FALSE);
 #else
 	gint idx = document_get_cur_idx(), resp;
 
-	if (app->save_filesel == NULL)
+	if (ui_widgets.save_filesel == NULL)
 	{
-		app->save_filesel = gtk_file_chooser_dialog_new(_("Save File"), GTK_WINDOW(app->window),
-					GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
-		gtk_window_set_modal(GTK_WINDOW(app->save_filesel), TRUE);
-		gtk_window_set_destroy_with_parent(GTK_WINDOW(app->save_filesel), TRUE);
-		gtk_window_set_skip_taskbar_hint(GTK_WINDOW(app->save_filesel), TRUE);
-		gtk_window_set_type_hint(GTK_WINDOW(app->save_filesel), GDK_WINDOW_TYPE_HINT_DIALOG);
-		gtk_dialog_set_default_response(GTK_DIALOG(app->save_filesel), GTK_RESPONSE_ACCEPT);
-		gtk_widget_set_name(app->save_filesel, "GeanyDialog");
+		GtkWidget *vbox, *check_open_new_tab, *rename_btn;
+		GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(app->window, "tooltips"));
 
-		g_signal_connect((gpointer) app->save_filesel, "delete_event",
+		ui_widgets.save_filesel = gtk_file_chooser_dialog_new(_("Save File"), GTK_WINDOW(app->window),
+					GTK_FILE_CHOOSER_ACTION_SAVE, NULL, NULL);
+		gtk_window_set_modal(GTK_WINDOW(ui_widgets.save_filesel), TRUE);
+		gtk_window_set_destroy_with_parent(GTK_WINDOW(ui_widgets.save_filesel), TRUE);
+		gtk_window_set_skip_taskbar_hint(GTK_WINDOW(ui_widgets.save_filesel), TRUE);
+		gtk_window_set_type_hint(GTK_WINDOW(ui_widgets.save_filesel), GDK_WINDOW_TYPE_HINT_DIALOG);
+		gtk_widget_set_name(ui_widgets.save_filesel, "GeanyDialog");
+
+		rename_btn = gtk_button_new_with_mnemonic(_("R_ename"));
+		gtk_tooltips_set_tip(tooltips, rename_btn,
+			_("Save the file and rename it."), NULL);
+		gtk_widget_show(rename_btn);
+		gtk_dialog_add_action_widget(GTK_DIALOG(ui_widgets.save_filesel),
+			rename_btn, GTK_RESPONSE_APPLY);
+
+		gtk_dialog_add_buttons(GTK_DIALOG(ui_widgets.save_filesel),
+			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
+		gtk_dialog_set_default_response(GTK_DIALOG(ui_widgets.save_filesel), GTK_RESPONSE_ACCEPT);
+
+		vbox = gtk_vbox_new(FALSE, 0);
+		check_open_new_tab = gtk_check_button_new_with_mnemonic(_("_Open file in a new tab"));
+		gtk_tooltips_set_tip(tooltips, check_open_new_tab,
+			_("Keep the current unsaved document open"
+			" and open the newly saved file in a new tab."), NULL);
+		gtk_box_pack_start(GTK_BOX(vbox), check_open_new_tab, FALSE, FALSE, 0);
+		gtk_widget_show_all(vbox);
+		gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(ui_widgets.save_filesel), vbox);
+
+		g_signal_connect(check_open_new_tab, "toggled",
+					G_CALLBACK(on_save_as_new_tab_toggled), rename_btn);
+
+		g_object_set_data_full(G_OBJECT(ui_widgets.save_filesel), "check_open_new_tab",
+					gtk_widget_ref(check_open_new_tab), (GDestroyNotify) gtk_widget_unref);
+
+		g_signal_connect((gpointer) ui_widgets.save_filesel, "delete_event",
 			G_CALLBACK(gtk_widget_hide_on_delete), NULL);
-		g_signal_connect((gpointer) app->save_filesel, "response",
+		g_signal_connect((gpointer) ui_widgets.save_filesel, "response",
 			G_CALLBACK(on_file_save_dialog_response), NULL);
 
-		gtk_window_set_transient_for(GTK_WINDOW(app->save_filesel), GTK_WINDOW(app->window));
+		gtk_window_set_transient_for(GTK_WINDOW(ui_widgets.save_filesel), GTK_WINDOW(app->window));
 	}
 
 	// If the current document has a filename we use that as the default.
 	if (doc_list[idx].file_name != NULL)
 	{
+#ifdef G_OS_WIN32
+		gchar *locale_filename = doc_list[idx].file_name;
+#else
 		gchar *locale_filename = utils_get_locale_from_utf8(doc_list[idx].file_name);
-
+#endif
 		if (g_path_is_absolute(locale_filename))
-			gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(app->save_filesel), locale_filename);
+			gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(ui_widgets.save_filesel), locale_filename);
 		else
-			gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(app->save_filesel), locale_filename);
+			gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(ui_widgets.save_filesel),
+				doc_list[idx].file_name);
+#ifndef G_OS_WIN32
 		g_free(locale_filename);
+#endif
 	}
 	else
 	{
@@ -307,13 +536,29 @@ gboolean dialogs_show_save_as()
 		else
 			fname = g_strdup(GEANY_STRING_UNTITLED);
 
-		gtk_file_chooser_unselect_all(GTK_FILE_CHOOSER(app->save_filesel));
-		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(app->save_filesel), fname);
+		gtk_file_chooser_unselect_all(GTK_FILE_CHOOSER(ui_widgets.save_filesel));
+		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(ui_widgets.save_filesel), fname);
+
+		// use default startup directory(if set) if no files are open
+		if (prefs.default_open_path != NULL && *prefs.default_open_path != '\0')
+		{
+			if (g_path_is_absolute(prefs.default_open_path))
+			{
+#ifdef G_OS_WIN32
+				gtk_file_chooser_set_current_folder(
+					GTK_FILE_CHOOSER(ui_widgets.save_filesel), prefs.default_open_path);
+#else
+				gchar *def_path = utils_get_locale_from_utf8(prefs.default_open_path);
+				gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(ui_widgets.save_filesel), def_path);
+				g_free(def_path);
+#endif
+			}
+		}
 		g_free(fname);
 	}
 
 	// Run the dialog synchronously, pausing this function call
-	resp = gtk_dialog_run(GTK_DIALOG(app->save_filesel));
+	resp = gtk_dialog_run(GTK_DIALOG(ui_widgets.save_filesel));
 	return (resp == GTK_RESPONSE_ACCEPT);
 #endif
 }
@@ -332,7 +577,7 @@ void dialogs_show_msgbox(gint type, const gchar *text, ...)
 	va_end(args);
 
 #ifdef G_OS_WIN32
-	win32_message_dialog(type, string);
+	win32_message_dialog(NULL, type, string);
 #else
 	dialog = gtk_message_dialog_new(GTK_WINDOW(app->window), GTK_DIALOG_DESTROY_WITH_PARENT,
                                   type, GTK_BUTTONS_OK, "%s", string);
@@ -349,7 +594,7 @@ gboolean dialogs_show_unsaved_file(gint idx)
 #ifndef G_OS_WIN32
 	GtkWidget *dialog, *button;
 #endif
-	gchar *msg, *short_fn = NULL;
+	gchar *msg, *msg2, *short_fn = NULL;
 	gint ret;
 
 	// display the file tab to remind the user of the document
@@ -363,15 +608,16 @@ gboolean dialogs_show_unsaved_file(gint idx)
 
 	msg = g_strdup_printf(_("The file '%s' is not saved."),
 		(short_fn != NULL) ? short_fn : GEANY_STRING_UNTITLED);
+	msg2 = _("Do you want to save it before closing?");
 	g_free(short_fn);
 
 #ifdef G_OS_WIN32
+	setptr(msg, g_strconcat(msg, "\n", msg2, NULL));
 	ret = win32_message_dialog_unsaved(msg);
 #else
 	dialog = gtk_message_dialog_new(GTK_WINDOW(app->window), GTK_DIALOG_DESTROY_WITH_PARENT,
                                   GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, "%s", msg);
-	gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
-		"%s", _("Do you want to save it before closing?"));
+	gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog), "%s", msg2);
 	gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 
 	button = ui_button_new_with_image(GTK_STOCK_CLEAR, _("_Don't save"));
@@ -409,6 +655,39 @@ gboolean dialogs_show_unsaved_file(gint idx)
 }
 
 
+#ifndef G_OS_WIN32
+static void
+on_font_apply_button_clicked           (GtkButton       *button,
+                                        gpointer         user_data)
+{
+	gchar *fontname;
+
+	fontname = gtk_font_selection_dialog_get_font_name(
+		GTK_FONT_SELECTION_DIALOG(ui_widgets.open_fontsel));
+	ui_set_editor_font(fontname);
+	g_free(fontname);
+}
+
+
+static void
+on_font_ok_button_clicked              (GtkButton       *button,
+                                        gpointer         user_data)
+{
+	// We do the same thing as apply, but we close the dialog after.
+	on_font_apply_button_clicked(button, NULL);
+	gtk_widget_hide(ui_widgets.open_fontsel);
+}
+
+
+static void
+on_font_cancel_button_clicked         (GtkButton       *button,
+                                        gpointer         user_data)
+{
+	gtk_widget_hide(ui_widgets.open_fontsel);
+}
+#endif
+
+
 /* This shows the font selection dialog to choose a font. */
 void dialogs_show_open_font()
 {
@@ -416,172 +695,33 @@ void dialogs_show_open_font()
 	win32_show_font_dialog();
 #else
 
-	if (app->open_fontsel == NULL)
+	if (ui_widgets.open_fontsel == NULL)
 	{
-		app->open_fontsel = gtk_font_selection_dialog_new(_("Choose font"));;
-		gtk_container_set_border_width(GTK_CONTAINER(app->open_fontsel), 4);
-		gtk_window_set_modal(GTK_WINDOW(app->open_fontsel), TRUE);
-		gtk_window_set_destroy_with_parent(GTK_WINDOW(app->open_fontsel), TRUE);
-		gtk_window_set_skip_taskbar_hint(GTK_WINDOW(app->open_fontsel), TRUE);
-		gtk_window_set_type_hint(GTK_WINDOW(app->open_fontsel), GDK_WINDOW_TYPE_HINT_DIALOG);
-		gtk_widget_set_name(app->open_fontsel, "GeanyDialog");
+		ui_widgets.open_fontsel = gtk_font_selection_dialog_new(_("Choose font"));;
+		gtk_container_set_border_width(GTK_CONTAINER(ui_widgets.open_fontsel), 4);
+		gtk_window_set_modal(GTK_WINDOW(ui_widgets.open_fontsel), TRUE);
+		gtk_window_set_destroy_with_parent(GTK_WINDOW(ui_widgets.open_fontsel), TRUE);
+		gtk_window_set_skip_taskbar_hint(GTK_WINDOW(ui_widgets.open_fontsel), TRUE);
+		gtk_window_set_type_hint(GTK_WINDOW(ui_widgets.open_fontsel), GDK_WINDOW_TYPE_HINT_DIALOG);
+		gtk_widget_set_name(ui_widgets.open_fontsel, "GeanyDialog");
 
-		gtk_widget_show(GTK_FONT_SELECTION_DIALOG(app->open_fontsel)->apply_button);
+		gtk_widget_show(GTK_FONT_SELECTION_DIALOG(ui_widgets.open_fontsel)->apply_button);
 
-		g_signal_connect((gpointer) app->open_fontsel,
+		g_signal_connect((gpointer) ui_widgets.open_fontsel,
 					"delete_event", G_CALLBACK(gtk_widget_hide_on_delete), NULL);
-		g_signal_connect((gpointer) GTK_FONT_SELECTION_DIALOG(app->open_fontsel)->ok_button,
+		g_signal_connect((gpointer) GTK_FONT_SELECTION_DIALOG(ui_widgets.open_fontsel)->ok_button,
 					"clicked", G_CALLBACK(on_font_ok_button_clicked), NULL);
-		g_signal_connect((gpointer) GTK_FONT_SELECTION_DIALOG(app->open_fontsel)->cancel_button,
+		g_signal_connect((gpointer) GTK_FONT_SELECTION_DIALOG(ui_widgets.open_fontsel)->cancel_button,
 					"clicked", G_CALLBACK(on_font_cancel_button_clicked), NULL);
-		g_signal_connect((gpointer) GTK_FONT_SELECTION_DIALOG(app->open_fontsel)->apply_button,
+		g_signal_connect((gpointer) GTK_FONT_SELECTION_DIALOG(ui_widgets.open_fontsel)->apply_button,
 					"clicked", G_CALLBACK(on_font_apply_button_clicked), NULL);
 
 		gtk_font_selection_dialog_set_font_name(
-			GTK_FONT_SELECTION_DIALOG(app->open_fontsel), app->editor_font);
-		gtk_window_set_transient_for(GTK_WINDOW(app->open_fontsel), GTK_WINDOW(app->window));
+			GTK_FONT_SELECTION_DIALOG(ui_widgets.open_fontsel), prefs.editor_font);
+		gtk_window_set_transient_for(GTK_WINDOW(ui_widgets.open_fontsel), GTK_WINDOW(app->window));
 	}
 	/* We make sure the dialog is visible. */
-	gtk_window_present(GTK_WINDOW(app->open_fontsel));
-#endif
-}
-
-
-void dialogs_show_word_count()
-{
-	GtkWidget *dialog, *label, *vbox, *table;
-	gint idx;
-	guint chars, lines, words;
-	gchar *text, *range;
-
-	idx = document_get_cur_idx();
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
-
-	dialog = gtk_dialog_new_with_buttons(_("Word Count"), GTK_WINDOW(app->window),
-										 GTK_DIALOG_DESTROY_WITH_PARENT,
-										 GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL, NULL);
-	vbox = ui_dialog_vbox_new(GTK_DIALOG(dialog));
-	gtk_widget_set_name(dialog, "GeanyDialog");
-
-	if (sci_can_copy(doc_list[idx].sci))
-	{
-		text = g_malloc0(sci_get_selected_text_length(doc_list[idx].sci) + 1);
-		sci_get_selected_text(doc_list[idx].sci, text);
-		range = _("selection");
-	}
-	else
-	{
-		text = g_malloc(sci_get_length(doc_list[idx].sci) + 1);
-		sci_get_text(doc_list[idx].sci, sci_get_length(doc_list[idx].sci) + 1 , text);
-		range = _("whole document");
-	}
-	utils_wordcount(text, &chars, &lines, &words);
-	g_free(text);
-
-	table = gtk_table_new(4, 2, FALSE);
-	gtk_table_set_row_spacings(GTK_TABLE(table), 5);
-	gtk_table_set_col_spacings(GTK_TABLE(table), 10);
-
-	label = gtk_label_new(_("Range:"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
-					(GtkAttachOptions) (GTK_FILL),
-					(GtkAttachOptions) (0), 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 1, 0);
-
-	label = gtk_label_new(range);
-	gtk_table_attach(GTK_TABLE(table), label, 1, 2, 0, 1,
-					(GtkAttachOptions) (GTK_FILL),
-					(GtkAttachOptions) (0), 20, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
-
-	label = gtk_label_new(_("Lines:"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
-					(GtkAttachOptions) (GTK_FILL),
-					(GtkAttachOptions) (0), 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 1, 0);
-
-	text = g_strdup_printf("%d", lines);
-	label = gtk_label_new(text);
-	gtk_table_attach(GTK_TABLE(table), label, 1, 2, 1, 2,
-					(GtkAttachOptions) (GTK_FILL),
-					(GtkAttachOptions) (0), 20, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
-	g_free(text);
-
-	label = gtk_label_new(_("Words:"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
-					(GtkAttachOptions) (GTK_FILL),
-					(GtkAttachOptions) (0), 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 1, 0);
-
-	text = g_strdup_printf("%d", words);
-	label = gtk_label_new(text);
-	gtk_table_attach(GTK_TABLE(table), label, 1, 2, 2, 3,
-					(GtkAttachOptions) (GTK_FILL),
-					(GtkAttachOptions) (0), 20, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
-	g_free(text);
-
-	label = gtk_label_new(_("Characters:"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4,
-					(GtkAttachOptions) (GTK_FILL),
-					(GtkAttachOptions) (0), 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 1, 0);
-
-	text = g_strdup_printf("%d", chars);
-	label = gtk_label_new(text);
-	gtk_table_attach(GTK_TABLE(table), label, 1, 2, 3, 4,
-					(GtkAttachOptions) (GTK_FILL),
-					(GtkAttachOptions) (0), 20, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
-	g_free(text);
-
-	gtk_container_add(GTK_CONTAINER(vbox), table);
-
-	g_signal_connect(dialog, "response", G_CALLBACK(gtk_widget_destroy), dialog);
-	g_signal_connect(dialog, "delete-event", G_CALLBACK(gtk_widget_destroy), dialog);
-
-	gtk_widget_show_all(dialog);
-}
-
-
-/* This shows the color selection dialog to choose a color. */
-void dialogs_show_color(gchar *colour)
-{
-#ifdef G_OS_WIN32
-	win32_show_color_dialog(colour);
-#else
-
-	if (app->open_colorsel == NULL)
-	{
-		app->open_colorsel = gtk_color_selection_dialog_new(_("Color Chooser"));
-		gtk_widget_set_name(app->open_colorsel, "GeanyDialog");
-		gtk_window_set_transient_for(GTK_WINDOW(app->open_colorsel), GTK_WINDOW(app->window));
-
-		g_signal_connect(GTK_COLOR_SELECTION_DIALOG(app->open_colorsel)->cancel_button, "clicked",
-						G_CALLBACK(on_color_cancel_button_clicked), NULL);
-		g_signal_connect(GTK_COLOR_SELECTION_DIALOG(app->open_colorsel)->ok_button, "clicked",
-						G_CALLBACK(on_color_ok_button_clicked), NULL);
-		g_signal_connect(app->open_colorsel, "delete_event",
-						G_CALLBACK(gtk_widget_hide_on_delete), NULL);
-	}
-	// if colour is non-NULL set it in the dialog as preselected colour
-	if (colour != NULL && (colour[0] == '0' || colour[0] == '#'))
-	{
-		GdkColor gc;
-
-		if (colour[0] == '0' && colour[1] == 'x')
-		{	// we have a string of the format "0x00ff00" and we need it to "#00ff00"
-			colour[1] = '#';
-			colour++;
-		}
-		gdk_color_parse(colour, &gc);
-		gtk_color_selection_set_current_color(GTK_COLOR_SELECTION(
-							GTK_COLOR_SELECTION_DIALOG(app->open_colorsel)->colorsel), &gc);
-	}
-
-	// We make sure the dialog is visible.
-	gtk_window_present(GTK_WINDOW(app->open_colorsel));
+	gtk_window_present(GTK_WINDOW(ui_widgets.open_fontsel));
 #endif
 }
 
@@ -623,7 +763,7 @@ void dialogs_show_goto_line()
 {
 	GtkWidget *dialog, *label, *entry, *vbox;
 
-	dialog = gtk_dialog_new_with_buttons(_("Go to line"), GTK_WINDOW(app->window),
+	dialog = gtk_dialog_new_with_buttons(_("Go to Line"), GTK_WINDOW(app->window),
 										GTK_DIALOG_DESTROY_WITH_PARENT,
 										GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 										GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
@@ -1015,21 +1155,23 @@ void dialogs_show_file_properties(gint idx)
 }
 
 
-static gboolean
-show_question(const gchar *yes_btn, const gchar *no_btn, const gchar *question_text,
-	const gchar *extra_text)
+static gboolean show_question(GtkWidget *parent, const gchar *yes_btn, const gchar *no_btn,
+	const gchar *question_text,	const gchar *extra_text)
 {
 	gboolean ret = FALSE;
 #ifdef G_OS_WIN32
 	gchar *string = (extra_text == NULL) ? g_strdup(question_text) :
 		g_strconcat(question_text, "\n\n", extra_text, NULL);
 
-	ret = win32_message_dialog(GTK_MESSAGE_QUESTION, string);
+	ret = win32_message_dialog(parent, GTK_MESSAGE_QUESTION, string);
 	g_free(string);
 #else
 	GtkWidget *dialog;
 
-	dialog = gtk_message_dialog_new(GTK_WINDOW(app->window),
+	if (parent == NULL)
+		parent = app->window;
+
+	dialog = gtk_message_dialog_new(GTK_WINDOW(parent),
 		GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION,
 		GTK_BUTTONS_NONE, "%s", question_text);
 	gtk_widget_set_name(dialog, "GeanyDialog");
@@ -1060,14 +1202,15 @@ gboolean dialogs_show_question(const gchar *text, ...)
 	va_start(args, text);
 	g_vsnprintf(string, 511, text, args);
 	va_end(args);
-	ret = show_question(GTK_STOCK_YES, GTK_STOCK_NO, string, NULL);
+	ret = show_question(app->window, GTK_STOCK_YES, GTK_STOCK_NO, string, NULL);
 	g_free(string);
 	return ret;
 }
 
 
-/* extra_text can be NULL; otherwise it is displayed below main_text. */
-gboolean dialogs_show_question_full(const gchar *yes_btn, const gchar *no_btn,
+/* extra_text can be NULL; otherwise it is displayed below main_text.
+ * if parent is NULL, app->window will be used */
+gboolean dialogs_show_question_full(GtkWidget *parent, const gchar *yes_btn, const gchar *no_btn,
 	const gchar *extra_text, const gchar *main_text, ...)
 {
 	gboolean ret = FALSE;
@@ -1077,7 +1220,7 @@ gboolean dialogs_show_question_full(const gchar *yes_btn, const gchar *no_btn,
 	va_start(args, main_text);
 	g_vsnprintf(string, 511, main_text, args);
 	va_end(args);
-	ret = show_question(yes_btn, no_btn, string, extra_text);
+	ret = show_question(parent, yes_btn, no_btn, string, extra_text);
 	g_free(string);
 	return ret;
 }
