@@ -18,9 +18,12 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: encodings.c 1270 2007-02-12 18:42:15Z eht16 $
+ *  $Id: encodings.c 1376 2007-03-09 13:52:26Z eht16 $
  */
 
+/*
+ * Encoding conversion and Byte Order Mark (BOM) handling.
+ */
 
 /*
  * Modified by the gedit Team, 2002. See the gedit AUTHORS file for a
@@ -139,12 +142,12 @@ const GeanyEncoding *encodings_get_from_charset(const gchar *charset)
 }
 
 
-const GeanyEncoding *encodings_get_from_index(gint index)
+const GeanyEncoding *encodings_get_from_index(gint idx)
 {
-	g_return_val_if_fail(index >= 0, NULL);
-	g_return_val_if_fail(index < GEANY_ENCODINGS_MAX, NULL);
+	g_return_val_if_fail(idx >= 0, NULL);
+	g_return_val_if_fail(idx < GEANY_ENCODINGS_MAX, NULL);
 
-	return &encodings[index];
+	return &encodings[idx];
 }
 
 
@@ -446,3 +449,4 @@ gboolean encodings_is_unicode_charset(const gchar *string)
 	return FALSE;
 }
 
+
