@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: filetypes.c 1530 2007-05-14 12:55:55Z ntrel $
+ * $Id: filetypes.c 1794 2007-08-14 15:36:12Z ntrel $
  */
 
 /*
@@ -38,6 +38,9 @@
 #include "sciwrappers.h"
 
 
+filetype *filetypes[GEANY_MAX_FILE_TYPES];
+
+
 /* This is the order of unique ids used in the config file.
  * The order must not be changed but can be appended to. */
 enum
@@ -71,7 +74,11 @@ enum
 	FILETYPE_UID_VHDL,		// 26
 	FILETYPE_UID_JS,		// 27
 	FILETYPE_UID_LUA,		// 28
-	FILETYPE_UID_HASKELL	// 29
+	FILETYPE_UID_HASKELL,	// 29
+	FILETYPE_UID_CS,		// 30
+	FILETYPE_UID_BASIC,		// 31
+	FILETYPE_UID_HAXE,		// 32
+	FILETYPE_UID_REST		// 33
 };
 
 
@@ -80,46 +87,6 @@ static GtkWidget *radio_items[GEANY_MAX_FILE_TYPES];
 static void filetypes_create_menu_item(GtkWidget *menu, const gchar *label, filetype *ftype);
 
 
-// If uid is valid, return corresponding filetype, otherwise NULL.
-filetype *filetypes_get_from_uid(gint uid)
-{
-	switch (uid)
-	{
-		case FILETYPE_UID_C:		return filetypes[GEANY_FILETYPES_C];
-		case FILETYPE_UID_CPP:		return filetypes[GEANY_FILETYPES_CPP];
-		case FILETYPE_UID_JAVA:		return filetypes[GEANY_FILETYPES_JAVA];
-		case FILETYPE_UID_PERL:		return filetypes[GEANY_FILETYPES_PERL];
-		case FILETYPE_UID_PHP:		return filetypes[GEANY_FILETYPES_PHP];
-		case FILETYPE_UID_XML:		return filetypes[GEANY_FILETYPES_XML];
-		case FILETYPE_UID_DOCBOOK:	return filetypes[GEANY_FILETYPES_DOCBOOK];
-		case FILETYPE_UID_PYTHON:	return filetypes[GEANY_FILETYPES_PYTHON];
-		case FILETYPE_UID_LATEX:	return filetypes[GEANY_FILETYPES_LATEX];
-		case FILETYPE_UID_PASCAL:	return filetypes[GEANY_FILETYPES_PASCAL];
-		case FILETYPE_UID_SH:		return filetypes[GEANY_FILETYPES_SH];
-		case FILETYPE_UID_MAKE:		return filetypes[GEANY_FILETYPES_MAKE];
-		case FILETYPE_UID_CSS:		return filetypes[GEANY_FILETYPES_CSS];
-		case FILETYPE_UID_CONF:		return filetypes[GEANY_FILETYPES_CONF];
-		case FILETYPE_UID_ASM:		return filetypes[GEANY_FILETYPES_ASM];
-		case FILETYPE_UID_SQL:		return filetypes[GEANY_FILETYPES_SQL];
-		case FILETYPE_UID_CAML:		return filetypes[GEANY_FILETYPES_CAML];
-		case FILETYPE_UID_OMS:		return filetypes[GEANY_FILETYPES_OMS];
-		case FILETYPE_UID_RUBY:		return filetypes[GEANY_FILETYPES_RUBY];
-		case FILETYPE_UID_TCL:		return filetypes[GEANY_FILETYPES_TCL];
-		case FILETYPE_UID_ALL:		return filetypes[GEANY_FILETYPES_ALL];
-		case FILETYPE_UID_D:		return filetypes[GEANY_FILETYPES_D];
-		case FILETYPE_UID_FORTRAN:	return filetypes[GEANY_FILETYPES_FORTRAN];
-		case FILETYPE_UID_DIFF:		return filetypes[GEANY_FILETYPES_DIFF];
-		case FILETYPE_UID_FERITE:	return filetypes[GEANY_FILETYPES_FERITE];
-		case FILETYPE_UID_HTML:		return filetypes[GEANY_FILETYPES_HTML];
-		case FILETYPE_UID_VHDL:		return filetypes[GEANY_FILETYPES_VHDL];
-		case FILETYPE_UID_JS:		return filetypes[GEANY_FILETYPES_JS];
-		case FILETYPE_UID_LUA:		return filetypes[GEANY_FILETYPES_LUA];
-		case FILETYPE_UID_HASKELL:	return filetypes[GEANY_FILETYPES_HASKELL];
-		default: 					return NULL;
-	}
-}
-
-
 /* Create the filetype array and fill it with the known filetypes. */
 void filetypes_init_types()
 {
@@ -137,7 +104,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_C]->uid = FILETYPE_UID_C;
 	filetypes[GEANY_FILETYPES_C]->lang = 0;
 	filetypes[GEANY_FILETYPES_C]->name = g_strdup("C");
-	filetypes[GEANY_FILETYPES_C]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_C]->title = g_strdup(_("C source file"));
 	filetypes[GEANY_FILETYPES_C]->extension = g_strdup("c");
 	filetypes[GEANY_FILETYPES_C]->pattern = utils_strv_new("*.c", "*.h", NULL);
@@ -150,7 +116,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_CPP]->uid = FILETYPE_UID_CPP;
 	filetypes[GEANY_FILETYPES_CPP]->lang = 1;
 	filetypes[GEANY_FILETYPES_CPP]->name = g_strdup("C++");
-	filetypes[GEANY_FILETYPES_CPP]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_CPP]->title = g_strdup(_("C++ source file"));
 	filetypes[GEANY_FILETYPES_CPP]->extension = g_strdup("cpp");
 	filetypes[GEANY_FILETYPES_CPP]->pattern = utils_strv_new("*.cpp", "*.cxx", "*.c++", "*.cc",
@@ -159,12 +124,23 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_CPP]->comment_open = g_strdup("//");
 	filetypes[GEANY_FILETYPES_CPP]->comment_close = NULL;
 
+#define CS
+	filetypes[GEANY_FILETYPES_CS]->id = GEANY_FILETYPES_CS;
+	filetypes[GEANY_FILETYPES_CS]->uid = FILETYPE_UID_CS;
+	filetypes[GEANY_FILETYPES_CS]->lang = 25;
+	filetypes[GEANY_FILETYPES_CS]->name = g_strdup("C#");
+	filetypes[GEANY_FILETYPES_CS]->title = g_strdup(_("C# source file"));
+	filetypes[GEANY_FILETYPES_CS]->extension = g_strdup("cs");
+	filetypes[GEANY_FILETYPES_CS]->pattern = utils_strv_new("*.cs", "*.lala", NULL);
+	filetypes[GEANY_FILETYPES_CS]->style_func_ptr = styleset_cs;
+	filetypes[GEANY_FILETYPES_CS]->comment_open = g_strdup("//");
+	filetypes[GEANY_FILETYPES_CS]->comment_close = NULL;
+
 #define D
 	filetypes[GEANY_FILETYPES_D]->id = GEANY_FILETYPES_D;
 	filetypes[GEANY_FILETYPES_D]->uid = FILETYPE_UID_D;
 	filetypes[GEANY_FILETYPES_D]->lang = 17;
 	filetypes[GEANY_FILETYPES_D]->name = g_strdup("D");
-	filetypes[GEANY_FILETYPES_D]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_D]->title = g_strdup(_("D source file"));
 	filetypes[GEANY_FILETYPES_D]->extension = g_strdup("d");
 	filetypes[GEANY_FILETYPES_D]->pattern = utils_strv_new("*.d", "*.di", NULL);
@@ -177,7 +153,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_JAVA]->name = g_strdup("Java");
 	filetypes[GEANY_FILETYPES_JAVA]->uid = FILETYPE_UID_JAVA;
 	filetypes[GEANY_FILETYPES_JAVA]->lang = 2;
-	filetypes[GEANY_FILETYPES_JAVA]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_JAVA]->title = g_strdup(_("Java source file"));
 	filetypes[GEANY_FILETYPES_JAVA]->extension = g_strdup("java");
 	filetypes[GEANY_FILETYPES_JAVA]->pattern = utils_strv_new("*.java", "*.jsp", NULL);
@@ -190,7 +165,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_PASCAL]->uid = FILETYPE_UID_PASCAL;
 	filetypes[GEANY_FILETYPES_PASCAL]->lang = 4;
 	filetypes[GEANY_FILETYPES_PASCAL]->name = g_strdup("Pascal");
-	filetypes[GEANY_FILETYPES_PASCAL]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_PASCAL]->title = g_strdup(_("Pascal source file"));
 	filetypes[GEANY_FILETYPES_PASCAL]->extension = g_strdup("pas");
 	filetypes[GEANY_FILETYPES_PASCAL]->pattern = utils_strv_new("*.pas", "*.pp", "*.inc", "*.dpr",
@@ -204,7 +178,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_ASM]->uid = FILETYPE_UID_ASM;
 	filetypes[GEANY_FILETYPES_ASM]->lang = 9;
 	filetypes[GEANY_FILETYPES_ASM]->name = g_strdup("ASM");
-	filetypes[GEANY_FILETYPES_ASM]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_ASM]->title = g_strdup(_("Assembler source file"));
 	filetypes[GEANY_FILETYPES_ASM]->extension = g_strdup("asm");
 	filetypes[GEANY_FILETYPES_ASM]->pattern = utils_strv_new("*.asm", NULL);
@@ -212,12 +185,23 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_ASM]->comment_open = g_strdup(";");
 	filetypes[GEANY_FILETYPES_ASM]->comment_close = NULL;
 
+#define BASIC
+	filetypes[GEANY_FILETYPES_BASIC]->id = GEANY_FILETYPES_BASIC;
+	filetypes[GEANY_FILETYPES_BASIC]->uid = FILETYPE_UID_BASIC;
+	filetypes[GEANY_FILETYPES_BASIC]->lang = 26;
+	filetypes[GEANY_FILETYPES_BASIC]->name = g_strdup("FreeBasic");
+	filetypes[GEANY_FILETYPES_BASIC]->title = g_strdup(_("FreeBasic source file"));
+	filetypes[GEANY_FILETYPES_BASIC]->extension = g_strdup("bas");
+	filetypes[GEANY_FILETYPES_BASIC]->pattern = utils_strv_new("*.bas", "*.bi", NULL);
+	filetypes[GEANY_FILETYPES_BASIC]->style_func_ptr = styleset_basic;
+	filetypes[GEANY_FILETYPES_BASIC]->comment_open = g_strdup("'");
+	filetypes[GEANY_FILETYPES_BASIC]->comment_close = NULL;
+
 #define FORTRAN
 	filetypes[GEANY_FILETYPES_FORTRAN]->id = GEANY_FILETYPES_FORTRAN;
 	filetypes[GEANY_FILETYPES_FORTRAN]->uid = FILETYPE_UID_FORTRAN;
 	filetypes[GEANY_FILETYPES_FORTRAN]->lang = 18;
 	filetypes[GEANY_FILETYPES_FORTRAN]->name = g_strdup("Fortran");
-	filetypes[GEANY_FILETYPES_FORTRAN]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_FORTRAN]->title = g_strdup(_("Fortran source file (F77)"));
 	filetypes[GEANY_FILETYPES_FORTRAN]->extension = g_strdup("f");
 	filetypes[GEANY_FILETYPES_FORTRAN]->pattern = utils_strv_new("*.f", "*.for", "*.ftn", "*.f77",
@@ -231,7 +215,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_CAML]->uid = FILETYPE_UID_CAML;
 	filetypes[GEANY_FILETYPES_CAML]->lang = -2;
 	filetypes[GEANY_FILETYPES_CAML]->name = g_strdup("CAML");
-	filetypes[GEANY_FILETYPES_CAML]->has_tags = FALSE;
 	filetypes[GEANY_FILETYPES_CAML]->title = g_strdup(_("(O)Caml source file"));
 	filetypes[GEANY_FILETYPES_CAML]->extension = g_strdup("ml");
 	filetypes[GEANY_FILETYPES_CAML]->pattern = utils_strv_new("*.ml", "*.mli", NULL);
@@ -244,7 +227,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_PERL]->uid = FILETYPE_UID_PERL;
 	filetypes[GEANY_FILETYPES_PERL]->lang = 5;
 	filetypes[GEANY_FILETYPES_PERL]->name = g_strdup("Perl");
-	filetypes[GEANY_FILETYPES_PERL]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_PERL]->title = g_strdup(_("Perl source file"));
 	filetypes[GEANY_FILETYPES_PERL]->extension = g_strdup("pl");
 	filetypes[GEANY_FILETYPES_PERL]->pattern = utils_strv_new("*.pl", "*.perl", "*.pm", "*.agi",
@@ -258,7 +240,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_PHP]->uid = FILETYPE_UID_PHP;
 	filetypes[GEANY_FILETYPES_PHP]->lang = 6;
 	filetypes[GEANY_FILETYPES_PHP]->name = g_strdup("PHP");
-	filetypes[GEANY_FILETYPES_PHP]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_PHP]->title = g_strdup(_("PHP source file"));
 	filetypes[GEANY_FILETYPES_PHP]->extension = g_strdup("php");
 	filetypes[GEANY_FILETYPES_PHP]->pattern = utils_strv_new("*.php", "*.php3", "*.php4", "*.php5",
@@ -272,7 +253,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_JS]->uid = FILETYPE_UID_JS;
 	filetypes[GEANY_FILETYPES_JS]->lang = 23;
 	filetypes[GEANY_FILETYPES_JS]->name = g_strdup("Javascript");
-	filetypes[GEANY_FILETYPES_JS]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_JS]->title = g_strdup(_("Javascript source file"));
 	filetypes[GEANY_FILETYPES_JS]->extension = g_strdup("js");
 	filetypes[GEANY_FILETYPES_JS]->pattern = utils_strv_new("*.js", NULL);
@@ -285,7 +265,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_PYTHON]->uid = FILETYPE_UID_PYTHON;
 	filetypes[GEANY_FILETYPES_PYTHON]->lang = 7;
 	filetypes[GEANY_FILETYPES_PYTHON]->name = g_strdup("Python");
-	filetypes[GEANY_FILETYPES_PYTHON]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_PYTHON]->title = g_strdup(_("Python source file"));
 	filetypes[GEANY_FILETYPES_PYTHON]->extension = g_strdup("py");
 	filetypes[GEANY_FILETYPES_PYTHON]->pattern = utils_strv_new("*.py", "*.pyw", NULL);
@@ -298,10 +277,9 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_RUBY]->uid = FILETYPE_UID_RUBY;
 	filetypes[GEANY_FILETYPES_RUBY]->lang = 14;
 	filetypes[GEANY_FILETYPES_RUBY]->name = g_strdup("Ruby");
-	filetypes[GEANY_FILETYPES_RUBY]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_RUBY]->title = g_strdup(_("Ruby source file"));
 	filetypes[GEANY_FILETYPES_RUBY]->extension = g_strdup("rb");
-	filetypes[GEANY_FILETYPES_RUBY]->pattern = utils_strv_new("*.rb", "*.rhtml", NULL);
+	filetypes[GEANY_FILETYPES_RUBY]->pattern = utils_strv_new("*.rb", "*.rhtml", "*.ruby", NULL);
 	filetypes[GEANY_FILETYPES_RUBY]->style_func_ptr = styleset_ruby;
 	filetypes[GEANY_FILETYPES_RUBY]->comment_open = g_strdup("#");
 	filetypes[GEANY_FILETYPES_RUBY]->comment_close = NULL;
@@ -311,7 +289,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_TCL]->uid = FILETYPE_UID_TCL;
 	filetypes[GEANY_FILETYPES_TCL]->lang = 15;
 	filetypes[GEANY_FILETYPES_TCL]->name = g_strdup("Tcl");
-	filetypes[GEANY_FILETYPES_TCL]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_TCL]->title = g_strdup(_("Tcl source file"));
 	filetypes[GEANY_FILETYPES_TCL]->extension = g_strdup("tcl");
 	filetypes[GEANY_FILETYPES_TCL]->pattern = utils_strv_new("*.tcl", "*.tk", "*.wish", NULL);
@@ -324,7 +301,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_LUA]->uid = FILETYPE_UID_LUA;
 	filetypes[GEANY_FILETYPES_LUA]->lang = 22;
 	filetypes[GEANY_FILETYPES_LUA]->name = g_strdup("Lua");
-	filetypes[GEANY_FILETYPES_LUA]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_LUA]->title = g_strdup(_("Lua source file"));
 	filetypes[GEANY_FILETYPES_LUA]->extension = g_strdup("lua");
 	filetypes[GEANY_FILETYPES_LUA]->pattern = utils_strv_new("*.lua", NULL);
@@ -337,7 +313,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_FERITE]->uid = FILETYPE_UID_FERITE;
 	filetypes[GEANY_FILETYPES_FERITE]->lang = 19;
 	filetypes[GEANY_FILETYPES_FERITE]->name = g_strdup("Ferite");
-	filetypes[GEANY_FILETYPES_FERITE]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_FERITE]->title = g_strdup(_("Ferite source file"));
 	filetypes[GEANY_FILETYPES_FERITE]->extension = g_strdup("fe");
 	filetypes[GEANY_FILETYPES_FERITE]->pattern = utils_strv_new("*.fe", NULL);
@@ -348,9 +323,8 @@ void filetypes_init_types()
 #define HASKELL
 	filetypes[GEANY_FILETYPES_HASKELL]->id = GEANY_FILETYPES_HASKELL;
 	filetypes[GEANY_FILETYPES_HASKELL]->uid = FILETYPE_UID_HASKELL;
-	filetypes[GEANY_FILETYPES_HASKELL]->lang = -2;
+	filetypes[GEANY_FILETYPES_HASKELL]->lang = 24;
 	filetypes[GEANY_FILETYPES_HASKELL]->name = g_strdup("Haskell");
-	filetypes[GEANY_FILETYPES_HASKELL]->has_tags = FALSE;
 	filetypes[GEANY_FILETYPES_HASKELL]->title = g_strdup(_("Haskell source file"));
 	filetypes[GEANY_FILETYPES_HASKELL]->extension = g_strdup("hs");
 	filetypes[GEANY_FILETYPES_HASKELL]->pattern = utils_strv_new("*.hs", "*.lhs", NULL);
@@ -363,11 +337,10 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_SH]->uid = FILETYPE_UID_SH;
 	filetypes[GEANY_FILETYPES_SH]->lang = 16;
 	filetypes[GEANY_FILETYPES_SH]->name = g_strdup("Sh");
-	filetypes[GEANY_FILETYPES_SH]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_SH]->title = g_strdup(_("Shell script file"));
 	filetypes[GEANY_FILETYPES_SH]->extension = g_strdup("sh");
 	filetypes[GEANY_FILETYPES_SH]->pattern = utils_strv_new("*.sh", "configure", "configure.in",
-		"configure.in.in", "configure.ac", "*.ksh", "*.zsh", NULL);
+		"configure.in.in", "configure.ac", "*.ksh", "*.zsh", "*.ash", "*.bash", NULL);
 	filetypes[GEANY_FILETYPES_SH]->style_func_ptr = styleset_sh;
 	filetypes[GEANY_FILETYPES_SH]->comment_open = g_strdup("#");
 	filetypes[GEANY_FILETYPES_SH]->comment_close = NULL;
@@ -377,7 +350,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_MAKE]->uid = FILETYPE_UID_MAKE;
 	filetypes[GEANY_FILETYPES_MAKE]->lang = 3;
 	filetypes[GEANY_FILETYPES_MAKE]->name = g_strdup("Make");
-	filetypes[GEANY_FILETYPES_MAKE]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_MAKE]->title = g_strdup(_("Makefile"));
 	filetypes[GEANY_FILETYPES_MAKE]->extension = g_strdup("mak");
 	filetypes[GEANY_FILETYPES_MAKE]->pattern = utils_strv_new(
@@ -391,8 +363,7 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_XML]->uid = FILETYPE_UID_XML;
 	filetypes[GEANY_FILETYPES_XML]->lang = -2;
 	filetypes[GEANY_FILETYPES_XML]->name = g_strdup("XML");
-	filetypes[GEANY_FILETYPES_XML]->has_tags = FALSE;
-	filetypes[GEANY_FILETYPES_XML]->title = g_strdup(_("XML source file"));
+	filetypes[GEANY_FILETYPES_XML]->title = g_strdup(_("XML document"));
 	filetypes[GEANY_FILETYPES_XML]->extension = g_strdup("xml");
 	filetypes[GEANY_FILETYPES_XML]->pattern = utils_strv_new(
 		"*.xml", "*.sgml", "*.xsl", "*.xslt", "*.xsd", NULL);
@@ -405,7 +376,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_DOCBOOK]->uid = FILETYPE_UID_DOCBOOK;
 	filetypes[GEANY_FILETYPES_DOCBOOK]->lang = 12;
 	filetypes[GEANY_FILETYPES_DOCBOOK]->name = g_strdup("Docbook");
-	filetypes[GEANY_FILETYPES_DOCBOOK]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_DOCBOOK]->title = g_strdup(_("Docbook source file"));
 	filetypes[GEANY_FILETYPES_DOCBOOK]->extension = g_strdup("docbook");
 	filetypes[GEANY_FILETYPES_DOCBOOK]->pattern = utils_strv_new("*.docbook", NULL);
@@ -418,7 +388,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_HTML]->uid = FILETYPE_UID_HTML;
 	filetypes[GEANY_FILETYPES_HTML]->lang = -2;
 	filetypes[GEANY_FILETYPES_HTML]->name = g_strdup("HTML");
-	filetypes[GEANY_FILETYPES_HTML]->has_tags = FALSE;
 	filetypes[GEANY_FILETYPES_HTML]->title = g_strdup(_("HTML source file"));
 	filetypes[GEANY_FILETYPES_HTML]->extension = g_strdup("html");
 	filetypes[GEANY_FILETYPES_HTML]->pattern = utils_strv_new(
@@ -432,7 +401,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_CSS]->uid = FILETYPE_UID_CSS;
 	filetypes[GEANY_FILETYPES_CSS]->lang = 13;
 	filetypes[GEANY_FILETYPES_CSS]->name = g_strdup("CSS");
-	filetypes[GEANY_FILETYPES_CSS]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_CSS]->title = g_strdup(_("Cascading StyleSheet"));
 	filetypes[GEANY_FILETYPES_CSS]->extension = g_strdup("css");
 	filetypes[GEANY_FILETYPES_CSS]->pattern = utils_strv_new("*.css", NULL);
@@ -445,7 +413,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_SQL]->uid = FILETYPE_UID_SQL;
 	filetypes[GEANY_FILETYPES_SQL]->lang = 11;
 	filetypes[GEANY_FILETYPES_SQL]->name = g_strdup("SQL");
-	filetypes[GEANY_FILETYPES_SQL]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_SQL]->title = g_strdup(_("SQL Dump file"));
 	filetypes[GEANY_FILETYPES_SQL]->extension = g_strdup("sql");
 	filetypes[GEANY_FILETYPES_SQL]->pattern = utils_strv_new("*.sql", NULL);
@@ -458,7 +425,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_LATEX]->uid = FILETYPE_UID_LATEX;
 	filetypes[GEANY_FILETYPES_LATEX]->lang = 8;
 	filetypes[GEANY_FILETYPES_LATEX]->name = g_strdup("LaTeX");
-	filetypes[GEANY_FILETYPES_LATEX]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_LATEX]->title = g_strdup(_("LaTeX source file"));
 	filetypes[GEANY_FILETYPES_LATEX]->extension = g_strdup("tex");
 	filetypes[GEANY_FILETYPES_LATEX]->pattern = utils_strv_new("*.tex", "*.sty", "*.idx", NULL);
@@ -471,7 +437,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_OMS]->uid = FILETYPE_UID_OMS;
 	filetypes[GEANY_FILETYPES_OMS]->lang = -2;
 	filetypes[GEANY_FILETYPES_OMS]->name = g_strdup("O-Matrix");
-	filetypes[GEANY_FILETYPES_OMS]->has_tags = FALSE;
 	filetypes[GEANY_FILETYPES_OMS]->title = g_strdup(_("O-Matrix source file"));
 	filetypes[GEANY_FILETYPES_OMS]->extension = g_strdup("oms");
 	filetypes[GEANY_FILETYPES_OMS]->pattern = utils_strv_new("*.oms", NULL);
@@ -484,7 +449,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_VHDL]->uid = FILETYPE_UID_VHDL;
 	filetypes[GEANY_FILETYPES_VHDL]->lang = 21;
 	filetypes[GEANY_FILETYPES_VHDL]->name = g_strdup("VHDL");
-	filetypes[GEANY_FILETYPES_VHDL]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_VHDL]->title = g_strdup(_("VHDL source file"));
 	filetypes[GEANY_FILETYPES_VHDL]->extension = g_strdup("vhd");
 	filetypes[GEANY_FILETYPES_VHDL]->pattern = utils_strv_new("*.vhd", "*.vhdl", NULL);
@@ -497,7 +461,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_DIFF]->uid = FILETYPE_UID_DIFF;
 	filetypes[GEANY_FILETYPES_DIFF]->lang = 20;
 	filetypes[GEANY_FILETYPES_DIFF]->name = g_strdup("Diff");
-	filetypes[GEANY_FILETYPES_DIFF]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_DIFF]->title = g_strdup(_("Diff file"));
 	filetypes[GEANY_FILETYPES_DIFF]->extension = g_strdup("diff");
 	filetypes[GEANY_FILETYPES_DIFF]->pattern = utils_strv_new("*.diff", "*.patch", "*.rej", NULL);
@@ -510,7 +473,6 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_CONF]->uid = FILETYPE_UID_CONF;
 	filetypes[GEANY_FILETYPES_CONF]->lang = 10;
 	filetypes[GEANY_FILETYPES_CONF]->name = g_strdup("Conf");
-	filetypes[GEANY_FILETYPES_CONF]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_CONF]->title = g_strdup(_("Config file"));
 	filetypes[GEANY_FILETYPES_CONF]->extension = g_strdup("conf");
 	filetypes[GEANY_FILETYPES_CONF]->pattern = utils_strv_new("*.conf", "*.ini", "config", "*rc",
@@ -519,12 +481,35 @@ void filetypes_init_types()
 	filetypes[GEANY_FILETYPES_CONF]->comment_open = g_strdup("#");
 	filetypes[GEANY_FILETYPES_CONF]->comment_close = NULL;
 
+#define HAXE
+	filetypes[GEANY_FILETYPES_HAXE]->id = GEANY_FILETYPES_HAXE;
+	filetypes[GEANY_FILETYPES_HAXE]->uid = FILETYPE_UID_HAXE;
+	filetypes[GEANY_FILETYPES_HAXE]->lang = 27;
+	filetypes[GEANY_FILETYPES_HAXE]->name = g_strdup("Haxe");
+	filetypes[GEANY_FILETYPES_HAXE]->title = g_strdup(_("Haxe source file"));
+	filetypes[GEANY_FILETYPES_HAXE]->extension = g_strdup("hx");
+	filetypes[GEANY_FILETYPES_HAXE]->pattern = utils_strv_new("*.hx", NULL);
+	filetypes[GEANY_FILETYPES_HAXE]->style_func_ptr = styleset_haxe;
+	filetypes[GEANY_FILETYPES_HAXE]->comment_open = g_strdup("//");
+	filetypes[GEANY_FILETYPES_HAXE]->comment_close = NULL;
+
+#define REST
+	filetypes[GEANY_FILETYPES_REST]->id = GEANY_FILETYPES_REST;
+	filetypes[GEANY_FILETYPES_REST]->uid = FILETYPE_UID_REST;
+	filetypes[GEANY_FILETYPES_REST]->lang = 28;
+	filetypes[GEANY_FILETYPES_REST]->name = g_strdup("reStructuredText");
+	filetypes[GEANY_FILETYPES_REST]->title = g_strdup(_("reStructuredText file"));
+	filetypes[GEANY_FILETYPES_REST]->extension = g_strdup("rest");
+	filetypes[GEANY_FILETYPES_REST]->pattern = utils_strv_new("*.rest", "*.reST", NULL);
+	filetypes[GEANY_FILETYPES_REST]->style_func_ptr = styleset_none;
+	filetypes[GEANY_FILETYPES_REST]->comment_open = NULL;
+	filetypes[GEANY_FILETYPES_REST]->comment_close = NULL;
+
 #define ALL
 	filetypes[GEANY_FILETYPES_ALL]->id = GEANY_FILETYPES_ALL;
 	filetypes[GEANY_FILETYPES_ALL]->name = g_strdup("None");
 	filetypes[GEANY_FILETYPES_ALL]->uid = FILETYPE_UID_ALL;
 	filetypes[GEANY_FILETYPES_ALL]->lang = -2;
-	filetypes[GEANY_FILETYPES_ALL]->has_tags = FALSE;
 	filetypes[GEANY_FILETYPES_ALL]->title = g_strdup(_("All files"));
 	filetypes[GEANY_FILETYPES_ALL]->extension = g_strdup("*");
 	filetypes[GEANY_FILETYPES_ALL]->pattern = utils_strv_new("*", NULL);
@@ -534,47 +519,95 @@ void filetypes_init_types()
 }
 
 
+#define create_sub_menu(menu, item, title) \
+	(menu) = gtk_menu_new(); \
+	(item) = gtk_menu_item_new_with_mnemonic((title)); \
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM((item)), (menu)); \
+	gtk_container_add(GTK_CONTAINER(filetype_menu), (item)); \
+	gtk_widget_show((item));
+
+
 /* Calls filetypes_init_types() and creates the filetype menu. */
 void filetypes_init()
 {
 	filetype_id ft_id;
 	GtkWidget *filetype_menu = lookup_widget(app->window, "set_filetype1_menu");
+	GtkWidget *sub_menu = filetype_menu;
+	GtkWidget *sub_menu_programming, *sub_menu_scripts, *sub_menu_markup, *sub_menu_misc;
+	GtkWidget *sub_item_programming, *sub_item_scripts, *sub_item_markup, *sub_item_misc;
 
 	filetypes_init_types();
 
+	create_sub_menu(sub_menu_programming, sub_item_programming, _("_Programming Languages"));
+	create_sub_menu(sub_menu_scripts, sub_item_scripts, _("_Scripting Languages"));
+	create_sub_menu(sub_menu_markup, sub_item_markup, _("_Markup Languages"));
+	create_sub_menu(sub_menu_misc, sub_item_misc, _("M_iscellaneous Languages"));
+
 	// Append all filetypes to the filetype menu
 	for (ft_id = 0; ft_id < GEANY_MAX_FILE_TYPES; ft_id++)
 	{
 		filetype *ft = filetypes[ft_id];
-		const gchar *title = (ft_id == GEANY_FILETYPES_ALL) ? _("None") : ft->title;
+		const gchar *title = ft->title;
 
 		// insert separators for different filetype groups
 		switch (ft_id)
 		{
-			case GEANY_FILETYPES_PERL:	// scripts
-			case GEANY_FILETYPES_XML:	// text documents
+			case GEANY_FILETYPES_GROUP_COMPILED:	// programming
+			{
+				sub_menu = sub_menu_programming;
+				break;
+			}
+			case GEANY_FILETYPES_GROUP_SCRIPT:	// scripts
+			{
+				sub_menu = sub_menu_scripts;
+				break;
+			}
+			case GEANY_FILETYPES_GROUP_MARKUP:	// markup
+			{	// (include also CSS, not really markup but fit quite well to HTML)
+				sub_menu = sub_menu_markup;
+				break;
+			}
+			case GEANY_FILETYPES_GROUP_MISC:	// misc
+			{
+				sub_menu = sub_menu_misc;
+				break;
+			}
 			case GEANY_FILETYPES_ALL:	// none
 			{
-				GtkWidget *sep = gtk_separator_menu_item_new();
-
-				gtk_widget_show(sep);
-				gtk_container_add(GTK_CONTAINER(filetype_menu), sep);
-			}
-			default:
+				sub_menu = filetype_menu;
+				title = _("None");
 				break;
+			}
+			default: break;
 		}
 		ft->item = NULL;
-		filetypes_create_menu_item(filetype_menu, title, ft);
+		filetypes_create_menu_item(sub_menu, title, ft);
 	}
 }
 
 
+// If uid is valid, return corresponding filetype, otherwise NULL.
+filetype *filetypes_get_from_uid(gint uid)
+{
+	gint i;
+
+	for (i = 0; i < GEANY_MAX_FILE_TYPES; i++)
+	{
+		filetype *ft = filetypes[i];
+
+		if (ft->uid == (guint) uid)
+			return ft;
+	}
+	return NULL;
+}
+
+
 static filetype *find_shebang(gint idx)
 {
 	gchar *line = sci_get_line(doc_list[idx].sci, 0);
 	filetype *ft = NULL;
 
-	if (strlen(line) > 2 && line[0] == '#' && line[1]=='!')
+	if (strlen(line) > 2 && line[0] == '#' && line[1] == '!')
 	{
 		/// TODO does g_path_get_basename() also work under Win32 for Unix filenames?
 		gchar *basename_interpreter = g_path_get_basename(line + 2);
@@ -599,12 +632,34 @@ static filetype *find_shebang(gint idx)
 			ft = filetypes[GEANY_FILETYPES_SH];
 		else if (strncmp(basename_interpreter, "csh", 3) == 0)
 			ft = filetypes[GEANY_FILETYPES_SH];
+		else if (strncmp(basename_interpreter, "ash", 3) == 0)
+			ft = filetypes[GEANY_FILETYPES_SH];
 		else if (strncmp(basename_interpreter, "dmd", 3) == 0)
 			ft = filetypes[GEANY_FILETYPES_D];
+		else if (strncmp(basename_interpreter, "wish", 4) == 0)
+			ft = filetypes[GEANY_FILETYPES_TCL];
 		// what else to add?
 
 		g_free(basename_interpreter);
 	}
+	// detect XML files
+	if (strncmp(line, "<?xml", 5) == 0)
+	{
+		// HTML and DocBook files might also start with <?xml, so detect them based on filename
+		// extension and use the detected filetype, else assume XML
+		ft = filetypes_detect_from_filename(doc_list[idx].file_name);
+		if (FILETYPE_ID(ft) != GEANY_FILETYPES_HTML &&
+			FILETYPE_ID(ft) != GEANY_FILETYPES_DOCBOOK &&
+			FILETYPE_ID(ft) != GEANY_FILETYPES_PERL &&	// Perl, Python and PHP only to be safe
+			FILETYPE_ID(ft) != GEANY_FILETYPES_PHP &&
+			FILETYPE_ID(ft) != GEANY_FILETYPES_PYTHON)
+
+			ft = filetypes[GEANY_FILETYPES_XML];
+	}
+	else if (strncmp(line, "<?php", 5) == 0)
+	{
+		ft = filetypes[GEANY_FILETYPES_PHP];
+	}
 
 	g_free(line);
 	return ft;
@@ -620,6 +675,7 @@ filetype *filetypes_detect_from_file(gint idx)
 		return filetypes[GEANY_FILETYPES_ALL];
 
 	// try to find a shebang and if found use it prior to the filename extension
+	// also checks for <?xml
 	ft = find_shebang(idx);
 	if (ft != NULL) return ft;
 
@@ -640,6 +696,10 @@ filetype *filetypes_detect_from_filename(const gchar *utf8_filename)
 
 	// to match against the basename of the file(because of Makefile*)
 	base_filename = g_path_get_basename(utf8_filename);
+#ifdef G_OS_WIN32
+	// use lower case basename
+	setptr(base_filename, g_utf8_strdown(base_filename, -1));
+#endif
 
 	for(i = 0; i < GEANY_MAX_FILE_TYPES; i++)
 	{
@@ -807,17 +867,17 @@ void filetypes_get_config(GKeyFile *config, GKeyFile *configh, gint ft)
 
 gchar *filetypes_get_conf_extension(gint filetype_idx)
 {
-	gchar *result, *tmp = g_strdup(filetypes[filetype_idx]->name);
+	gchar *result;
 
 	// Handle any special extensions different from lowercase filetype->name
 	switch (filetype_idx)
 	{
 		case GEANY_FILETYPES_CPP: result = g_strdup("cpp"); break;
+		case GEANY_FILETYPES_CS: result = g_strdup("cs"); break;
 		case GEANY_FILETYPES_MAKE: result = g_strdup("makefile"); break;
 		case GEANY_FILETYPES_OMS: result = g_strdup("oms"); break;
-		default: result = g_ascii_strdown(tmp, -1); break;
+		default: result = g_ascii_strdown(filetypes[filetype_idx]->name, -1); break;
 	}
-	g_free(tmp);
 	return result;
 }
 
@@ -904,3 +964,12 @@ GtkFileFilter *filetypes_create_file_filter(filetype *ft)
 }
 
 
+// Indicates whether there is a tag parser for the filetype or not.
+gboolean filetype_has_tags(filetype *ft)
+{
+	g_return_val_if_fail(ft != NULL, FALSE);
+
+	return ft->lang >= 0;
+}
+
+
