@@ -18,57 +18,64 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: filetypes.h 1530 2007-05-14 12:55:55Z ntrel $
+ * $Id: filetypes.h 1800 2007-08-15 15:37:21Z ntrel $
  */
 
 
 #ifndef GEANY_FILETYPES_H
 #define GEANY_FILETYPES_H 1
 
-#ifndef PLAT_GTK
-#   define PLAT_GTK 1	// needed for ScintillaWidget.h
-#endif
-
 #include "Scintilla.h"
 #include "ScintillaWidget.h"
 
 
+// each group should be alpha-sorted, based on filetype::name (not enum name)
 typedef enum
 {
 	// normally compiled languages
-	GEANY_FILETYPES_C = 0,
+	GEANY_FILETYPES_GROUP_COMPILED = 0,
+	GEANY_FILETYPES_ASM = GEANY_FILETYPES_GROUP_COMPILED,
+	GEANY_FILETYPES_C,
 	GEANY_FILETYPES_CPP,
+	GEANY_FILETYPES_CS,
+	GEANY_FILETYPES_CAML,
 	GEANY_FILETYPES_D,
+	GEANY_FILETYPES_FORTRAN,
+	GEANY_FILETYPES_BASIC,	// FreeBasic
+	GEANY_FILETYPES_HASKELL,
+	GEANY_FILETYPES_HAXE,
 	GEANY_FILETYPES_JAVA,
 	GEANY_FILETYPES_PASCAL,
-	GEANY_FILETYPES_ASM,
-	GEANY_FILETYPES_FORTRAN,
-	GEANY_FILETYPES_CAML,
-	GEANY_FILETYPES_HASKELL,
 	GEANY_FILETYPES_VHDL,
 
 	// script languages
-	GEANY_FILETYPES_PERL,
-	GEANY_FILETYPES_PHP,
+	GEANY_FILETYPES_GROUP_SCRIPT,
+	GEANY_FILETYPES_FERITE = GEANY_FILETYPES_GROUP_SCRIPT,
 	GEANY_FILETYPES_JS,
-	GEANY_FILETYPES_PYTHON,
-	GEANY_FILETYPES_RUBY,
-	GEANY_FILETYPES_TCL,
 	GEANY_FILETYPES_LUA,
-	GEANY_FILETYPES_FERITE,
-	GEANY_FILETYPES_SH,
 	GEANY_FILETYPES_MAKE,
 	GEANY_FILETYPES_OMS,
+	GEANY_FILETYPES_PERL,
+	GEANY_FILETYPES_PHP,
+	GEANY_FILETYPES_PYTHON,
+	GEANY_FILETYPES_RUBY,
+	GEANY_FILETYPES_SH,
+	GEANY_FILETYPES_TCL,
 
- 	// documents and others
- 	GEANY_FILETYPES_XML,
+ 	// markup langauges
+	GEANY_FILETYPES_GROUP_MARKUP,
+	GEANY_FILETYPES_CSS = GEANY_FILETYPES_GROUP_MARKUP,
 	GEANY_FILETYPES_DOCBOOK,
 	GEANY_FILETYPES_HTML,
-	GEANY_FILETYPES_CSS,
-	GEANY_FILETYPES_SQL,
-	GEANY_FILETYPES_LATEX,
+ 	GEANY_FILETYPES_XML,
+
+	// miscellaneous languages
+	GEANY_FILETYPES_GROUP_MISC,
+	GEANY_FILETYPES_CONF = GEANY_FILETYPES_GROUP_MISC,
 	GEANY_FILETYPES_DIFF,
-	GEANY_FILETYPES_CONF,
+	GEANY_FILETYPES_LATEX,
+	GEANY_FILETYPES_REST,
+	GEANY_FILETYPES_SQL,
 
 	GEANY_FILETYPES_ALL,	// must be last filetype, used for 'None' item.
 	GEANY_MAX_FILE_TYPES
@@ -95,7 +102,7 @@ struct build_programs
 	gboolean modified;
 };
 
-typedef struct filetype
+struct filetype
 {
 	filetype_id		  id;
 	guint	 		  uid;				// unique id as reference for saved filetype in config file
@@ -103,7 +110,6 @@ typedef struct filetype
 	langType 		  lang;				// represents the langType of tagmanager(see the table
 										// in tagmanager/parsers.h), -1 represents all, -2 none
 	gchar	 		 *name;				// will be used as name for tagmanager
-	gboolean 		  has_tags;			// indicates whether there is a tag parser for it or not
 	gchar	 		 *title;			// will be shown in the file open dialog
 	gchar	 		 *extension;
 	gchar			**pattern;
@@ -114,9 +120,9 @@ typedef struct filetype
 	struct build_programs	*programs;
 	struct build_actions	*actions;
 	void (*style_func_ptr) (ScintillaObject*);
-} filetype;
+};
 
-filetype *filetypes[GEANY_MAX_FILE_TYPES];
+extern filetype *filetypes[GEANY_MAX_FILE_TYPES];
 
 
 // If uid is valid, return corresponding filetype, otherwise NULL.
@@ -150,4 +156,6 @@ GtkFileFilter *filetypes_create_file_filter_all_source();
 
 gchar *filetypes_get_conf_extension(gint filetype_idx);
 
+gboolean filetype_has_tags(filetype *ft);
+
 #endif
