@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: keybindings.h 1511 2007-05-06 14:49:51Z eht16 $
+ * $Id: keybindings.h 1881 2007-09-13 16:40:19Z ntrel $
  */
 
 
@@ -32,17 +32,20 @@ typedef struct binding
 {
 	guint key;
 	GdkModifierType mods;
-	// at the moment only needed as keys for the configuration file because indices or tranlatable
+	// at the moment only needed as keys for the configuration file because indices or translatable
 	// strings as keys are not very useful
 	const gchar *name;
 	const gchar *label;
 	// function pointer to a callback function, just to keep the code in keypress event
 	// callback function clear
 	KBCallback cb_func;
+	// string to use as a section name in the preferences dialog in keybinding treeview as well as
+	// in the keybinding help dialog, set only for the first binding in the section
+	gchar *section;
 } binding;
 
 
-enum
+typedef enum
 {
 	GEANY_KEYS_MENU_NEW = 0,
 	GEANY_KEYS_MENU_OPEN,
@@ -62,6 +65,7 @@ enum
 	GEANY_KEYS_MENU_INSERTDATE,
 	GEANY_KEYS_MENU_PREFERENCES,
 
+	GEANY_KEYS_MENU_FIND,
 	GEANY_KEYS_MENU_FINDNEXT,
 	GEANY_KEYS_MENU_FINDPREVIOUS,
 	GEANY_KEYS_MENU_FINDINFILES,
@@ -70,10 +74,6 @@ enum
 	GEANY_KEYS_MENU_FINDPREVSEL,
 	GEANY_KEYS_MENU_NEXTMESSAGE,
 	GEANY_KEYS_MENU_GOTOLINE,
-	GEANY_KEYS_POPUP_FINDUSAGE,
-	GEANY_KEYS_POPUP_GOTOTAGDEFINITION,
-	GEANY_KEYS_POPUP_GOTOTAGDECLARATION,
-	GEANY_KEYS_POPUP_CONTEXTACTION,
 
 	GEANY_KEYS_MENU_TOGGLEALL,
 	GEANY_KEYS_MENU_FULLSCREEN,
@@ -100,6 +100,8 @@ enum
 	GEANY_KEYS_MENU_OPENCOLORCHOOSER,
 	GEANY_KEYS_MENU_INSERTSPECIALCHARS,
 
+	GEANY_KEYS_MENU_HELP,
+
 	GEANY_KEYS_SWITCH_EDITOR,
 	GEANY_KEYS_SWITCH_SCRIBBLE,
 	GEANY_KEYS_SWITCH_VTE,
@@ -107,15 +109,25 @@ enum
 	GEANY_KEYS_SWITCH_TABLEFT,
 	GEANY_KEYS_SWITCH_TABRIGHT,
 	GEANY_KEYS_SWITCH_TABLASTUSED,
+	GEANY_KEYS_MOVE_TABLEFT,
+	GEANY_KEYS_MOVE_TABRIGHT,
+	GEANY_KEYS_NAV_FORWARD,
+	GEANY_KEYS_NAV_BACK,
 
-	GEANY_KEYS_EDIT_TOLOWERCASE,
-	GEANY_KEYS_EDIT_TOUPPERCASE,
+	GEANY_KEYS_EDIT_TOGGLECASE,
 	GEANY_KEYS_EDIT_DUPLICATELINE,
+	GEANY_KEYS_EDIT_DELETELINE,
+	GEANY_KEYS_EDIT_COPYLINE,
+	GEANY_KEYS_EDIT_CUTLINE,
+	GEANY_KEYS_EDIT_TRANSPOSELINE,
 	GEANY_KEYS_EDIT_COMMENTLINETOGGLE,
 	GEANY_KEYS_EDIT_COMMENTLINE,
 	GEANY_KEYS_EDIT_UNCOMMENTLINE,
 	GEANY_KEYS_EDIT_INCREASEINDENT,
 	GEANY_KEYS_EDIT_DECREASEINDENT,
+	GEANY_KEYS_EDIT_INCREASEINDENTBYSPACE,
+	GEANY_KEYS_EDIT_DECREASEINDENTBYSPACE,
+	GEANY_KEYS_EDIT_AUTOINDENT,
 	GEANY_KEYS_EDIT_SENDTOCMD1,
 	GEANY_KEYS_EDIT_SENDTOCMD2,
 	GEANY_KEYS_EDIT_SENDTOCMD3,
@@ -124,24 +136,36 @@ enum
 	GEANY_KEYS_EDIT_GOTONEXTMARKER,
 	GEANY_KEYS_EDIT_GOTOPREVIOUSMARKER,
 	GEANY_KEYS_EDIT_SELECTWORD,
+	GEANY_KEYS_EDIT_SELECTLINE,
+	GEANY_KEYS_EDIT_SELECTPARAGRAPH,
+	GEANY_KEYS_EDIT_SCROLLTOLINE,
+	GEANY_KEYS_EDIT_SCROLLLINEUP,
+	GEANY_KEYS_EDIT_SCROLLLINEDOWN,
 	GEANY_KEYS_EDIT_INSERTALTWHITESPACE,
 
+	GEANY_KEYS_POPUP_FINDUSAGE,
+	GEANY_KEYS_POPUP_GOTOTAGDEFINITION,
+	GEANY_KEYS_POPUP_GOTOTAGDECLARATION,
+	GEANY_KEYS_POPUP_CONTEXTACTION,
+
 	GEANY_KEYS_EDIT_AUTOCOMPLETE,
 	GEANY_KEYS_EDIT_CALLTIP,
 	GEANY_KEYS_EDIT_MACROLIST,
 	GEANY_KEYS_EDIT_COMPLETECONSTRUCT,
 	GEANY_KEYS_EDIT_SUPPRESSCOMPLETION,
 	GEANY_MAX_KEYS
-};
+}
+GeanyKeyCommand;
 
 binding	*keys[GEANY_MAX_KEYS];
 
 
-
 void keybindings_init(void);
 
 void keybindings_free(void);
 
+void keybindings_cmd(GeanyKeyCommand cmd_id);
+
 /* just write the content of the keys array to the config file */
 void keybindings_write_to_file(void);
 
