@@ -18,9 +18,12 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: main.c 1269 2007-02-12 17:21:57Z eht16 $
+ * $Id: main.c 1923 2007-10-01 12:22:44Z eht16 $
  */
 
+/*
+ * Program initialization and cleanup.
+ */
 
 #include <signal.h>
 #include <time.h>
@@ -32,7 +35,12 @@
 
 #include "geany.h"
 
+#if HAVE_LOCALE_H
+# include <locale.h>
+#endif
+
 #include "main.h"
+#include "prefs.h"
 #include "interface.h"
 #include "support.h"
 #include "callbacks.h"
@@ -40,6 +48,7 @@
 #include "ui_utils.h"
 #include "utils.h"
 #include "document.h"
+#include "filetypes.h"
 #include "keyfile.h"
 #include "win32.h"
 #include "msgwindow.h"
@@ -49,11 +58,15 @@
 #include "treeviews.h"
 #include "notebook.h"
 #include "keybindings.h"
-#include "sci_cb.h"
+#include "editor.h"
 #include "search.h"
 #include "build.h"
 #include "highlighting.h"
 #include "symbols.h"
+#include "project.h"
+#include "tools.h"
+#include "navqueue.h"
+#include "plugins.h"
 
 #ifdef HAVE_SOCKET
 # include "socket.h"
@@ -63,12 +76,20 @@
 # include "vte.h"
 #endif
 
-#define N_(String) (String)
+#ifndef N_
+# define N_(String) (String)
+#endif
 
 
+GeanyApp *app;
 
+GeanyStatus	 main_status;
 CommandLineOptions cl_options;	// fields initialised in parse_command_line_options
 
+
+static gboolean want_plugins;
+
+// command-line options
 static gboolean debug_mode = FALSE;
 static gboolean ignore_global_tags = FALSE;
 static gboolean no_msgwin = FALSE;
@@ -82,24 +103,34 @@ static gchar *lib_vte = NULL;
 static gboolean ignore_socket = FALSE;
 #endif
 static gboolean generate_datafiles = FALSE;
+static gboolean generate_tags = FALSE;
+static gboolean no_preprocessing = FALSE;
+static gboolean ft_names = FALSE;
+static gboolean no_plugins = FALSE;
 
+// in alphabetical order of short options
 static GOptionEntry entries[] =
 {
-	{ "config", 'c', 0, G_OPTION_ARG_FILENAME, &alternate_config, N_("use an alternate configuration directory"), NULL },
-	{ "debug", 'd', 0, G_OPTION_ARG_NONE, &debug_mode, N_("runs in debug mode (means being verbose)"), NULL },
-	{ "generate-data-files", 'g', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &generate_datafiles, "", NULL },
+	{ "column", 0, 0, G_OPTION_ARG_INT, &cl_options.goto_column, N_("Set initial column number for the first opened file (useful in conjunction with --line)"), NULL },
+	{ "config", 'c', 0, G_OPTION_ARG_FILENAME, &alternate_config, N_("Use an alternate configuration directory"), NULL },
+	{ "debug", 'd', 0, G_OPTION_ARG_NONE, &debug_mode, N_("Runs in debug mode (means being verbose)"), NULL },
+	{ "ft-names", 0, 0, G_OPTION_ARG_NONE, &ft_names, N_("Print internal filetype names"), NULL },
+	{ "generate-tags", 'g', 0, G_OPTION_ARG_NONE, &generate_tags, N_("Generate global tags file (see documentation)"), NULL },
+	{ "no-preprocessing", 'P', 0, G_OPTION_ARG_NONE, &no_preprocessing, NULL, NULL },
+	{ "generate-data-files", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &generate_datafiles, NULL, NULL },
 #ifdef HAVE_SOCKET
-	{ "new-instance", 'i', 0, G_OPTION_ARG_NONE, &ignore_socket, N_("don't open files in a running instance, force opening a new instance"), NULL },
+	{ "new-instance", 'i', 0, G_OPTION_ARG_NONE, &ignore_socket, N_("Don't open files in a running instance, force opening a new instance"), NULL },
 #endif
-	{ "line", 'l', 0, G_OPTION_ARG_INT, &cl_options.goto_line, N_("set initial line number for the first opened file"), NULL },
-	{ "no-msgwin", 'm', 0, G_OPTION_ARG_NONE, &no_msgwin, N_("don't show message window at startup"), NULL },
-	{ "no-ctags", 'n', 0, G_OPTION_ARG_NONE, &ignore_global_tags, N_("don't load auto completion data (see documentation)"), NULL },
+	{ "line", 'l', 0, G_OPTION_ARG_INT, &cl_options.goto_line, N_("Set initial line number for the first opened file"), NULL },
+	{ "no-msgwin", 'm', 0, G_OPTION_ARG_NONE, &no_msgwin, N_("Don't show message window at startup"), NULL },
+	{ "no-ctags", 'n', 0, G_OPTION_ARG_NONE, &ignore_global_tags, N_("Don't load auto completion data (see documentation)"), NULL },
+	{ "no-plugins", 'p', 0, G_OPTION_ARG_NONE, &no_plugins, N_("Don't load plugins"), NULL },
 	{ "no-session", 's', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &cl_options.load_session, N_("don't load the previous session's files"), NULL },
 #ifdef HAVE_VTE
-	{ "no-terminal", 't', 0, G_OPTION_ARG_NONE, &no_vte, N_("don't load terminal support"), NULL },
-	{ "vte-lib", 0, 0, G_OPTION_ARG_FILENAME, &lib_vte, N_("filename of libvte.so"), NULL },
+	{ "no-terminal", 't', 0, G_OPTION_ARG_NONE, &no_vte, N_("Don't load terminal support"), NULL },
+	{ "vte-lib", 0, 0, G_OPTION_ARG_FILENAME, &lib_vte, N_("Filename of libvte.so"), NULL },
 #endif
-	{ "version", 'v', 0, G_OPTION_ARG_NONE, &show_version, N_("show version and exit"), NULL },
+	{ "version", 'v', 0, G_OPTION_ARG_NONE, &show_version, N_("Show version and exit"), NULL },
 	{ NULL, 0, 0, 0, NULL, NULL, NULL }
 };
 
@@ -127,21 +158,21 @@ static void apply_settings(void)
 	ui_update_fold_items();
 
 	// toolbar, message window and sidebar are by default visible, so don't change it if it is true
-	if (! app->toolbar_visible)
+	if (! prefs.toolbar_visible)
 	{
 		app->ignore_callback = TRUE;
 		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget(app->window, "menu_show_toolbar1")), FALSE);
 		gtk_widget_hide(app->toolbar);
 		app->ignore_callback = FALSE;
 	}
-	if (! app->msgwindow_visible)
+	if (! ui_prefs.msgwindow_visible)
 	{
 		app->ignore_callback = TRUE;
 		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget(app->window, "menu_show_messages_window1")), FALSE);
 		gtk_widget_hide(lookup_widget(app->window, "scrolledwindow1"));
 		app->ignore_callback = FALSE;
 	}
-	if (! app->sidebar_visible)
+	if (! ui_prefs.sidebar_visible)
 	{
 		app->ignore_callback = TRUE;
 		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget(app->window, "menu_show_sidebar1")), FALSE);
@@ -149,78 +180,85 @@ static void apply_settings(void)
 	}
 	ui_treeviews_show_hide(TRUE);
 	// sets the icon style of the toolbar
-	switch (app->toolbar_icon_style)
+	switch (prefs.toolbar_icon_style)
 	{
 		case GTK_TOOLBAR_BOTH:
 		{
 			//gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget(app->window, "images_and_text1")), TRUE);
-			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget(app->toolbar_menu, "images_and_text2")), TRUE);
+			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget(ui_widgets.toolbar_menu, "images_and_text2")), TRUE);
 			break;
 		}
 		case GTK_TOOLBAR_ICONS:
 		{
 			//gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget(app->window, "images_only1")), TRUE);
-			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget(app->toolbar_menu, "images_only2")), TRUE);
+			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget(ui_widgets.toolbar_menu, "images_only2")), TRUE);
 			break;
 		}
 		case GTK_TOOLBAR_TEXT:
 		{
 			//gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget(app->window, "text_only1")), TRUE);
-			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget(app->toolbar_menu, "text_only2")), TRUE);
+			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget(ui_widgets.toolbar_menu, "text_only2")), TRUE);
 			break;
 		}
 	}
-	gtk_toolbar_set_style(GTK_TOOLBAR(app->toolbar), app->toolbar_icon_style);
+	gtk_toolbar_set_style(GTK_TOOLBAR(app->toolbar), prefs.toolbar_icon_style);
 
 	// sets the icon size of the toolbar, use user preferences (.gtkrc) if not set
-	if (app->toolbar_icon_size == GTK_ICON_SIZE_SMALL_TOOLBAR ||
-		app->toolbar_icon_size == GTK_ICON_SIZE_LARGE_TOOLBAR)
+	if (prefs.toolbar_icon_size == GTK_ICON_SIZE_SMALL_TOOLBAR ||
+		prefs.toolbar_icon_size == GTK_ICON_SIZE_LARGE_TOOLBAR)
 	{
-		gtk_toolbar_set_icon_size(GTK_TOOLBAR(app->toolbar), app->toolbar_icon_size);
+		gtk_toolbar_set_icon_size(GTK_TOOLBAR(app->toolbar), prefs.toolbar_icon_size);
 	}
-	ui_update_toolbar_icons(app->toolbar_icon_size);
+	ui_update_toolbar_icons(prefs.toolbar_icon_size);
 
 	// line number and markers margin are by default enabled
-	if (! app->show_markers_margin)
+	if (! editor_prefs.show_markers_margin)
 	{
 		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget(app->window, "menu_markers_margin1")), FALSE);
-		app->show_markers_margin = FALSE;
+		editor_prefs.show_markers_margin = FALSE;
 	}
-	if (! app->show_linenumber_margin)
+	if (! editor_prefs.show_linenumber_margin)
 	{
 		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget(app->window, "menu_linenumber_margin1")), FALSE);
-		app->show_linenumber_margin = FALSE;
+		editor_prefs.show_linenumber_margin = FALSE;
 	}
 
 	// interprets the saved window geometry
-	if (app->pref_main_save_winpos && app->geometry[0] != -1)
+	if (prefs.save_winpos && ui_prefs.geometry[0] != -1)
 	{
-		gtk_window_move(GTK_WINDOW(app->window), app->geometry[0], app->geometry[1]);
-		gtk_window_set_default_size(GTK_WINDOW(app->window), app->geometry[2], app->geometry[3]);
+		gtk_window_move(GTK_WINDOW(app->window), ui_prefs.geometry[0], ui_prefs.geometry[1]);
+		gtk_window_set_default_size(GTK_WINDOW(app->window), ui_prefs.geometry[2], ui_prefs.geometry[3]);
+		if (ui_prefs.geometry[4] == 1)
+			gtk_window_maximize(GTK_WINDOW(app->window));
+	}
+
+	// hide statusbar if desired
+	if (! prefs.statusbar_visible)
+	{
+		gtk_widget_hide(app->statusbar);
 	}
 
 	app->ignore_callback = TRUE;
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
-				lookup_widget(app->window, "menu_line_breaking1")), app->pref_editor_line_breaking);
+				lookup_widget(app->window, "menu_line_breaking1")), editor_prefs.line_wrapping);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
-				lookup_widget(app->window, "menu_use_auto_indention1")), app->pref_editor_use_auto_indention);
+				lookup_widget(app->window, "menu_use_auto_indentation1")),
+				(editor_prefs.indent_mode != INDENT_NONE));
 	app->ignore_callback = FALSE;
 
-	// connect the toolbar dropdown menus
+	// connect the toolbar dropdown menu for the new button
 	gtk_menu_tool_button_set_menu(GTK_MENU_TOOL_BUTTON(
-			lookup_widget(app->window, "menutoolbutton1")), app->new_file_menu);
-	gtk_menu_tool_button_set_menu(GTK_MENU_TOOL_BUTTON(
-			lookup_widget(app->window, "toolbutton9")), app->recent_files_toolbar);
+			lookup_widget(app->window, "menutoolbutton1")), ui_widgets.new_file_menu);
 
 	// set the tab placements of the notebooks
-	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(app->notebook), app->tab_pos_editor);
-	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(msgwindow.notebook), app->tab_pos_msgwin);
-	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(app->treeview_notebook), app->tab_pos_sidebar);
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(app->notebook), prefs.tab_pos_editor);
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(msgwindow.notebook), prefs.tab_pos_msgwin);
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(app->treeview_notebook), prefs.tab_pos_sidebar);
 
 	ui_update_toolbar_items();
 
 	// whether to show notebook tabs or not
-	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(app->notebook), app->show_notebook_tabs);
+	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(app->notebook), prefs.show_notebook_tabs);
 }
 
 
@@ -228,99 +266,64 @@ static void main_init(void)
 {
 	// inits
 	app->window				= NULL;
-	app->open_fontsel		= NULL;
-	app->open_colorsel		= NULL;
-	app->open_filesel		= NULL;
-	app->save_filesel		= NULL;
-	app->prefs_dialog		= NULL;
-	app->default_tag_tree	= NULL;
-	app->main_window_realized= FALSE;
-	app->tab_order_ltr		= FALSE;
-	app->quitting			= FALSE;
+	app->project			= NULL;
+	ui_widgets.open_fontsel		= NULL;
+	ui_widgets.open_colorsel		= NULL;
+	ui_widgets.open_filesel		= NULL;
+	ui_widgets.save_filesel		= NULL;
+	ui_widgets.prefs_dialog		= NULL;
+	tv.default_tag_tree	= NULL;
+	main_status.main_window_realized= FALSE;
+	prefs.tab_order_ltr		= FALSE;
+	main_status.quitting			= FALSE;
 	app->ignore_callback	= FALSE;
 	app->tm_workspace				= tm_get_workspace();
-	app->recent_queue				= g_queue_new();
-	app->opening_session_files		= FALSE;
+	ui_prefs.recent_queue				= g_queue_new();
+	main_status.opening_session_files		= FALSE;
 
 	app->window = create_window1();
-	app->new_file_menu = gtk_menu_new();
-	app->recent_files_toolbar = gtk_menu_new();
-	app->recent_files_menubar = gtk_menu_new();
-	gtk_menu_item_set_submenu(GTK_MENU_ITEM(lookup_widget(app->window, "recent_files1")),
-		app->recent_files_menubar);
+	ui_widgets.new_file_menu = gtk_menu_new();
+	ui_widgets.recent_files_toolbar = gtk_menu_new();
+	ui_widgets.recent_files_menuitem = lookup_widget(app->window, "recent_files1");
+	ui_widgets.recent_files_menubar = gtk_menu_new();
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(ui_widgets.recent_files_menuitem),
+							ui_widgets.recent_files_menubar);
 
-	// store important pointers in the MyApp structure
+	// store important pointers in the GeanyApp structure
 	app->toolbar = lookup_widget(app->window, "toolbar1");
-	app->tagbar = lookup_widget(app->window, "scrolledwindow2");
 	app->treeview_notebook = lookup_widget(app->window, "notebook3");
 	app->notebook = lookup_widget(app->window, "notebook1");
 	app->statusbar = lookup_widget(app->window, "statusbar");
 	app->popup_menu = create_edit_menu1();
-	app->toolbar_menu = create_toolbar_popup_menu1();
-	app->compile_button = lookup_widget(app->window, "toolbutton13");
-	app->run_button = lookup_widget(app->window, "toolbutton26");
-	app->popup_goto_items[0] = lookup_widget(app->popup_menu, "goto_tag_definition1");
-	app->popup_goto_items[1] = lookup_widget(app->popup_menu, "goto_tag_declaration1");
-	app->popup_goto_items[2] = lookup_widget(app->popup_menu, "find_usage1");
-	app->popup_items[0] = lookup_widget(app->popup_menu, "cut1");
-	app->popup_items[1] = lookup_widget(app->popup_menu, "copy1");
-	app->popup_items[2] = lookup_widget(app->popup_menu, "delete1");
-	app->popup_items[3] = lookup_widget(app->popup_menu, "to_lower_case1");
-	app->popup_items[4] = lookup_widget(app->popup_menu, "to_upper_case1");
-	app->menu_copy_items[0] = lookup_widget(app->window, "menu_cut1");
-	app->menu_copy_items[1] = lookup_widget(app->window, "menu_copy1");
-	app->menu_copy_items[2] = lookup_widget(app->window, "menu_delete1");
-	app->menu_copy_items[3] = lookup_widget(app->window, "menu_to_lower_case2");
-	app->menu_copy_items[4] = lookup_widget(app->window, "menu_to_upper_case2");
-	app->menu_insert_include_item[0] = lookup_widget(app->popup_menu, "insert_include1");
-	app->menu_insert_include_item[1] = lookup_widget(app->window, "insert_include2");
-	app->save_buttons[0] = lookup_widget(app->window, "menu_save1");
-	app->save_buttons[1] = lookup_widget(app->window, "toolbutton10");
-	app->save_buttons[2] = lookup_widget(app->window, "menu_save_all1");
-	app->save_buttons[3] = lookup_widget(app->window, "toolbutton22");
-	app->sensitive_buttons[0] = lookup_widget(app->window, "menu_close1");
-	app->sensitive_buttons[1] = lookup_widget(app->window, "toolbutton15");
-	app->sensitive_buttons[2] = lookup_widget(app->window, "menu_change_font1");
-	app->sensitive_buttons[3] = lookup_widget(app->window, "entry1");
-	app->sensitive_buttons[4] = lookup_widget(app->window, "toolbutton18");
-	app->sensitive_buttons[5] = lookup_widget(app->window, "toolbutton20");
-	app->sensitive_buttons[6] = lookup_widget(app->window, "toolbutton21");
-	app->sensitive_buttons[7] = lookup_widget(app->window, "menu_close_all1");
-	app->sensitive_buttons[8] = lookup_widget(app->window, "menu_save_all1");
-	app->sensitive_buttons[9] = lookup_widget(app->window, "toolbutton22");
-	app->sensitive_buttons[10] = app->compile_button;
-	app->sensitive_buttons[11] = lookup_widget(app->window, "menu_save_as1");
-	app->sensitive_buttons[12] = lookup_widget(app->window, "toolbutton23");
-	app->sensitive_buttons[13] = lookup_widget(app->window, "menu_count_words1");
-	app->sensitive_buttons[14] = lookup_widget(app->window, "menu_build1");
-	app->sensitive_buttons[15] = lookup_widget(app->window, "add_comments1");
-	app->sensitive_buttons[16] = lookup_widget(app->window, "search1");
-	app->sensitive_buttons[17] = lookup_widget(app->window, "menu_paste1");
-	app->sensitive_buttons[18] = lookup_widget(app->window, "menu_undo2");
-	app->sensitive_buttons[19] = lookup_widget(app->window, "preferences2");
-	app->sensitive_buttons[20] = lookup_widget(app->window, "menu_reload1");
-	app->sensitive_buttons[21] = lookup_widget(app->window, "menu_item4");
-	app->sensitive_buttons[22] = lookup_widget(app->window, "menu_markers_margin1");
-	app->sensitive_buttons[23] = lookup_widget(app->window, "menu_linenumber_margin1");
-	app->sensitive_buttons[24] = lookup_widget(app->window, "menu_choose_color1");
-	app->sensitive_buttons[25] = lookup_widget(app->window, "menu_zoom_in1");
-	app->sensitive_buttons[26] = lookup_widget(app->window, "menu_zoom_out1");
-	app->sensitive_buttons[27] = lookup_widget(app->window, "normal_size1");
-	app->sensitive_buttons[28] = lookup_widget(app->window, "toolbutton24");
-	app->sensitive_buttons[29] = lookup_widget(app->window, "toolbutton25");
-	app->sensitive_buttons[30] = lookup_widget(app->window, "entry_goto_line");
-	app->sensitive_buttons[31] = lookup_widget(app->window, "treeview6");
-	app->sensitive_buttons[32] = lookup_widget(app->window, "print1");
-	app->sensitive_buttons[33] = lookup_widget(app->window, "menu_reload_as1");
-	app->sensitive_buttons[34] = lookup_widget(app->window, "menu_select_all1");
-	app->sensitive_buttons[35] = lookup_widget(app->window, "insert_date1");
-	app->sensitive_buttons[36] = lookup_widget(app->window, "menu_format1");
-	app->redo_items[0] = lookup_widget(app->popup_menu, "redo1");
-	app->redo_items[1] = lookup_widget(app->window, "menu_redo2");
-	app->redo_items[2] = lookup_widget(app->window, "toolbutton_redo");
-	app->undo_items[0] = lookup_widget(app->popup_menu, "undo1");
-	app->undo_items[1] = lookup_widget(app->window, "menu_undo2");
-	app->undo_items[2] = lookup_widget(app->window, "toolbutton_undo");
+	ui_widgets.toolbar_menu = create_toolbar_popup_menu1();
+	ui_widgets.popup_goto_items[0] = lookup_widget(app->popup_menu, "goto_tag_definition1");
+	ui_widgets.popup_goto_items[1] = lookup_widget(app->popup_menu, "goto_tag_declaration1");
+	ui_widgets.popup_goto_items[2] = lookup_widget(app->popup_menu, "find_usage1");
+	ui_widgets.popup_copy_items[0] = lookup_widget(app->popup_menu, "cut1");
+	ui_widgets.popup_copy_items[1] = lookup_widget(app->popup_menu, "copy1");
+	ui_widgets.popup_copy_items[2] = lookup_widget(app->popup_menu, "delete1");
+	ui_widgets.menu_copy_items[0] = lookup_widget(app->window, "menu_cut1");
+	ui_widgets.menu_copy_items[1] = lookup_widget(app->window, "menu_copy1");
+	ui_widgets.menu_copy_items[2] = lookup_widget(app->window, "menu_delete1");
+	ui_widgets.menu_insert_include_items[0] = lookup_widget(app->popup_menu, "insert_include1");
+	ui_widgets.menu_insert_include_items[1] = lookup_widget(app->window, "insert_include2");
+	ui_widgets.save_buttons[0] = lookup_widget(app->window, "menu_save1");
+	ui_widgets.save_buttons[1] = lookup_widget(app->window, "toolbutton10");
+	ui_widgets.save_buttons[2] = lookup_widget(app->window, "menu_save_all1");
+	ui_widgets.save_buttons[3] = lookup_widget(app->window, "toolbutton22");
+	ui_widgets.redo_items[0] = lookup_widget(app->popup_menu, "redo1");
+	ui_widgets.redo_items[1] = lookup_widget(app->window, "menu_redo2");
+	ui_widgets.redo_items[2] = lookup_widget(app->window, "toolbutton_redo");
+	ui_widgets.undo_items[0] = lookup_widget(app->popup_menu, "undo1");
+	ui_widgets.undo_items[1] = lookup_widget(app->window, "menu_undo2");
+	ui_widgets.undo_items[2] = lookup_widget(app->window, "toolbutton_undo");
+
+	ui_init();
+
+	// set widget names for matching with .gtkrc-2.0
+	gtk_widget_set_name(app->window, "GeanyMainWindow");
+	gtk_widget_set_name(ui_widgets.toolbar_menu, "GeanyToolbarMenu");
+	gtk_widget_set_name(app->popup_menu, "GeanyEditMenu");
 }
 
 
@@ -334,8 +337,9 @@ gchar *get_argv_filename(const gchar *filename)
 		result = g_strdup(filename);
 	else
 	{
-		//use current dir
+		// use current dir
 		gchar *cur_dir = g_get_current_dir();
+
 		result = g_strjoin(
 			G_DIR_SEPARATOR_S, cur_dir, filename, NULL);
 		g_free(cur_dir);
@@ -352,7 +356,7 @@ static void setup_paths()
 	// set paths
 #ifdef G_OS_WIN32
 	// take the installation directory(the one where geany.exe is located) as the base for the
-	// language catalogs, documentation and data files
+	// documentation and data files
 	gchar *install_dir = g_win32_get_package_installation_directory("geany", NULL);
 
 	data_dir = g_strconcat(install_dir, "\\data", NULL); // e.g. C:\Program Files\geany\data
@@ -375,20 +379,27 @@ static void setup_paths()
 
 static void locale_init()
 {
+#ifdef ENABLE_NLS
 	gchar *locale_dir = NULL;
 
+#if HAVE_LOCALE_H
+	setlocale(LC_ALL, "");
+#endif
+
 #ifdef G_OS_WIN32
-	locale_dir = g_strdup(app->datadir);
+	gchar *install_dir = g_win32_get_package_installation_directory("geany", NULL);
+	// e.g. C:\Program Files\geany\lib\locale
+	locale_dir = g_strconcat(install_dir, "\\lib\\locale", NULL);
+	g_free(install_dir);
 #else
 	locale_dir = g_strdup(PACKAGE_LOCALE_DIR);
 #endif
 
-#ifdef ENABLE_NLS
 	bindtextdomain(GETTEXT_PACKAGE, locale_dir);
 	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
 	textdomain(GETTEXT_PACKAGE);
-#endif
 	g_free(locale_dir);
+#endif
 }
 
 
@@ -400,8 +411,9 @@ static void parse_command_line_options(gint *argc, gchar ***argv)
 	// first initialise cl_options fields with default values
 	cl_options.load_session = TRUE;
 	cl_options.goto_line = -1;
+	cl_options.goto_column = -1;
 
-	context = g_option_context_new(_(" - A fast and lightweight IDE"));
+	context = g_option_context_new(_("[FILES...]"));
 	g_option_context_add_main_entries(context, entries, GETTEXT_PACKAGE);
 	g_option_group_set_translation_domain(g_option_context_get_main_group(context), GETTEXT_PACKAGE);
 	g_option_context_add_group(context, gtk_get_option_group(TRUE));
@@ -419,9 +431,11 @@ static void parse_command_line_options(gint *argc, gchar ***argv)
 		exit(0);
 	}
 
-	app->debug_mode = debug_mode;
 #ifdef GEANY_DEBUG
+	app->debug_mode = TRUE;
 	geany_debug("debug mode built in (can't be disabled)");
+#else
+	app->debug_mode = debug_mode;
 #endif
 
 	if (alternate_config)
@@ -432,13 +446,47 @@ static void parse_command_line_options(gint *argc, gchar ***argv)
 	else
 		app->configdir = g_strconcat(GEANY_HOME_DIR, G_DIR_SEPARATOR_S, ".", PACKAGE, NULL);
 
+#ifdef GEANY_DEBUG
+	if (generate_datafiles)
+	{
+		filetypes_init_types();
+		configuration_generate_data_files();	// currently only filetype_extensions.conf
+		exit(0);
+	}
+#endif
+	if (generate_tags)
+	{
+		gboolean ret;
+
+		filetypes_init_types();
+		configuration_read_filetype_extensions();	// needed for *.lang.tags filetype matching
+		ret = symbols_generate_global_tags(*argc, *argv, ! no_preprocessing);
+		exit(ret);
+	}
+
+	if (ft_names)
+	{
+		int i;
+
+		printf("Geany's internal filetype names:\n");
+		filetypes_init_types();
+		for (i = 0; i < GEANY_MAX_FILE_TYPES; i++)
+		{
+			printf("%s\n", filetypes[i]->name);
+		}
+		filetypes_free_types();
+		exit(0);
+	}
+
 #ifdef HAVE_SOCKET
 	socket_info.ignore_socket = ignore_socket;
+	if (ignore_socket)
+		cl_options.load_session = FALSE;
 #endif
 #ifdef HAVE_VTE
 	vte_info.lib_vte = lib_vte;
 #endif
-	app->ignore_global_tags = ignore_global_tags;
+	cl_options.ignore_global_tags = ignore_global_tags;
 }
 
 
@@ -446,7 +494,6 @@ static void parse_command_line_options(gint *argc, gchar ***argv)
 static gint setup_config_dir()
 {
 	gint mkdir_result = 0;
-
 	gchar *tmp = app->configdir;
 
 	// convert configdir to locale encoding to avoid troubles
@@ -477,12 +524,91 @@ static void signal_cb(gint sig)
 }
 
 
+// open files from command line
+static gboolean open_cl_files(gint argc, gchar **argv)
+{
+	gint i;
+
+	if (argc <= 1) return FALSE;
+
+	document_delay_colourise();
+
+	for(i = 1; i < argc; i++)
+	{
+		gchar *filename = get_argv_filename(argv[i]);
+
+		if (filename != NULL &&
+			g_file_test(filename, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
+		{
+			gint idx;
+
+			idx = document_open_file(filename, FALSE, NULL, NULL);
+			// add recent file manually because opening_session_files is set
+			if (DOC_IDX_VALID(idx))
+				ui_add_recent_file(doc_list[idx].file_name);
+		}
+		else if (filename != NULL)
+		{	// create new file if it doesn't exist
+			gint idx;
+
+			idx = document_new_file(filename, NULL, NULL);
+			if (DOC_IDX_VALID(idx))
+				ui_add_recent_file(doc_list[idx].file_name);
+		}
+		else
+		{
+			gchar *msg = _("Could not find file '%s'.");
+
+			g_printerr(msg, filename);	// also print to the terminal
+			g_printerr("\n");
+			msgwin_status_add(msg, filename);
+		}
+		g_free(filename);
+	}
+	document_colourise_new();
+	return TRUE;
+}
+
+
+static void load_project_file()
+{
+	gchar *locale_filename;
+
+	g_return_if_fail(project_prefs.session_file != NULL);
+
+	locale_filename = utils_get_locale_from_utf8(project_prefs.session_file);
+
+	if (*locale_filename)
+		project_load_file(locale_filename);
+
+	g_free(locale_filename);
+	g_free(project_prefs.session_file);	// no longer needed
+}
+
+
+static void load_settings()
+{
+	configuration_load();
+	// let cmdline options overwrite configuration settings
+#ifdef HAVE_VTE
+	vte_info.have_vte = (no_vte) ? FALSE : vte_info.load_vte;
+#endif
+	if (no_msgwin) ui_prefs.msgwindow_visible = FALSE;
+
+	want_plugins = prefs.load_plugins && !no_plugins;
+}
+
+
 gint main(gint argc, gchar **argv)
 {
 	gint idx;
 	gint config_dir_result;
 
-	app = g_new0(MyApp, 1);
+	app = g_new0(GeanyApp, 1);
+	memset(&main_status, 0, sizeof(GeanyStatus));
+	memset(&prefs, 0, sizeof(GeanyPrefs));
+	memset(&ui_prefs, 0, sizeof(UIPrefs));
+	memset(&ui_widgets, 0, sizeof(UIWidgets));
 
 	setup_paths();
 	locale_init();
@@ -524,7 +650,8 @@ gint main(gint argc, gchar **argv)
 	}
 #endif
 
-	geany_debug("GTK+ %u.%u.%u, GLib %u.%u.%u",
+	geany_debug("Geany %s, GTK+ %u.%u.%u, GLib %u.%u.%u",
+		VERSION,
 		gtk_major_version, gtk_minor_version, gtk_micro_version,
 		glib_major_version, glib_minor_version, glib_micro_version);
 	gtk_init(&argc, &argv);
@@ -535,30 +662,23 @@ gint main(gint argc, gchar **argv)
 	gtk_window_set_default_size(GTK_WINDOW(app->window), GEANY_WINDOW_DEFAULT_WIDTH, GEANY_WINDOW_DEFAULT_HEIGHT);
 	encodings_init();
 
-	configuration_load();
-	// do this here to let cmdline options overwrite configuration settings
-#ifdef HAVE_VTE
-	vte_info.have_vte = (no_vte) ? FALSE : vte_info.load_vte;
-#endif
-	if (no_msgwin) app->msgwindow_visible = FALSE;
+	load_settings();
 
 	msgwin_init();
+	build_init();
 	search_init();
 	ui_create_insert_menu_items();
 	ui_create_insert_date_menu_items();
 	keybindings_init();
+	tools_create_insert_custom_command_menu_items();
 	notebook_init();
+	filetypes_init();
 	templates_init();
+	navqueue_init();
 	document_init_doclist();
-	filetypes_init_types();
-#ifdef GEANY_DEBUG
-	if (generate_datafiles)
-	{
-		configuration_generate_data_files();
-		exit(0);
-	}
-#endif
+	treeviews_init();
 	configuration_read_filetype_extensions();
+	configuration_read_autocompletions();
 
 	// set window icon
 	{
@@ -571,13 +691,15 @@ gint main(gint argc, gchar **argv)
 
 	// registering some basic events
 	g_signal_connect(G_OBJECT(app->window), "delete_event", G_CALLBACK(on_exit_clicked), NULL);
-	g_signal_connect(G_OBJECT(app->window), "configure-event", G_CALLBACK(on_window_configure_event), NULL);
-	g_signal_connect(G_OBJECT(app->window), "key-press-event", G_CALLBACK(on_window_key_press_event), NULL);
+	g_signal_connect(G_OBJECT(app->window), "key-press-event", G_CALLBACK(keybindings_got_event), NULL);
 	g_signal_connect(G_OBJECT(app->toolbar), "button-press-event", G_CALLBACK(toolbar_popup_menu), NULL);
+	g_signal_connect(G_OBJECT(lookup_widget(app->window, "textview_scribble")),
+							"motion-notify-event", G_CALLBACK(on_motion_event), NULL);
+	g_signal_connect(G_OBJECT(lookup_widget(app->window, "entry1")),
+							"motion-notify-event", G_CALLBACK(on_motion_event), NULL);
+	g_signal_connect(G_OBJECT(lookup_widget(app->window, "entry_goto_line")),
+							"motion-notify-event", G_CALLBACK(on_motion_event), NULL);
 
-	treeviews_prepare_openfiles();
-	treeviews_create_taglist_popup_menu();
-	treeviews_create_openfiles_popup_menu();
 #ifdef HAVE_VTE
 	vte_init();
 #endif
@@ -591,64 +713,46 @@ gint main(gint argc, gchar **argv)
 	// apply all configuration options
 	apply_settings();
 
-	// open files from command line
-	app->opening_session_files = TRUE;
-	if (argc > 1)
-	{
-		gint i;
-		for(i = 1; i < argc; i++)
-		{
-			if (argv[i] && g_file_test(argv[i], G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
-			{
-				gchar *filename = get_argv_filename(argv[i]);
-				gint idx;
+#ifdef HAVE_PLUGINS
+	// load any enabled plugins before we open any documents
+	if (want_plugins)
+		plugins_init();
+#endif
 
-				idx = document_open_file(-1, filename, 0, FALSE, NULL, NULL);
-				// add recent file manually because opening_session_files is set
-				if (DOC_IDX_VALID(idx))
-					ui_add_recent_file(doc_list[idx].file_name);
-				g_free(filename);
-			}
-			else
-				geany_debug("Could not load file '%s'.", argv[i]);
-		}
-	}
-	else if (app->pref_main_load_session && cl_options.load_session)
+	// load any command line files or session files
+	main_status.opening_session_files = TRUE;
+	if (! open_cl_files(argc, argv))
 	{
-		if (! configuration_open_files())
+		if (prefs.load_session && cl_options.load_session)
 		{
-			ui_update_popup_copy_items(-1);
-			ui_update_popup_reundo_items(-1);
+			load_project_file();
+
+			// load session files
+			if (! configuration_open_files())
+			{
+				ui_update_popup_copy_items(-1);
+				ui_update_popup_reundo_items(-1);
+			}
 		}
 	}
-	app->opening_session_files = FALSE;
+	main_status.opening_session_files = FALSE;
 
 	// open a new file if no other file was opened
-	if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook)) == 0)	document_new_file(NULL);
+	if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook)) == 0)
+		document_new_file(NULL, NULL, NULL);
 
-	ui_close_buttons_toggle();
+	ui_document_buttons_update();
 	ui_save_buttons_toggle(FALSE);
 
 	idx = document_get_cur_idx();
 	gtk_widget_grab_focus(GTK_WIDGET(doc_list[idx].sci));
-	gtk_tree_model_foreach(GTK_TREE_MODEL(tv.store_openfiles), treeviews_find_node, GINT_TO_POINTER(idx));
+	treeviews_select_openfiles_item(idx);
 	build_menu_update(idx);
-	ui_update_tag_list(idx, FALSE);
-
-#ifdef G_OS_WIN32
-	// hide "Build" menu item, at least until it is available for Windows
-	gtk_widget_hide(app->compile_button);
-	{
-		GtkWidget *compiler_tab;
-		compiler_tab = gtk_notebook_get_tab_label(GTK_NOTEBOOK(msgwindow.notebook),
-			gtk_notebook_get_nth_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_COMPILER));
-		gtk_widget_set_sensitive(compiler_tab, FALSE);
-	}
-#endif
+	treeviews_update_tag_list(idx, FALSE);
 
 	// finally realize the window to show the user what we have done
 	gtk_widget_show(app->window);
-	app->main_window_realized = TRUE;
+	main_status.main_window_realized = TRUE;
 
 	configuration_apply_settings();
 
@@ -676,6 +780,11 @@ void main_quit()
 	socket_finalize();
 #endif
 
+#ifdef HAVE_PLUGINS
+	if (want_plugins)
+		plugins_free();
+#endif
+	navqueue_free();
 	keybindings_free();
 	filetypes_save_commands();
 	filetypes_free_types();
@@ -686,43 +795,47 @@ void main_quit()
 	build_finalize();
 	document_finalize();
 	symbols_finalize();
+	editor_finalize();
+	if (app->project != NULL) project_close();
 
 	tm_workspace_free(TM_WORK_OBJECT(app->tm_workspace));
 	g_free(app->configdir);
 	g_free(app->datadir);
 	g_free(app->docdir);
-	g_free(app->custom_date_format);
-	g_free(app->editor_font);
-	g_free(app->tagbar_font);
-	g_free(app->msgwin_font);
-	g_free(app->long_line_color);
-	g_free(app->pref_template_developer);
-	g_free(app->pref_template_company);
-	g_free(app->pref_template_mail);
-	g_free(app->pref_template_initial);
-	g_free(app->pref_template_version);
-	g_free(app->tools_make_cmd);
-	g_free(app->tools_term_cmd);
-	g_free(app->tools_browser_cmd);
-	g_free(app->tools_print_cmd);
-	g_free(app->tools_grep_cmd);
-	while (! g_queue_is_empty(app->recent_queue))
+	g_free(prefs.default_open_path);
+	g_free(ui_prefs.custom_date_format);
+	g_free(prefs.editor_font);
+	g_free(prefs.tagbar_font);
+	g_free(prefs.msgwin_font);
+	g_free(editor_prefs.long_line_color);
+	g_free(prefs.context_action_cmd);
+	g_free(prefs.template_developer);
+	g_free(prefs.template_company);
+	g_free(prefs.template_mail);
+	g_free(prefs.template_initial);
+	g_free(prefs.template_version);
+	g_free(prefs.tools_make_cmd);
+	g_free(prefs.tools_term_cmd);
+	g_free(prefs.tools_browser_cmd);
+	g_free(prefs.tools_print_cmd);
+	g_free(prefs.tools_grep_cmd);
+	g_strfreev(ui_prefs.custom_commands);
+	while (! g_queue_is_empty(ui_prefs.recent_queue))
 	{
-		g_free(g_queue_pop_tail(app->recent_queue));
+		g_free(g_queue_pop_tail(ui_prefs.recent_queue));
 	}
-	g_queue_free(app->recent_queue);
+	g_queue_free(ui_prefs.recent_queue);
 
-	if (app->prefs_dialog && GTK_IS_WIDGET(app->prefs_dialog)) gtk_widget_destroy(app->prefs_dialog);
-	if (app->save_filesel && GTK_IS_WIDGET(app->save_filesel)) gtk_widget_destroy(app->save_filesel);
-	if (app->open_filesel && GTK_IS_WIDGET(app->open_filesel)) gtk_widget_destroy(app->open_filesel);
-	if (app->open_fontsel && GTK_IS_WIDGET(app->open_fontsel)) gtk_widget_destroy(app->open_fontsel);
-	if (app->open_colorsel && GTK_IS_WIDGET(app->open_colorsel)) gtk_widget_destroy(app->open_colorsel);
-	if (app->default_tag_tree && GTK_IS_WIDGET(app->default_tag_tree))
+	if (ui_widgets.prefs_dialog && GTK_IS_WIDGET(ui_widgets.prefs_dialog)) gtk_widget_destroy(ui_widgets.prefs_dialog);
+	if (ui_widgets.save_filesel && GTK_IS_WIDGET(ui_widgets.save_filesel)) gtk_widget_destroy(ui_widgets.save_filesel);
+	if (ui_widgets.open_filesel && GTK_IS_WIDGET(ui_widgets.open_filesel)) gtk_widget_destroy(ui_widgets.open_filesel);
+	if (ui_widgets.open_fontsel && GTK_IS_WIDGET(ui_widgets.open_fontsel)) gtk_widget_destroy(ui_widgets.open_fontsel);
+	if (ui_widgets.open_colorsel && GTK_IS_WIDGET(ui_widgets.open_colorsel)) gtk_widget_destroy(ui_widgets.open_colorsel);
+	if (tv.default_tag_tree && GTK_IS_WIDGET(tv.default_tag_tree))
 	{
-		g_object_unref(app->default_tag_tree);
-		gtk_widget_destroy(app->default_tag_tree);
+		g_object_unref(tv.default_tag_tree);
+		gtk_widget_destroy(tv.default_tag_tree);
 	}
-	scintilla_release_resources();
 #ifdef HAVE_VTE
 	if (vte_info.have_vte) vte_close();
 	g_free(vte_info.lib_vte);
@@ -733,8 +846,8 @@ void main_quit()
 	// destroy popup menus
 	if (app->popup_menu && GTK_IS_WIDGET(app->popup_menu))
 					gtk_widget_destroy(app->popup_menu);
-	if (app->toolbar_menu && GTK_IS_WIDGET(app->toolbar_menu))
-					gtk_widget_destroy(app->toolbar_menu);
+	if (ui_widgets.toolbar_menu && GTK_IS_WIDGET(ui_widgets.toolbar_menu))
+					gtk_widget_destroy(ui_widgets.toolbar_menu);
 	if (tv.popup_taglist && GTK_IS_WIDGET(tv.popup_taglist))
 					gtk_widget_destroy(tv.popup_taglist);
 	if (tv.popup_openfiles && GTK_IS_WIDGET(tv.popup_openfiles))
@@ -750,3 +863,16 @@ void main_quit()
 
 	gtk_main_quit();
 }
+
+
+// malloc compatibility code
+#undef malloc
+void *malloc(size_t n);
+
+// Allocate an N-byte block of memory from the heap. If N is zero, allocate a 1-byte block.
+void *rpl_malloc(size_t n)
+{
+	if (n == 0)
+		n = 1;
+	return malloc(n);
+}
