@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: main.h 1227 2007-01-24 22:38:53Z eht16 $
+ * $Id: main.h 1815 2007-08-23 11:34:06Z ntrel $
  */
 
 
@@ -27,14 +27,26 @@
 
 typedef struct
 {
-	gboolean load_session;
-	gint goto_line;
-	gint goto_column;
+	gboolean	load_session;
+	gint		goto_line;
+	gint		goto_column;
+	gboolean	ignore_global_tags;
 } CommandLineOptions;
 
 extern CommandLineOptions cl_options;
 
 
+typedef struct GeanyStatus
+{
+	gboolean	opening_session_files;	// state at startup while opening session files
+	gboolean	quitting;	// state when Geany is quitting completely
+	gboolean	main_window_realized;
+}
+GeanyStatus;
+
+extern GeanyStatus main_status;
+
+
 gchar *get_argv_filename(const gchar *filename);
 
 void main_quit();
