@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: msgwindow.c 1376 2007-03-09 13:52:26Z eht16 $
+ * $Id: msgwindow.c 1913 2007-09-27 11:39:21Z ntrel $
  */
 
 /*
@@ -31,12 +31,16 @@
 #include "geany.h"
 
 #include "support.h"
+#include "prefs.h"
 #include "callbacks.h"
 #include "msgwindow.h"
 #include "ui_utils.h"
 #include "utils.h"
 #include "document.h"
+#include "filetypes.h"
 #include "build.h"
+#include "main.h"
+#include "vte.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -114,7 +118,7 @@ static void prepare_status_tree_view(void)
 
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(msgwindow.tree_status), FALSE);
 
-	pfd = pango_font_description_from_string(app->msgwin_font);
+	pfd = pango_font_description_from_string(prefs.msgwin_font);
 	gtk_widget_modify_font(msgwindow.tree_status, pfd);
 	pango_font_description_free(pfd);
 
@@ -132,16 +136,19 @@ static void prepare_msg_tree_view(void)
 	GtkTreeSelection *selection;
 	PangoFontDescription *pfd;
 
-	msgwindow.store_msg = gtk_list_store_new(4, G_TYPE_INT, G_TYPE_INT, GDK_TYPE_COLOR, G_TYPE_STRING);
+	// doc idx, line, bg, fg, str
+	msgwindow.store_msg = gtk_list_store_new(5, G_TYPE_INT, G_TYPE_INT,
+		GDK_TYPE_COLOR, GDK_TYPE_COLOR, G_TYPE_STRING);
 	gtk_tree_view_set_model(GTK_TREE_VIEW(msgwindow.tree_msg), GTK_TREE_MODEL(msgwindow.store_msg));
 
 	renderer = gtk_cell_renderer_text_new();
-	column = gtk_tree_view_column_new_with_attributes(NULL, renderer, "background-gdk", 2, "text", 3, NULL);
+	column = gtk_tree_view_column_new_with_attributes(NULL, renderer,
+		"background-gdk", 2, "foreground-gdk", 3, "text", 4, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(msgwindow.tree_msg), column);
 
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(msgwindow.tree_msg), FALSE);
 
-	pfd = pango_font_description_from_string(app->msgwin_font);
+	pfd = pango_font_description_from_string(prefs.msgwin_font);
 	gtk_widget_modify_font(msgwindow.tree_msg, pfd);
 	pango_font_description_free(pfd);
 
@@ -173,7 +180,7 @@ static void prepare_compiler_tree_view(void)
 
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(msgwindow.tree_compiler), FALSE);
 
-	pfd = pango_font_description_from_string(app->msgwin_font);
+	pfd = pango_font_description_from_string(prefs.msgwin_font);
 	gtk_widget_modify_font(msgwindow.tree_compiler, pfd);
 	pango_font_description_free(pfd);
 
@@ -190,6 +197,22 @@ static void prepare_compiler_tree_view(void)
 
 static const GdkColor color_error = {0, 65535, 0, 0};
 
+static const GdkColor *get_color(gint msg_color)
+{
+	static const GdkColor dark_red = {0, 65535 / 2, 0, 0};
+	static const GdkColor blue = {0, 0, 0, 0xD000};	// not too bright ;-)
+	static const GdkColor black = {0, 0, 0, 0};
+
+	switch (msg_color)
+	{
+		case COLOR_RED: return &color_error;
+		case COLOR_DARK_RED: return &dark_red;
+		case COLOR_BLUE: return &blue;
+		default: return &black;
+	}
+}
+
+
 void msgwin_compiler_add_fmt(gint msg_color, const gchar *format, ...)
 {
 	gchar string[512];
@@ -207,23 +230,12 @@ void msgwin_compiler_add(gint msg_color, const gchar *msg)
 {
 	GtkTreeIter iter;
 	GtkTreePath *path;
-	const GdkColor *color;
-	const GdkColor dark_red = {0, 65535 / 2, 0, 0};
-	const GdkColor blue = {0, 0, 0, 0xD000};	// not too bright ;-)
-	const GdkColor black = {0, 0, 0, 0};
-
-	switch (msg_color)
-	{
-		case COLOR_RED: color = &color_error; break;
-		case COLOR_DARK_RED: color = &dark_red; break;
-		case COLOR_BLUE: color = &blue; break;
-		default: color = &black;
-	}
+	const GdkColor *color = get_color(msg_color);
 
 	gtk_list_store_append(msgwindow.store_compiler, &iter);
 	gtk_list_store_set(msgwindow.store_compiler, &iter, 0, color, 1, msg, -1);
 
-	if (app->msgwindow_visible)
+	if (ui_prefs.msgwindow_visible)
 	{
 		path = gtk_tree_model_get_path(
 			gtk_tree_view_get_model(GTK_TREE_VIEW(msgwindow.tree_compiler)), &iter);
@@ -238,7 +250,7 @@ void msgwin_compiler_add(gint msg_color, const gchar *msg)
 
 void msgwin_show_hide(gboolean show)
 {
-	app->msgwindow_visible = show;
+	ui_prefs.msgwindow_visible = show;
 	app->ignore_callback = TRUE;
 	gtk_check_menu_item_set_active(
 		GTK_CHECK_MENU_ITEM(lookup_widget(app->window, "menu_show_messages_window1")),
@@ -248,7 +260,7 @@ void msgwin_show_hide(gboolean show)
 }
 
 
-void msgwin_msg_add_fmt(gint line, gint idx, const gchar *format, ...)
+void msgwin_msg_add_fmt(gint msg_color, gint line, gint idx, const gchar *format, ...)
 {
 	gchar string[512];
 	va_list args;
@@ -257,26 +269,48 @@ void msgwin_msg_add_fmt(gint line, gint idx, const gchar *format, ...)
 	g_vsnprintf(string, 512, format, args);
 	va_end(args);
 
-	msgwin_msg_add(line, idx, string);
+	msgwin_msg_add(msg_color, line, idx, string);
 }
 
 
 // adds string to the msg treeview
-void msgwin_msg_add(gint line, gint idx, const gchar *string)
+void msgwin_msg_add(gint msg_color, gint line, gint idx, const gchar *string)
 {
 	GtkTreeIter iter;
 	static gint state = 0;
+	const GdkColor *color = get_color(msg_color);
 
-	if (! app->msgwindow_visible) msgwin_show_hide(TRUE);
+	if (! ui_prefs.msgwindow_visible) msgwin_show_hide(TRUE);
 
 	gtk_list_store_append(msgwindow.store_msg, &iter);
 	gtk_list_store_set(msgwindow.store_msg, &iter, 0, line, 1, idx, 2,
-		((state++ % 2) == 0) ? &white : &dark, 3, string, -1);
+		((state++ % 2) == 0) ? &white : &dark, 3, color, 4, string, -1);
 
 	gtk_widget_set_sensitive(lookup_widget(app->window, "next_message1"), TRUE);
 }
 
 
+/* Log a status message *without* setting the status bar.
+ * This is a temporary function for the plugin API for Geany 0.12.
+ * In future, msgwin_status_add() will act like this. */
+void msgwin_status_add_new(const gchar *format, ...)
+{
+	gchar string[512];
+	va_list args;
+	gboolean suppress;
+
+	va_start(args, format);
+	g_vsnprintf(string, 512, format, args);
+	va_end(args);
+
+	// hack to prevent setting the status bar
+	suppress = prefs.suppress_status_messages;
+	prefs.suppress_status_messages = TRUE;
+	msgwin_status_add("%s", string);
+	prefs.suppress_status_messages = suppress;
+}
+
+
 // logs a status message (use ui_set_statusbar() to just display text on the statusbar)
 void msgwin_status_add(const gchar *format, ...)
 {
@@ -291,7 +325,8 @@ void msgwin_status_add(const gchar *format, ...)
 	va_end(args);
 
 	// display status message in status bar
-	ui_set_statusbar("%s", string);
+	if (! prefs.suppress_status_messages)
+		ui_set_statusbar("%s", string);
 
 	// add a timestamp to status messages
 	time_str = utils_get_current_time_string();
@@ -307,12 +342,12 @@ void msgwin_status_add(const gchar *format, ...)
 		((state++ % 2) == 0) ? &white : &dark, 1, statusmsg, -1);
 	g_free(statusmsg);
 
-	if (app->main_window_realized)
+	if (main_status.main_window_realized)
 	{
 		GtkTreePath *path = gtk_tree_model_get_path(gtk_tree_view_get_model(GTK_TREE_VIEW(msgwindow.tree_status)), &iter);
 
 		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(msgwindow.tree_status), path, NULL, FALSE, 0.0, 0.0);
-		if (app->switch_msgwin_pages) gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_STATUS);
+		if (prefs.switch_msgwin_pages) gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_STATUS);
 		gtk_tree_path_free(path);
 	}
 }
@@ -322,24 +357,9 @@ static void
 on_message_treeview_clear_activate     (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	GtkListStore *store;
+	gint tabnum = GPOINTER_TO_INT(user_data);
 
-	switch (GPOINTER_TO_INT(user_data))
-	{
-		case MSG_MESSAGE:
-		gtk_widget_set_sensitive(lookup_widget(app->window, "next_message1"), FALSE);
-		store = msgwindow.store_msg;
-		break;
-
-		case MSG_COMPILER:
-		gtk_widget_set_sensitive(build_get_menu_items(-1)->item_next_error, FALSE);
-		store = msgwindow.store_compiler;
-		break;
-
-		default: // MSG_STATUS
-		store = msgwindow.store_status;
-	}
-	gtk_list_store_clear(store);
+	msgwin_clear_tab(tabnum);
 }
 
 
@@ -510,7 +530,7 @@ gboolean msgwin_goto_compiler_file_line()
 				g_free(utf8_filename);
 
 				if (idx < 0)	// file not already open
-					idx = document_open_file(-1, filename, 0, FALSE, NULL, NULL);
+					idx = document_open_file(filename, FALSE, NULL, NULL);
 
 				if (idx >= 0 && doc_list[idx].is_valid)
 				{
@@ -651,8 +671,10 @@ void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir, gch
 			data.file_idx = 2;
 			break;
 		}
+		case GEANY_FILETYPES_BASIC:
 		case GEANY_FILETYPES_PASCAL:
 		{
+			// getdrive.bas(52) error 18: Syntax error in '? GetAllDrives'
 			// bandit.pas(149,3) Fatal: Syntax error, ";" expected but "ELSE" found
 			data.pattern = "(";
 			data.min_fields = 2;
@@ -729,6 +751,17 @@ void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir, gch
 		case GEANY_FILETYPES_ALL:
 		default:	// The default is a GNU gcc type error
 		{
+			if (build_info.file_type_id == GEANY_FILETYPES_JAVA &&
+				strncmp(string, "[javac]", 7) == 0)
+			{
+				/* Java Apache Ant.
+				 * [javac] <Full Path to File + extension>:<line n°>: <error> */
+				data.pattern = " :";
+				data.min_fields = 4;
+				data.line_idx = 2;
+				data.file_idx = 1;
+				break;
+			}
 			// don't accidently find libtool versions x:y:x and think it is a file name
 			if (strstr(string, "libtool --mode=link") == NULL)
 			{
@@ -759,7 +792,7 @@ gboolean msgwin_goto_messages_file_line()
 		gint idx, line;
 		gchar *string;
 
-		gtk_tree_model_get(model, &iter, 0, &line, 1, &idx, 3, &string, -1);
+		gtk_tree_model_get(model, &iter, 0, &line, 1, &idx, 4, &string, -1);
 		if (line >= 0 && idx >= 0)
 		{
 			ret = utils_goto_line(idx, line);	// checks valid idx
@@ -771,7 +804,7 @@ gboolean msgwin_goto_messages_file_line()
 			if (filename != NULL && line > -1)
 			{
 				// use document_open_file to find an already open file, or open it in place
-				idx = document_open_file(-1, filename, 0, FALSE, NULL, NULL);
+				idx = document_open_file(filename, FALSE, NULL, NULL);
 				// utils_goto_file_line will check valid filename.
 				ret = utils_goto_file_line(filename, FALSE, line);
 			}
@@ -859,3 +892,51 @@ static gboolean on_msgwin_button_press_event(GtkWidget *widget, GdkEventButton *
 }
 
 
+void msgwin_switch_tab(MessageWindowTabNum tabnum, gboolean show)
+{
+	GtkWidget *widget = NULL;	// widget to focus
+
+	switch (tabnum)
+	{
+		case MSG_SCRATCH: widget = lookup_widget(app->window, "textview_scribble"); break;
+#ifdef HAVE_VTE
+		case MSG_VTE: widget = (vte_info.have_vte) ? vc->vte : NULL; break;
+#endif
+		default: break;
+	}
+
+	/* the msgwin must be visible before we switch to the VTE page so that
+	 * the font settings are applied on realization */
+	if (show)
+		msgwin_show_hide(TRUE);
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), tabnum);
+	if (show && widget)
+		gtk_widget_grab_focus(widget);
+}
+
+
+void msgwin_clear_tab(MessageWindowTabNum tabnum)
+{
+	GtkListStore *store = NULL;
+
+	switch (tabnum)
+	{
+		case MSG_MESSAGE:
+			gtk_widget_set_sensitive(lookup_widget(app->window, "next_message1"), FALSE);
+			store = msgwindow.store_msg;
+			break;
+
+		case MSG_COMPILER:
+			gtk_widget_set_sensitive(build_get_menu_items(-1)->item_next_error, FALSE);
+			store = msgwindow.store_compiler;
+			break;
+
+		case MSG_STATUS: store = msgwindow.store_status; break;
+		default: return;
+	}
+	if (store == NULL)
+		return;
+	gtk_list_store_clear(store);
+}
+
+
