@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: msgwindow.h 1269 2007-02-12 17:21:57Z eht16 $
+ * $Id: msgwindow.h 1899 2007-09-24 16:07:44Z ntrel $
  */
 
 
@@ -34,14 +34,14 @@ enum
 	COLOR_BLUE
 };
 
-enum
+typedef enum
 {
 	MSG_STATUS = 0, // force it to start at 0 to keep in sync with the notebook page numbers
 	MSG_COMPILER,
 	MSG_MESSAGE,
 	MSG_SCRATCH,
 	MSG_VTE,
-};
+} MessageWindowTabNum;
 
 
 
@@ -69,7 +69,14 @@ void msgwin_finalize();
 
 void msgwin_show_hide(gboolean show);
 
-void msgwin_msg_add(gint line, gint idx, const gchar *string);
+void msgwin_switch_tab(MessageWindowTabNum tabnum, gboolean show);
+
+void msgwin_clear_tab(MessageWindowTabNum tabnum);
+
+
+void msgwin_msg_add_fmt(gint msg_color, gint line, gint idx, const gchar *format, ...) G_GNUC_PRINTF (4, 5);
+
+void msgwin_msg_add(gint msg_color, gint line, gint idx, const gchar *string);
 
 void msgwin_compiler_add_fmt(gint msg_color, const gchar *format, ...) G_GNUC_PRINTF (2, 3);
 
@@ -77,11 +84,13 @@ void msgwin_compiler_add(gint msg_color, const gchar *msg);
 
 void msgwin_status_add(const gchar *format, ...) G_GNUC_PRINTF (1, 2);
 
+
 void msgwin_menu_add_common_items(GtkMenu *menu);
 
 gboolean msgwin_goto_compiler_file_line();
 
-void msgwin_parse_compiler_error_line(const gchar *string, gchar **filename, gint *line);
+void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir,
+									  gchar **filename, gint *line);
 
 gboolean msgwin_goto_messages_file_line();
 
