@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: notebook.c 1394 2007-03-14 12:11:50Z ntrel $
+ * $Id: notebook.c 1925 2007-10-02 14:23:11Z ntrel $
  */
 
 /*
@@ -27,6 +27,7 @@
 
 #include "geany.h"
 #include "notebook.h"
+#include "prefs.h"
 #include "document.h"
 #include "ui_utils.h"
 #include "treeviews.h"
@@ -303,8 +304,7 @@ static void tab_count_changed()
 /* Returns page number of notebook page, or -1 on error */
 gint notebook_new_tab(gint doc_idx)
 {
-	GtkWidget *hbox, *but;
-	GtkWidget *align;
+	GtkWidget *hbox;
 	gint tabnum;
 	gchar *title;
 	document *this = &(doc_list[doc_idx]);
@@ -317,25 +317,49 @@ gint notebook_new_tab(gint doc_idx)
 
 	this->tab_label = gtk_label_new(title);
 
-	hbox = gtk_hbox_new(FALSE, 0);
-	but = gtk_button_new();
-	gtk_container_add(GTK_CONTAINER(but),
-		ui_new_image_from_inline(GEANY_IMAGE_SMALL_CROSS, FALSE));
-	gtk_container_set_border_width(GTK_CONTAINER(but), 0);
-	gtk_widget_set_size_request(but, 19, 18);
-
-	align = gtk_alignment_new(1.0, 0.0, 0.0, 0.0);
-	gtk_container_add(GTK_CONTAINER(align), but);
-
-	gtk_button_set_relief(GTK_BUTTON(but), GTK_RELIEF_NONE);
+	hbox = gtk_hbox_new(FALSE, 2);
 	gtk_box_pack_start(GTK_BOX(hbox), this->tab_label, FALSE, FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(hbox), align, TRUE, TRUE, 0);
+
+	if (prefs.show_tab_cross)
+	{
+		GtkWidget *image, *btn, *align;
+		GtkRcStyle *rcstyle;
+		GtkRequisition size;
+
+		btn = gtk_button_new();
+		gtk_button_set_relief(GTK_BUTTON(btn), GTK_RELIEF_NONE);
+
+		/* don't allow focus on the close button */
+		gtk_button_set_focus_on_click(GTK_BUTTON(btn), FALSE);
+
+		/* make it as small as possible */
+		rcstyle = gtk_rc_style_new();
+		rcstyle->xthickness = rcstyle->ythickness = 0;
+		gtk_widget_modify_style(btn, rcstyle);
+		gtk_rc_style_unref(rcstyle);
+
+		image = gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
+		gtk_widget_size_request(image, &size);
+		gtk_widget_set_size_request(btn, size.width, size.height);
+		gtk_container_add(GTK_CONTAINER(btn), image);
+
+		align = gtk_alignment_new(1.0, 0.0, 0.0, 0.0);
+		gtk_container_add(GTK_CONTAINER(align), btn);
+
+		gtk_box_pack_start(GTK_BOX(hbox), align, TRUE, TRUE, 0);
+
+		g_signal_connect(G_OBJECT(btn), "clicked",
+			G_CALLBACK(notebook_tab_close_clicked_cb), page);
+	}
+	else
+		gtk_widget_set_size_request(hbox, -1, 18); // keep the familiar tab height
+
 	gtk_widget_show_all(hbox);
 
 	this->tabmenu_label = gtk_label_new(title);
 	gtk_misc_set_alignment(GTK_MISC(this->tabmenu_label), 0.0, 0);
 
-	if (app->tab_order_ltr)
+	if (prefs.tab_order_ltr)
 		tabnum = gtk_notebook_append_page_menu(GTK_NOTEBOOK(app->notebook), page,
 			hbox, this->tabmenu_label);
 	else
@@ -344,10 +368,6 @@ gint notebook_new_tab(gint doc_idx)
 
 	tab_count_changed();
 
-	// signal for clicking the tab-close button
-	g_signal_connect(G_OBJECT(but), "clicked",
-		G_CALLBACK(notebook_tab_close_clicked_cb), page);
-
 	// This is where tab DnD is enabled for GTK 2.10 and higher
 #if GTK_CHECK_VERSION(2, 10, 0)
 	if (gtk_check_version(2, 10, 0) == NULL) // null means version ok
