@@ -1,8 +1,8 @@
 /*
  *      notebook.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
- *      Copyright 2006 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2006-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: notebook.h 748 2006-08-20 11:54:33Z ntrel $
+ * $Id: notebook.h 1188 2007-01-14 17:36:42Z eht16 $
  */
 
 #ifndef GEANY_NOTEBOOK_H
@@ -26,7 +26,10 @@
 
 void notebook_init();
 
-/* Returns index of notebook page, or -1 on error */
-gint notebook_new_tab(gint doc_idx, gchar *title, GtkWidget *page);
+/* Returns page number of notebook page, or -1 on error */
+gint notebook_new_tab(gint doc_idx);
+
+// Always use this instead of gtk_notebook_remove_page().
+void notebook_remove_page(gint page_num);
 
 #endif
