@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: project.c 1535 2007-05-15 15:16:16Z eht16 $
+ * $Id: project.c 1815 2007-08-23 11:34:06Z ntrel $
  */
 
 /*
@@ -124,21 +124,14 @@ void project_new()
 	gtk_table_set_col_spacings(GTK_TABLE(table), 10);
 
 	label = gtk_label_new(_("Name:"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
-					(GtkAttachOptions) (GTK_FILL),
-					(GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0);
 
 	e->name = gtk_entry_new();
 	gtk_entry_set_max_length(GTK_ENTRY(e->name), MAX_NAME_LEN);
-	gtk_table_attach(GTK_TABLE(table), e->name, 1, 2, 0, 1,
-					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-					(GtkAttachOptions) (0), 0, 0);
+
+	ui_table_add_row(GTK_TABLE(table), 0, label, e->name, NULL);
 
 	label = gtk_label_new(_("Filename:"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
-					(GtkAttachOptions) (GTK_FILL),
-					(GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0);
 
 	e->file_name = gtk_entry_new();
@@ -151,14 +144,10 @@ void project_new()
 	bbox = gtk_hbox_new(FALSE, 6);
 	gtk_box_pack_start_defaults(GTK_BOX(bbox), e->file_name);
 	gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 0);
-	gtk_table_attach(GTK_TABLE(table), bbox, 1, 2, 1, 2,
-					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-					(GtkAttachOptions) (0), 0, 0);
+
+	ui_table_add_row(GTK_TABLE(table), 1, label, bbox, NULL);
 
 	label = gtk_label_new(_("Base path:"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
-					(GtkAttachOptions) (GTK_FILL),
-					(GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0);
 
 	e->base_path = gtk_entry_new();
@@ -167,9 +156,8 @@ void project_new()
 		"This can be a new path, or an existing directory tree."), NULL);
 	bbox = ui_path_box_new(_("Choose Project Base Path"),
 		GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, GTK_ENTRY(e->base_path));
-	gtk_table_attach(GTK_TABLE(table), bbox, 1, 2, 2, 3,
-					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-					(GtkAttachOptions) (0), 0, 0);
+
+	ui_table_add_row(GTK_TABLE(table), 2, label, bbox, NULL);
 
 	gtk_container_add(GTK_CONTAINER(vbox), table);
 
@@ -277,6 +265,14 @@ void project_open()
 }
 
 
+// Called when opening, closing and updating projects.
+static void update_ui()
+{
+	ui_set_window_title(-1);
+	build_menu_update(-1);
+}
+
+
 void project_close()
 {
 	g_return_if_fail(app->project != NULL);
@@ -295,7 +291,7 @@ void project_close()
 	g_free(app->project);
 	app->project = NULL;
 
-	build_menu_update(-1);
+	update_ui();
 }
 
 
@@ -474,7 +470,7 @@ void project_properties()
 		if (! update_config(e))
 			goto retry;
 		// successfully updated properties
-		build_menu_update(-1);
+		update_ui();
 	}
 
 	gtk_widget_destroy(e->dialog);
@@ -489,7 +485,7 @@ static gboolean close_open_project()
 {
 	if (app->project != NULL)
 	{
-		if (dialogs_show_question_full(GTK_STOCK_OK, GTK_STOCK_CANCEL,
+		if (dialogs_show_question_full(NULL, GTK_STOCK_OK, GTK_STOCK_CANCEL,
 			_("Do you want to close it before proceeding?"),
 			_("The '%s' project is already open. "), app->project->name))
 		{
@@ -544,7 +540,7 @@ static gboolean update_config(const PropertyDialogElements *e)
 		gchar *locale_path = utils_get_locale_from_utf8(base_path);
 		if (! g_file_test(locale_path, G_FILE_TEST_IS_DIR))
 		{
-			if (dialogs_show_question_full(GTK_STOCK_OK, GTK_STOCK_CANCEL,
+			if (dialogs_show_question_full(NULL, GTK_STOCK_OK, GTK_STOCK_CANCEL,
 				_("Create the project's base path directory?"),
 				_("The path \"%s\" does not exist."),
 				base_path))
@@ -807,7 +803,7 @@ static gboolean load_config(const gchar *filename)
 
 	g_key_file_free(config);
 
-	build_menu_update(-1);
+	update_ui();
 	return TRUE;
 }
 
@@ -898,8 +894,8 @@ void project_load_prefs(GKeyFile *config)
 /* Initialize project-related preferences in the Preferences dialog. */
 void project_setup_prefs()
 {
-	GtkWidget *path_entry = lookup_widget(app->prefs_dialog, "project_file_path_entry");
-	GtkWidget *path_btn = lookup_widget(app->prefs_dialog, "project_file_path_button");
+	GtkWidget *path_entry = lookup_widget(ui_widgets.prefs_dialog, "project_file_path_entry");
+	GtkWidget *path_btn = lookup_widget(ui_widgets.prefs_dialog, "project_file_path_button");
 
 	g_return_if_fail(local_prefs.project_file_path != NULL);
 	gtk_entry_set_text(GTK_ENTRY(path_entry), local_prefs.project_file_path);
@@ -911,7 +907,7 @@ void project_setup_prefs()
 /* Update project-related preferences after using the Preferences dialog. */
 void project_apply_prefs()
 {
-	GtkWidget *path_entry = lookup_widget(app->prefs_dialog, "project_file_path_entry");
+	GtkWidget *path_entry = lookup_widget(ui_widgets.prefs_dialog, "project_file_path_entry");
 	const gchar *str;
 
 	str = gtk_entry_get_text(GTK_ENTRY(path_entry));
