@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: sciwrappers.c 1514 2007-05-08 16:03:07Z eht16 $
+ * $Id: sciwrappers.c 1904 2007-09-25 16:44:33Z ntrel $
  */
 
 /*
@@ -151,11 +151,10 @@ gboolean sci_get_visible_eols(ScintillaObject* sci)
 
 void sci_set_visible_white_spaces(ScintillaObject* sci, gboolean set )
 {
-	if(set){
+	if (set)
 		SSM(sci,SCI_SETVIEWWS,SCWS_VISIBLEALWAYS,0);
-	}else{
+	else
 		SSM(sci,SCI_SETVIEWWS,SCWS_INVISIBLE,0);
-	}
 }
 
 gboolean sci_get_visible_white_spaces(ScintillaObject* sci)
@@ -165,12 +164,13 @@ gboolean sci_get_visible_white_spaces(ScintillaObject* sci)
 
 void sci_set_lines_wrapped(ScintillaObject* sci, gboolean set )
 {
-	if( set ){
+	if (set)
+	{
 		SSM(sci,SCI_SETWRAPMODE,SC_WRAP_WORD,0);
 		SSM(sci, SCI_SETWRAPVISUALFLAGS, SC_WRAPVISUALFLAG_END, 0);
-	}else{
-		SSM(sci,SCI_SETWRAPMODE,SC_WRAP_NONE,0);
 	}
+	else
+		SSM(sci,SCI_SETWRAPMODE,SC_WRAP_NONE,0);
 }
 
 gboolean sci_get_lines_wrapped(ScintillaObject* sci)
@@ -231,9 +231,10 @@ gboolean sci_can_redo( ScintillaObject* sci )
 
 void sci_undo( ScintillaObject* sci )
 {
-	if( sci_can_undo(sci) ){
+	if( sci_can_undo(sci) )
 		SSM( sci, SCI_UNDO, 0, 0);
-	}else{ // change it to a document function
+	else
+	{ // change it to a document function
 
 	}
 }
@@ -241,9 +242,10 @@ void sci_undo( ScintillaObject* sci )
 
 void sci_redo( ScintillaObject* sci )
 {
-	if( sci_can_redo( sci ) ){
+	if( sci_can_redo( sci ) )
 		SSM( sci, SCI_REDO,0,0);
-	}else{// change it to a document function
+	else
+	{ // change it to a document function
 
 	}
 }
@@ -466,7 +468,7 @@ gint sci_get_selection_end(ScintillaObject* sci)
 }
 
 
-void sci_replace_sel(ScintillaObject* sci, gchar* text)
+void sci_replace_sel(ScintillaObject* sci, const gchar* text)
 {
 	SSM(sci, SCI_REPLACESEL,0, (sptr_t) text);
 }
@@ -509,6 +511,8 @@ void sci_get_text(ScintillaObject* sci, gint len, gchar* text)
 }
 
 
+/* Text must be allocated sci_get_selected_text_length() + 1, because
+ * the selection will be NULL-terminated. */
 void sci_get_selected_text(ScintillaObject* sci, gchar* text)
 {
 	SSM( sci, SCI_GETSELTEXT, 0, (sptr_t) text);
@@ -604,12 +608,6 @@ gint sci_get_end_styled(ScintillaObject * sci)
 }
 
 
-gint sci_get_line_end_styled(ScintillaObject * sci, gint end_styled)
-{
-	return SSM(sci,SCI_LINEFROMPOSITION, end_styled,0);
-}
-
-
 void sci_set_tab_width(ScintillaObject * sci, gint width)
 {
 	SSM(sci, SCI_SETTABWIDTH, width, 0);
@@ -634,7 +632,7 @@ void sci_set_savepoint(ScintillaObject *sci)
 }
 
 
-void sci_set_indentionguides(ScintillaObject *sci, gboolean enable)
+void sci_set_indentation_guides(ScintillaObject *sci, gboolean enable)
 {
 	SSM(sci, SCI_SETINDENTATIONGUIDES, enable, 0);
 }
@@ -836,16 +834,9 @@ gboolean sci_get_readonly(ScintillaObject *sci)
 }
 
 
-gint sci_get_current_line(ScintillaObject *sci, gint pos)
+gint sci_get_current_line(ScintillaObject *sci)
 {
-	if (pos >= 0)
-	{
-		return SSM(sci, SCI_LINEFROMPOSITION, pos, 0);
-	}
-	else
-	{
-		return SSM(sci, SCI_LINEFROMPOSITION, SSM(sci, SCI_GETCURRENTPOS, 0, 0), 0);
-	}
+	return SSM(sci, SCI_LINEFROMPOSITION, SSM(sci, SCI_GETCURRENTPOS, 0, 0), 0);
 }
 
 /* Get number of lines partially or fully selected.
@@ -951,3 +942,13 @@ int sci_get_line_indentation(ScintillaObject *sci, gint line)
 	return SSM(sci, SCI_GETLINEINDENTATION, line, 0);
 }
 
+void sci_set_caret_policy_x(ScintillaObject *sci, gint policy, gint slop)
+{
+	SSM(sci, SCI_SETXCARETPOLICY, policy, slop);
+}
+
+void sci_set_caret_policy_y(ScintillaObject *sci, gint policy, gint slop)
+{
+	SSM(sci, SCI_SETYCARETPOLICY, policy, slop);
+}
+
