@@ -18,16 +18,13 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: sciwrappers.h 1514 2007-05-08 16:03:07Z eht16 $
+ * $Id: sciwrappers.h 1904 2007-09-25 16:44:33Z ntrel $
  */
 
 #ifndef GEANY_SCIWRAPPERS_H
 #define GEANY_SCIWRAPPERS_H 1
 
 #include "Scintilla.h"
-#ifndef PLAT_GTK
-#   define PLAT_GTK 1
-#endif
 #include "ScintillaWidget.h"
 
 
@@ -79,7 +76,7 @@ void 				sci_clear					(ScintillaObject* sci);
 
 gint				sci_get_selection_start		(ScintillaObject* sci);
 gint				sci_get_selection_end		(ScintillaObject* sci);
-void 				sci_replace_sel				(ScintillaObject* sci, gchar* text);
+void 				sci_replace_sel				(ScintillaObject* sci, const gchar* text);
 gint				sci_get_selection_mode		(ScintillaObject* sci);
 void				sci_set_selection_mode		(ScintillaObject* sci, gint mode);
 gint				sci_get_pos_at_line_sel_start(ScintillaObject*sci, gint line);
@@ -117,12 +114,11 @@ void				sci_colourise				(ScintillaObject* sci, gint start, gint end);
 void				sci_set_lexer				(ScintillaObject * sci, gint lexer);
 void				sci_clear_all				(ScintillaObject * sci);
 gint				sci_get_end_styled			(ScintillaObject * sci);
-gint				sci_get_line_end_styled		(ScintillaObject * sci, gint end_styled);
 void				sci_set_tab_width			(ScintillaObject * sci, gint width);
 gint				sci_get_tab_width			(ScintillaObject * sci);
 gchar				sci_get_char_at				(ScintillaObject * sci, gint pos);
 void				sci_set_savepoint			(ScintillaObject * sci);
-void				sci_set_indentionguides		(ScintillaObject * sci, gboolean enable);
+void				sci_set_indentation_guides	(ScintillaObject * sci, gboolean enable);
 void				sci_use_popup				(ScintillaObject * sci, gboolean enable);
 void				sci_goto_pos				(ScintillaObject * sci, gint pos, gboolean unfold);
 void				sci_set_search_anchor		(ScintillaObject * sci);
@@ -159,7 +155,7 @@ gboolean			sci_get_readonly			(ScintillaObject * sci);
 gint				sci_get_zoom				(ScintillaObject * sci);
 void				sci_cmd						(ScintillaObject * sci, gint cmd);
 
-gint				sci_get_current_line		(ScintillaObject * sci, gint pos);
+gint				sci_get_current_line		(ScintillaObject * sci);
 gint				sci_get_lines_selected		(ScintillaObject * sci);
 gint				sci_get_first_visible_line	(ScintillaObject * sci);
 
@@ -177,5 +173,7 @@ void				sci_set_tab_indents			(ScintillaObject * sci, gboolean set);
 void				sci_set_use_tabs			(ScintillaObject * sci, gboolean set);
 
 void				sci_set_scrollbar_mode		(ScintillaObject * sci, gboolean visible);
+void				sci_set_caret_policy_x		(ScintillaObject * sci, gint policy, gint slop);
+void				sci_set_caret_policy_y		(ScintillaObject * sci, gint policy, gint slop);
 
 #endif
