@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: search.c 1541 2007-05-16 17:09:02Z eht16 $
+ * $Id: search.c 1887 2007-09-17 16:33:17Z ntrel $
  */
 
 /*
@@ -30,13 +30,14 @@
 
 #include "geany.h"
 #include "search.h"
+#include "prefs.h"
 #include "support.h"
 #include "utils.h"
 #include "msgwindow.h"
 #include "document.h"
 #include "sciwrappers.h"
 #include "ui_utils.h"
-#include "sci_cb.h"
+#include "editor.h"
 
 #include <unistd.h>
 #include <string.h>
@@ -145,7 +146,7 @@ static GtkWidget *add_find_checkboxes(GtkDialog *dialog)
 
 	if (dialog != GTK_DIALOG(widgets.find_dialog))
 	{
-		check_back = gtk_check_button_new_with_mnemonic(_("_Search backwards"));
+		check_back = gtk_check_button_new_with_mnemonic(_("Search _backwards"));
 		g_object_set_data_full(G_OBJECT(dialog), "check_back",
 						gtk_widget_ref(check_back), (GDestroyNotify)gtk_widget_unref);
 		gtk_button_set_focus_on_click(GTK_BUTTON(check_back), FALSE);
@@ -173,7 +174,7 @@ static GtkWidget *add_find_checkboxes(GtkDialog *dialog)
 	gtk_container_add(GTK_CONTAINER(fbox), checkbox7);
 	gtk_container_add(GTK_CONTAINER(fbox), check_back);
 
-	checkbox1 = gtk_check_button_new_with_mnemonic(_("_Case sensitive"));
+	checkbox1 = gtk_check_button_new_with_mnemonic(_("C_ase sensitive"));
 	g_object_set_data_full(G_OBJECT(dialog), "check_case",
 					gtk_widget_ref(checkbox1), (GDestroyNotify)gtk_widget_unref);
 	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox1), FALSE);
@@ -213,14 +214,16 @@ static gchar *get_default_text(gint idx)
 
 	if (sci_get_lines_selected(doc_list[idx].sci) == 1)
 	{
-		s = g_malloc(sci_get_selected_text_length(doc_list[idx].sci));
+		gint len = sci_get_selected_text_length(doc_list[idx].sci);
+
+		s = g_malloc(len + 1);
 		sci_get_selected_text(doc_list[idx].sci, s);
 	}
 	else if (sci_get_lines_selected(doc_list[idx].sci) == 0)
 	{	// use the word at current cursor position
 		gchar word[GEANY_MAX_WORD_LENGTH];
 
-		sci_cb_find_current_word(doc_list[idx].sci, -1, word, sizeof(word), NULL);
+		editor_find_current_word(doc_list[idx].sci, -1, word, sizeof(word), NULL);
 		if (word[0] != '\0') s = g_strdup(word);
 	}
 	return s;
@@ -234,6 +237,7 @@ static void setup_find_next(const gchar *text)
 	search_data.text = g_strdup(text);
 	search_data.flags = 0;
 	search_data.backwards = FALSE;
+	search_data.search_bar = FALSE;
 }
 
 
@@ -269,7 +273,7 @@ void search_find_selection(gint idx, gboolean search_backwards)
 	if (s)
 	{
 		setup_find_next(s);	// allow find next/prev
-		document_find_text(idx, s, 0, search_backwards, TRUE);
+		document_find_text(idx, s, 0, search_backwards, TRUE, NULL);
 		g_free(s);
 	}
 }
@@ -307,10 +311,11 @@ void search_show_find_dialog()
 		gtk_dialog_add_action_widget(GTK_DIALOG(widgets.find_dialog), button,
 			GEANY_RESPONSE_FIND);
 
-		label = gtk_label_new(_("Search for:"));
+		label = gtk_label_new_with_mnemonic(_("_Search for:"));
 		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
 		entry = gtk_combo_box_entry_new_text();
+		gtk_label_set_mnemonic_widget(GTK_LABEL(label), entry);
 		gtk_entry_set_max_length(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry))), 248);
 		gtk_entry_set_width_chars(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry))), 50);
 		if (sel) gtk_entry_set_text(GTK_ENTRY(GTK_BIN(entry)->child), sel);
@@ -422,17 +427,18 @@ void search_show_replace_dialog()
 		button = gtk_button_new_with_mnemonic(_("_Replace"));
 		gtk_dialog_add_action_widget(GTK_DIALOG(widgets.replace_dialog), button,
 			GEANY_RESPONSE_REPLACE);
-		button = gtk_button_new_with_mnemonic(_("Re_place & Find"));
+		button = gtk_button_new_with_mnemonic(_("Replace & Fi_nd"));
 		gtk_dialog_add_action_widget(GTK_DIALOG(widgets.replace_dialog), button,
 			GEANY_RESPONSE_REPLACE_AND_FIND);
 
-		label_find = gtk_label_new(_("Search for:"));
+		label_find = gtk_label_new_with_mnemonic(_("_Search for:"));
 		gtk_misc_set_alignment(GTK_MISC(label_find), 0, 0.5);
 
-		label_replace = gtk_label_new(_("Replace with:"));
+		label_replace = gtk_label_new_with_mnemonic(_("Re_place with:"));
 		gtk_misc_set_alignment(GTK_MISC(label_replace), 0, 0.5);
 
 		entry_find = gtk_combo_box_entry_new_text();
+		gtk_label_set_mnemonic_widget(GTK_LABEL(label_find), entry_find);
 		gtk_entry_set_max_length(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_find))), 248);
 		gtk_entry_set_width_chars(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_find))), 50);
 		if (sel) gtk_entry_set_text(GTK_ENTRY(GTK_BIN(entry_find)->child), sel);
@@ -440,6 +446,7 @@ void search_show_replace_dialog()
 						gtk_widget_ref(entry_find), (GDestroyNotify)gtk_widget_unref);
 
 		entry_replace = gtk_combo_box_entry_new_text();
+		gtk_label_set_mnemonic_widget(GTK_LABEL(label_replace), entry_replace);
 		gtk_entry_set_max_length(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_replace))), 248);
 		gtk_entry_set_width_chars(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_replace))), 50);
 		g_object_set_data_full(G_OBJECT(widgets.replace_dialog), "entry_replace",
@@ -556,7 +563,7 @@ void search_show_find_in_files_dialog()
 		GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(app->window, "tooltips"));
 
 		widgets.find_in_files_dialog = gtk_dialog_new_with_buttons(
-			_("Find in files"), GTK_WINDOW(app->window), GTK_DIALOG_DESTROY_WITH_PARENT,
+			_("Find in Files"), GTK_WINDOW(app->window), GTK_DIALOG_DESTROY_WITH_PARENT,
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
 		vbox = ui_dialog_vbox_new(GTK_DIALOG(widgets.find_in_files_dialog));
 		gtk_box_set_spacing(GTK_BOX(vbox), 9);
@@ -566,11 +573,12 @@ void search_show_find_in_files_dialog()
 		gtk_dialog_set_default_response(GTK_DIALOG(widgets.find_in_files_dialog),
 			GTK_RESPONSE_ACCEPT);
 
-		label1 = gtk_label_new(_("Directory:"));
+		label1 = gtk_label_new_with_mnemonic(_("_Directory:"));
 		gtk_misc_set_alignment(GTK_MISC(label1), 0, 0.5);
 
 		dir_combo = gtk_combo_box_entry_new_text();
 		entry = gtk_bin_get_child(GTK_BIN(dir_combo));
+		gtk_label_set_mnemonic_widget(GTK_LABEL(label1), entry);
 		gtk_entry_set_max_length(GTK_ENTRY(entry), 248);
 		gtk_entry_set_width_chars(GTK_ENTRY(entry), 50);
 		g_object_set_data_full(G_OBJECT(widgets.find_in_files_dialog), "dir_combo",
@@ -580,11 +588,12 @@ void search_show_find_in_files_dialog()
 			GTK_ENTRY(entry));
 		gtk_box_pack_start(GTK_BOX(dbox), label1, FALSE, FALSE, 0);
 
-		label = gtk_label_new(_("Search for:"));
+		label = gtk_label_new_with_mnemonic(_("_Search for:"));
 		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
 		combo = gtk_combo_box_entry_new_text();
 		entry = gtk_bin_get_child(GTK_BIN(combo));
+		gtk_label_set_mnemonic_widget(GTK_LABEL(label), entry);
 		gtk_entry_set_max_length(GTK_ENTRY(entry), 248);
 		gtk_entry_set_width_chars(GTK_ENTRY(entry), 50);
 		gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);
@@ -628,7 +637,7 @@ void search_show_find_in_files_dialog()
 						gtk_widget_ref(check_recursive), (GDestroyNotify)gtk_widget_unref);
 		gtk_button_set_focus_on_click(GTK_BUTTON(check_recursive), FALSE);
 
-		checkbox1 = gtk_check_button_new_with_mnemonic(_("_Case sensitive"));
+		checkbox1 = gtk_check_button_new_with_mnemonic(_("C_ase sensitive"));
 		g_object_set_data_full(G_OBJECT(widgets.find_in_files_dialog), "check_case",
 						gtk_widget_ref(checkbox1), (GDestroyNotify)gtk_widget_unref);
 		gtk_button_set_focus_on_click(GTK_BUTTON(checkbox1), FALSE);
@@ -819,6 +828,7 @@ on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		search_replace_escape = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
 						lookup_widget(GTK_WIDGET(widgets.find_dialog), "check_escape")));
 		search_data.backwards = FALSE;
+		search_data.search_bar = FALSE;
 
 		if (! DOC_IDX_VALID(idx)) return;
 
@@ -843,9 +853,9 @@ on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 			case GEANY_RESPONSE_FIND:
 			case GEANY_RESPONSE_FIND_PREVIOUS:
 				document_find_text(idx, search_data.text, search_data.flags,
-					(response == GEANY_RESPONSE_FIND_PREVIOUS), TRUE);
+					(response == GEANY_RESPONSE_FIND_PREVIOUS), TRUE, GTK_WIDGET(widgets.find_dialog));
 				check_close = FALSE;
-				if (app->pref_main_suppress_search_dialogs)
+				if (prefs.suppress_search_dialogs)
 					check_close = TRUE;
 				break;
 
@@ -863,9 +873,9 @@ on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 					gint count = search_mark(idx, search_data.text, search_data.flags);
 
 					if (count == 0)
-						ui_set_statusbar(_("No matches found for '%s'."), search_data.text);
+						ui_set_statusbar(_("No matches found for \"%s\"."), search_data.text);
 					else
-						ui_set_statusbar(_("Found %d matches for '%s'."), count,
+						ui_set_statusbar(_("Found %d matches for \"%s\"."), count,
 							search_data.text);
 				}
 				break;
@@ -949,7 +959,7 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 				search_backwards_re);
 			if (rep != -1)
 				document_find_text(idx, find, search_flags_re, search_backwards_re,
-					TRUE);
+					TRUE, NULL);
 			break;
 		}
 		case GEANY_RESPONSE_REPLACE:
@@ -960,7 +970,8 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		}
 		case GEANY_RESPONSE_FIND:
 		{
-			document_find_text(idx, find, search_flags_re, search_backwards_re, TRUE);
+			document_find_text(idx, find, search_flags_re, search_backwards_re, TRUE,
+							   GTK_WIDGET(dialog));
 			break;
 		}
 		case GEANY_RESPONSE_REPLACE_IN_FILE:
@@ -1121,10 +1132,10 @@ search_find_in_files(const gchar *search_text, const gchar *dir, const gchar *op
 
 	if (! search_text || ! *search_text || ! dir) return TRUE;
 
-	if (! g_file_test(app->tools_grep_cmd, G_FILE_TEST_IS_EXECUTABLE))
+	if (! g_file_test(prefs.tools_grep_cmd, G_FILE_TEST_IS_EXECUTABLE))
 	{
 		msgwin_status_add(_("Cannot execute grep tool '%s';"
-			" check the path setting in Preferences."), app->tools_grep_cmd);
+			" check the path setting in Preferences."), prefs.tools_grep_cmd);
 		return FALSE;
 	}
 
@@ -1134,14 +1145,14 @@ search_find_in_files(const gchar *search_text, const gchar *dir, const gchar *op
 	// set grep command and options
 	argv_prefix = g_new0(gchar*, 1 + opts_argv_len + 3 + 1);	// last +1 for recursive arg
 
-	argv_prefix[0] = g_strdup(app->tools_grep_cmd);
+	argv_prefix[0] = g_strdup(prefs.tools_grep_cmd);
 	for (i = 0; i < opts_argv_len; i++)
 	{
 		argv_prefix[i + 1] = g_strdup(opts_argv[i]);
 	}
 	g_strfreev(opts_argv);
 
-	i++;	// correct for app->tools_grep_cmd
+	i++;	// correct for prefs.tools_grep_cmd
 	argv_prefix[i++] = g_strdup("--");
 	argv_prefix[i++] = g_strdup(search_text);
 
@@ -1189,9 +1200,9 @@ search_find_in_files(const gchar *search_text, const gchar *dir, const gchar *op
 		g_child_watch_add(child_pid, search_close_pid, NULL);
 
 		str = g_strdup_printf(_("%s %s -- %s (in directory: %s)"),
-			app->tools_grep_cmd, opts, search_text, dir);
+			prefs.tools_grep_cmd, opts, search_text, dir);
 		utf8_str = utils_get_utf8_from_locale(str);
-		msgwin_msg_add(-1, -1, utf8_str);
+		msgwin_msg_add(COLOR_BLUE, -1, -1, utf8_str);
 		utils_free_pointers(str, utf8_str, NULL);
 		ret = TRUE;
 	}
@@ -1250,7 +1261,7 @@ static gboolean search_read_io              (GIOChannel *source,
 
 		while (g_io_channel_read_line(source, &msg, NULL, NULL, NULL) && msg)
 		{
-			msgwin_msg_add(-1, -1, g_strstrip(msg));
+			msgwin_msg_add(COLOR_BLACK, -1, -1, g_strstrip(msg));
 			g_free(msg);
 		}
 	}
@@ -1265,6 +1276,7 @@ static void search_close_pid(GPid child_pid, gint status, gpointer user_data)
 {
 #ifdef G_OS_UNIX
 	const gchar *msg = _("Search failed.");
+	gint color = COLOR_DARK_RED;
 
 	if (WIFEXITED(status))
 	{
@@ -1275,14 +1287,15 @@ static void search_close_pid(GPid child_pid, gint status, gpointer user_data)
 				gint count = gtk_tree_model_iter_n_children(
 					GTK_TREE_MODEL(msgwindow.store_msg), NULL) - 1;
 
-				msgwin_msg_add_fmt(-1, -1, _("Search completed with %d matches."), count);
+				msgwin_msg_add_fmt(COLOR_BLUE, -1, -1, _("Search completed with %d matches."), count);
 				ui_set_statusbar(_("Search completed with %d matches."), count);
 				break;
 			}
 			case 1:
 				msg = _("No matches found.");
+				color = COLOR_BLUE;
 			default:
-				msgwin_msg_add(-1, -1, msg);
+				msgwin_msg_add(color, -1, -1, msg);
 				ui_set_statusbar("%s", msg);
 				break;
 		}
@@ -1297,35 +1310,38 @@ static void search_close_pid(GPid child_pid, gint status, gpointer user_data)
 static gint find_document_usage(gint idx, const gchar *search_text, gint flags)
 {
 	gchar *buffer, *short_file_name;
-	gint pos, line, count = 0;
 	struct TextToFind ttf;
+	gint count = 0;
 
 	g_return_val_if_fail(DOC_IDX_VALID(idx), 0);
 
+	short_file_name = g_path_get_basename(DOC_FILENAME(idx));
+
 	ttf.chrg.cpMin = 0;
 	ttf.chrg.cpMax = sci_get_length(doc_list[idx].sci);
 	ttf.lpstrText = (gchar *)search_text;
 	while (1)
 	{
-		pos = sci_find_text(doc_list[idx].sci, flags, &ttf);
-		if (pos == -1) break;
+		gint pos, line, start, find_len;
 
+		pos = sci_find_text(doc_list[idx].sci, flags, &ttf);
+		if (pos == -1)
+			break;	// no more matches
+		find_len = ttf.chrgText.cpMax - ttf.chrgText.cpMin;
+		if (find_len == 0)
+			break;	// Ignore regex ^ or $
+
+		count++;
 		line = sci_get_line_from_position(doc_list[idx].sci, pos);
 		buffer = sci_get_line(doc_list[idx].sci, line);
-
-		if (doc_list[idx].file_name == NULL)
-			short_file_name = g_strdup(GEANY_STRING_UNTITLED);
-		else
-			short_file_name = g_path_get_basename(doc_list[idx].file_name);
-
-		msgwin_msg_add_fmt(line + 1, idx,
+		msgwin_msg_add_fmt(COLOR_BLACK, line + 1, idx,
 			"%s:%d : %s", short_file_name, line + 1, g_strstrip(buffer));
-
 		g_free(buffer);
-		g_free(short_file_name);
-		ttf.chrg.cpMin = ttf.chrgText.cpMax + 1;
-		count++;
+
+		start = ttf.chrgText.cpMax + 1;
+		ttf.chrg.cpMin = start;
 	}
+	g_free(short_file_name);
 	return count;
 }
 
@@ -1357,15 +1373,16 @@ void search_find_usage(const gchar *search_text, gint flags, gboolean in_session
 
 	if (! found) // no matches were found
 	{
-		ui_set_statusbar(_("No matches found for '%s'."), search_text);
-		msgwin_msg_add_fmt(-1, -1, _("No matches found for '%s'."), search_text);
+		ui_set_statusbar(_("No matches found for \"%s\"."), search_text);
+		msgwin_msg_add_fmt(COLOR_BLUE, -1, -1, _("No matches found for \"%s\"."), search_text);
 	}
 	else
 	{
 		gint count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(msgwindow.store_msg), NULL);
 
-		ui_set_statusbar(_("Found %d matches for '%s'."), count, search_text);
-		msgwin_msg_add_fmt(-1, -1, _("Found %d matches for '%s'."), count, search_text);
+		ui_set_statusbar(_("Found %d matches for \"%s\"."), count, search_text);
+		msgwin_msg_add_fmt(COLOR_BLUE, -1, -1, _("Found %d matches for \"%s\"."), count,
+			search_text);
 	}
 }
 
