@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: socket.c 1369 2007-03-06 16:57:09Z ntrel $
+ * $Id: socket.c 1868 2007-09-11 15:21:11Z ntrel $
  */
 
 /*
@@ -157,9 +157,13 @@ gint socket_init(gint argc, gchar **argv)
 
 #ifdef G_OS_WIN32
 	HANDLE hmutex;
+	// we need a mutex name which is unique for the used configuration dir,
+	// but we can't use the whole path, so build a hash on the configdir and use it
+	gchar *mutex_name = g_strdup_printf("Geany%d", g_str_hash(app->configdir));
 
 	socket_init_win32();
-	hmutex = CreateMutexA(NULL, FALSE, "Geany");
+	hmutex = CreateMutexA(NULL, FALSE, mutex_name);
+	g_free(mutex_name);
 	if (! hmutex)
 	{
 		geany_debug("cannot create Mutex\n");
@@ -416,12 +420,12 @@ gboolean socket_lock_input_cb(GIOChannel *source, GIOCondition condition, gpoint
 				g_strstrip(buf); // remove \n char
 
 				if (g_file_test(buf, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
-					document_open_file(-1, buf, 0, FALSE, NULL, NULL);
+					document_open_file(buf, FALSE, NULL, NULL);
 				else
 				{	// create new file if it doesn't exist
 					gint idx;
 
-					idx = document_new_file(buf, NULL);
+					idx = document_new_file(buf, NULL, NULL);
 					if (DOC_IDX_VALID(idx))
 						ui_add_recent_file(doc_list[idx].file_name);
 					else
