@@ -18,38 +18,36 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: symbols.h 1463 2007-04-18 15:21:33Z ntrel $
+ * $Id: symbols.h 1934 2007-10-09 12:04:15Z eht16 $
  */
 
 
 #ifndef GEANY_SYMBOLS_H
 #define GEANY_SYMBOLS_H 1
 
-#include "filetypes.h"
-
 extern const guint TM_GLOBAL_TYPE_MASK;
 
 
 void symbols_global_tags_loaded(gint file_type_idx);
 
-GString *symbols_find_tags_as_string(GPtrArray *tags_array, guint tag_types);
+GString *symbols_find_tags_as_string(GPtrArray *tags_array, guint tag_types, gint lang);
 
-const gchar *symbols_get_context_separator(filetype_id ft_id);
+const gchar *symbols_get_context_separator(gint ft_id);
 
 const GList *symbols_get_tag_list(gint idx, guint tag_types);
 
 GString *symbols_get_macro_list();
 
-TMTag *symbols_find_in_workspace(const gchar *tag_name, gint type);
-
 const gchar **symbols_get_html_entities();
 
 void symbols_finalize();
 
-gboolean symbols_recreate_tag_list(gint idx);
+gboolean symbols_recreate_tag_list(gint idx, gboolean sort_by_name);
 
-int symbols_generate_global_tags(int argc, char **argv);
+gint symbols_generate_global_tags(gint argc, gchar **argv, gboolean want_preprocess);
 
 void symbols_show_load_tags_dialog();
 
+gboolean symbols_goto_tag(const gchar *name, gboolean definition);
+
 #endif
