@@ -1,7 +1,8 @@
 /*
- *      treeviws.h
+ *      treeviews.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: treeviews.h 1053 2006-12-05 10:37:36Z ntrel $
+ * $Id: treeviews.h 1815 2007-08-23 11:34:06Z ntrel $
  */
 
 
@@ -26,33 +27,27 @@
 #define GEANY_TREEVIEWS_H 1
 
 
-
-typedef struct treeviews {
-	GtkListStore	*store_openfiles;
+typedef struct SidebarTreeviews
+{
 	GtkWidget		*tree_openfiles;
+	GtkWidget		*default_tag_tree;
 	GtkWidget		*popup_taglist;
 	GtkWidget		*popup_openfiles;
-	GtkTreeIter		 tag_function;
-	GtkTreeIter		 tag_class;
-	GtkTreeIter		 tag_macro;
-	GtkTreeIter		 tag_member;
-	GtkTreeIter		 tag_variable;
-	GtkTreeIter		 tag_namespace;
-	GtkTreeIter		 tag_struct;
-	GtkTreeIter		 tag_other;
-} treeviews;
+} SidebarTreeviews;
 
-treeviews tv;
+extern SidebarTreeviews tv;
 
+enum
+{
+	SYMBOLS_COLUMN_ICON,
+	SYMBOLS_COLUMN_NAME,
+	SYMBOLS_COLUMN_LINE,
+	SYMBOLS_N_COLUMNS,
+};
 
+void treeviews_init();
 
-void treeviews_prepare_taglist(GtkWidget *tree, GtkTreeStore *store);
-
-gint treeviews_sort_tag_list(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer user_data);
-
-void treeviews_init_tag_list(gint idx);
-
-void treeviews_prepare_openfiles();
+void treeviews_update_tag_list(gint idx, gboolean update);
 
 void treeviews_openfiles_add(gint idx);
 
@@ -60,15 +55,8 @@ void treeviews_openfiles_update(gint idx);
 
 void treeviews_openfiles_update_all();
 
+void treeviews_select_openfiles_item(gint idx);
+
 void treeviews_remove_document(gint idx);
 
-void treeviews_create_openfiles_popup_menu();
-
-void treeviews_create_taglist_popup_menu();
-
-/* compares the given data (GINT_TO_PONTER(idx)) with the idx from the selected row of openfiles
- * treeview, in case of a match the row is selected and TRUE is returned
- * (called indirectly from gtk_tree_model_foreach()) */
-gboolean treeviews_find_node(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data);
-
 #endif
