@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: utils.c 1528 2007-05-11 17:31:06Z ntrel $
+ * $Id: utils.c 1904 2007-09-25 16:44:33Z ntrel $
  */
 
 /*
@@ -45,8 +45,10 @@
 
 #include <glib/gstdio.h>
 
+#include "prefs.h"
 #include "support.h"
 #include "document.h"
+#include "filetypes.h"
 #include "sciwrappers.h"
 #include "dialogs.h"
 #include "win32.h"
@@ -61,7 +63,7 @@ void utils_start_browser(const gchar *uri)
 #else
 	const gchar *argv[3];
 
-	argv[0] = app->tools_browser_cmd;
+	argv[0] = prefs.tools_browser_cmd;
 	argv[1] = uri;
 	argv[2] = NULL;
 
@@ -101,7 +103,8 @@ gint utils_get_line_endings(gchar* buffer, glong size)
 
 	for ( i = 0; i < size ; i++ )
 	{
-		if ( buffer[i] == 0x0a ){
+		if ( buffer[i] == 0x0a )
+		{
 			// LF
 			lf++;
 		}
@@ -112,7 +115,9 @@ gint utils_get_line_endings(gchar* buffer, glong size)
 				// Last char
 				// CR
 				cr++;
-			} else {
+			}
+			else
+			{
 				if (buffer[i+1] != 0x0a)
 				{
 					// CR
@@ -131,29 +136,29 @@ gint utils_get_line_endings(gchar* buffer, glong size)
 	/* Vote for the maximum */
 	mode = SC_EOL_LF;
 	max_mode = lf;
-	if (crlf > max_mode) {
+	if (crlf > max_mode)
+	{
 		mode = SC_EOL_CRLF;
 		max_mode = crlf;
 	}
-	if (cr > max_mode) {
+	if (cr > max_mode)
+	{
 		mode = SC_EOL_CR;
 		max_mode = cr;
 	}
 	//geany_debug("EOL chars: LF = %d, CR = %d, CRLF = %d", lf, cr, crlf);
 
-
 	return mode;
 }
 
 
-gboolean utils_isbrace(gchar c)
+gboolean utils_isbrace(gchar c, gboolean include_angles)
 {
 	switch (c)
 	{
-		// match < and > only if desired, because I don't like it, but some people do
 		case '<':
 		case '>':
-		return app->brace_match_ltgt;
+		return include_angles;
 
 		case '(':
 		case ')':
@@ -166,13 +171,12 @@ gboolean utils_isbrace(gchar c)
 }
 
 
-gboolean utils_is_opening_brace(gchar c)
+gboolean utils_is_opening_brace(gchar c, gboolean include_angles)
 {
 	switch (c)
 	{
-		// match < only if desired, because I don't like it, but some people do
 		case '<':
-		return app->brace_match_ltgt;
+		return include_angles;
 
 		case '(':
 		case '{':
@@ -182,27 +186,7 @@ gboolean utils_is_opening_brace(gchar c)
 }
 
 
-/* returns the line of the given tag */
-gint utils_get_local_tag(gint idx, const gchar *qual_name)
-{
-	guint line;
-	gchar *spos;
-
-	g_return_val_if_fail((doc_list[idx].sci && qual_name), -1);
-
-	spos = strchr(qual_name, '[');
-	if (spos && strchr(spos+1, ']'))
-	{
-		line = atol(spos + 1);
-		if (line > 0)
-		{
-			return line;
-		}
-	}
-	return -1;
-}
-
-
+// line is counted with 1 as the first line, not 0
 gboolean utils_goto_file_line(const gchar *file, gboolean is_tm_filename, gint line)
 {
 	gint file_idx = document_find_by_filename(file, is_tm_filename);
@@ -213,6 +197,7 @@ gboolean utils_goto_file_line(const gchar *file, gboolean is_tm_filename, gint l
 }
 
 
+// line is counted with 1 as the first line, not 0
 gboolean utils_goto_line(gint idx, gint line)
 {
 	gint page_num;
@@ -249,7 +234,7 @@ gint utils_write_file(const gchar *filename, const gchar *text)
 
 	len = strlen(text);
 
-	fp = fopen(filename, "w");
+	fp = g_fopen(filename, "w");
 	if (fp != NULL)
 	{
 		bytes_written = fwrite(text, sizeof (gchar), len, fp);
@@ -264,7 +249,8 @@ gint utils_write_file(const gchar *filename, const gchar *text)
 	}
 	else
 	{
-		geany_debug("utils_write_file(): could not write to file %s", filename);
+		geany_debug("utils_write_file(): could not write to file %s (%s)",
+			filename, g_strerror(errno));
 		return errno;
 	}
 	return 0;
@@ -283,7 +269,8 @@ gchar *utils_find_open_xml_tag(const gchar sel[], gint size, gboolean check_tag)
 	gchar *result = g_malloc(max_tag_size);
 	const gchar *begin, *cur;
 
-	if (size < 3) {
+	if (size < 3)
+	{
 		// Smallest tag is "<p>" which is 3 characters
 		return result;
 	}
@@ -348,12 +335,12 @@ gboolean utils_check_disk_status(gint idx, gboolean force)
 	}
 	else if (doc_list[idx].mtime < st.st_mtime)
 	{
-		gchar *basename = g_path_get_basename(doc_list[idx].file_name);
+		gchar *base_name = g_path_get_basename(doc_list[idx].file_name);
 
-		if (dialogs_show_question_full(_("_Reload"), GTK_STOCK_CANCEL,
+		if (dialogs_show_question_full(NULL, _("_Reload"), GTK_STOCK_CANCEL,
 			_("Do you want to reload it?"),
 			_("The file '%s' on the disk is more recent than\n"
-				"the current buffer."), basename))
+				"the current buffer."), base_name))
 		{
 			document_reload_file(idx, NULL);
 			doc_list[idx].last_check = t;
@@ -361,7 +348,7 @@ gboolean utils_check_disk_status(gint idx, gboolean force)
 		else
 			doc_list[idx].mtime = st.st_mtime;
 
-		g_free(basename);
+		g_free(base_name);
 		ret = TRUE; // file has changed
 	}
 	g_free(locale_filename);
@@ -519,7 +506,7 @@ gint utils_get_current_function(gint idx, const gchar **tagname)
 		return tag_line;
 	}
 
-	line = sci_get_current_line(doc_list[idx].sci, -1);
+	line = sci_get_current_line(doc_list[idx].sci);
 	fold_level = sci_get_fold_level(doc_list[idx].sci, line);
 	// check if the cached line and file index have changed since last time:
 	if (! current_function_changed(idx, line, fold_level))
@@ -636,58 +623,6 @@ gboolean utils_atob(const gchar *str)
 }
 
 
-/* (stolen from bluefish, thanks)
- * Returns number of characters, lines and words in the supplied gchar*.
- * Handles UTF-8 correctly. Input must be properly encoded UTF-8.
- * Words are defined as any characters grouped, separated with spaces.
- */
-void utils_wordcount(gchar *text, guint *chars, guint *lines, guint *words)
-{
-	guint in_word = 0;
-	gunichar utext;
-
-	if (!text) return; // politely refuse to operate on NULL
-
-	*chars = *words = *lines = 0;
-	while (*text != '\0')
-	{
-		(*chars)++;
-
-		switch (*text)
-		{
-			case '\n':
-				(*lines)++;
-			case '\r':
-			case '\f':
-			case '\t':
-			case ' ':
-			case '\v':
-				mb_word_separator:
-				if (in_word)
-				{
-					in_word = 0;
-					(*words)++;
-				}
-				break;
-			default:
-				utext = g_utf8_get_char_validated(text, 2); // This might be an utf-8 char
-				if (g_unichar_isspace(utext)) // Unicode encoded space?
-					goto mb_word_separator;
-				if (g_unichar_isgraph(utext)) // Is this something printable?
-					in_word = 1;
-				break;
-		}
-		text = g_utf8_next_char(text); // Even if the current char is 2 bytes, this will iterate correctly.
-	}
-
-	// Capture last word, if there's no whitespace at the end of the file.
-	if (in_word) (*words)++;
-	// We start counting line numbers from 1
-	if (*chars > 0) (*lines)++;
-}
-
-
-/* currently unused */
 gboolean utils_is_absolute_path(const gchar *path)
 {
 	if (! path || *path == '\0')
@@ -810,14 +745,13 @@ gint utils_make_settings_dir(const gchar *dir, const gchar *data_dir, const gcha
 
 	if (error_nr == 0 && ! g_file_test(conf_file, G_FILE_TEST_EXISTS))
 	{	// try to write geany.conf
-		error_nr = utils_write_file(conf_file, "");
-
- 		if (error_nr == 0 && ! g_file_test(conf_file, G_FILE_TEST_EXISTS))
-		{ // check whether write test was successful, otherwise directory is not writable
-			geany_debug("The chosen configuration directory is not writable.");
-			errno = EPERM;
-		}
+		error_nr = access(app->configdir, W_OK);
 	}
+#ifdef G_OS_WIN32
+# define DIR_SEP "\\" // on Windows we need an additional dir separator
+#else
+# define DIR_SEP ""
+#endif
 
 	// make subdir for filetype definitions
 	if (error_nr == 0)
@@ -834,7 +768,7 @@ gint utils_make_settings_dir(const gchar *dir, const gchar *data_dir, const gcha
 			gchar *text = g_strconcat(
 "Copy files from ", data_dir, " to this directory to overwrite "
 "them. To use the defaults, just delete the file in this directory.\nFor more information read "
-"the documentation (in ", doc_dir, "index.html or visit " GEANY_HOMEPAGE ").", NULL);
+"the documentation (in ", doc_dir, DIR_SEP "index.html or visit " GEANY_HOMEPAGE ").", NULL);
 			utils_write_file(filedefs_readme, text);
 			g_free(text);
 		}
@@ -855,7 +789,7 @@ gint utils_make_settings_dir(const gchar *dir, const gchar *data_dir, const gcha
 		{
 			gchar *text = g_strconcat(
 "There are several template files in this directory. For these templates you can use wildcards.\n\
-For more information read the documentation (in ", doc_dir, "index.html or visit " GEANY_HOMEPAGE ").",
+For more information read the documentation (in ", doc_dir, DIR_SEP "index.html or visit " GEANY_HOMEPAGE ").",
 					NULL);
 			utils_write_file(templates_readme, text);
 			g_free(text);
@@ -871,8 +805,9 @@ For more information read the documentation (in ", doc_dir, "index.html or visit
 }
 
 
-/* replaces all occurrences of needle in haystack with replacement
- * all strings have to NULL-terminated and needle and replacement have to be different,
+/* Replaces all occurrences of needle in haystack with replacement.
+ * New code should use utils_string_replace_all() instead.
+ * All strings have to be NULL-terminated and needle and replacement have to be different,
  * e.g. needle "%" and replacement "%%" causes an endless loop */
 gchar *utils_str_replace(gchar *haystack, const gchar *needle, const gchar *replacement)
 {
@@ -882,12 +817,20 @@ gchar *utils_str_replace(gchar *haystack, const gchar *needle, const gchar *repl
 	gchar *result;
 	GString *str;
 
-	if (haystack == NULL) return NULL;
+	if (haystack == NULL)
+		return NULL;
+
+	if (needle == NULL || replacement == NULL)
+		return haystack;
+
+	if (utils_str_equal(needle, replacement))
+		return haystack;
 
 	start = strstr(haystack, needle);
 	lt_pos = utils_strpos(haystack, needle);
 
-	if (start == NULL || lt_pos == -1) return haystack;
+	if (start == NULL || lt_pos == -1)
+		return haystack;
 
 	// substitute by copying
 	str = g_string_sized_new(strlen(haystack));
@@ -1124,7 +1067,7 @@ gchar *utils_get_current_file_dir()
 /* very simple convenience function */
 void utils_beep()
 {
-	if (app->beep_on_errors) gdk_beep();
+	if (prefs.beep_on_errors) gdk_beep();
 }
 
 
@@ -1241,7 +1184,11 @@ GIOChannel *utils_set_up_io_channel(
 	GIOChannel *ioc;
 	//const gchar *encoding;
 
+	#ifdef G_OS_WIN32
+	ioc = g_io_channel_win32_new_fd(fd);
+	#else
 	ioc = g_io_channel_unix_new(fd);
+	#endif
 
 	if (nblock)
 		g_io_channel_set_flags(ioc, G_IO_FLAG_NONBLOCK, NULL);
@@ -1469,41 +1416,10 @@ gchar *utils_get_utf8_from_locale(const gchar *locale_text)
 }
 
 
-/* Returns a string containing whitespace of the amount a according to the
- * setting app->pref_editor_use_tabs filled with simple space characters or with the right amount
- * of tabulator characters (a is filled with tabulators *and* spaces if a isn't a multiple of
- * app->pref_editor_tab_width).
- * If alternative is set to TRUE, it returns the opposite of app->pref_editor_use_tabs. */
-gchar *utils_get_whitespace(gint a, gboolean alternative)
-{
-	gchar *str;
-	gboolean use_tabs;
-
-	g_return_val_if_fail(a > 0, NULL);
-
-	use_tabs = (alternative) ? ! app->pref_editor_use_tabs : app->pref_editor_use_tabs;
-
-	if (use_tabs)
-	{	// first fill text with tabluators and fill the rest with spaces
-		gint tabs = a / app->pref_editor_tab_width;
-		gint spaces = a % app->pref_editor_tab_width;
-		gint len = tabs + spaces;
-
-		str = g_malloc(len + 1);
-
-		memset(str, '\t', tabs);
-		memset(str + tabs, ' ', spaces);
-		str[len] = '\0';
- 	}
-	else
-		str = g_strnfill(a, ' ');
-
-	return str;
-}
-
-
-/* Frees all passed pointers if they are non-NULL, the first argument is nothing special,
- * it will also be freed, the list should be ended with NULL */
+/* Frees all passed pointers if they are *ALL* non-NULL.
+ * Do not use if any pointers may be NULL.
+ * The first argument is nothing special, it will also be freed.
+ * The list must be ended with NULL. */
 void utils_free_pointers(gpointer first, ...)
 {
 	va_list a;
@@ -1685,3 +1601,51 @@ GSList *utils_get_file_list(const gchar *path, guint *length, GError **error)
 }
 
 
+/* returns TRUE if any letter in str is a capital, FALSE otherwise. Should be Unicode safe. */
+gboolean utils_str_has_upper(const gchar *str)
+{
+	gunichar c;
+
+	if (str == NULL || *str == '\0' || ! g_utf8_validate(str, -1, NULL))
+		return FALSE;
+
+	while (*str != '\0')
+	{
+		c = g_utf8_get_char(str);
+		// check only letters and stop once the first non-capital was found
+		if (g_unichar_isalpha(c) && g_unichar_isupper(c))
+			return TRUE;
+		str = g_utf8_next_char(str);
+	}
+	return FALSE;
+}
+
+
+/* Replaces all occurrences of needle in str with replace.
+ * Currently this is not safe if replace matches needle.
+ * Returns: TRUE if needle was found. */
+gboolean utils_string_replace_all(GString *str, const gchar *needle, const gchar *replace)
+{
+	const gchar *c;
+	gssize pos = -1;
+
+	g_return_val_if_fail(NZV(needle), FALSE);
+	g_return_val_if_fail(! NZV(replace) || strstr(replace, needle) == NULL, FALSE);
+
+	while (1)
+	{
+		c = strstr(str->str, needle);
+		if (c == NULL)
+			break;
+		else
+		{
+			pos = c - str->str;
+			g_string_erase(str, pos, strlen(needle));
+			if (replace)
+				g_string_insert(str, pos, replace);
+		}
+	}
+	return (pos != -1);
+}
+
+
