@@ -18,24 +18,36 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: utils.h 1270 2007-02-12 18:42:15Z eht16 $
+ * $Id: utils.h 1914 2007-09-27 15:09:54Z ntrel $
  */
 
 
 #ifndef GEANY_UTILS_H
 #define GEANY_UTILS_H 1
 
+// Returns: TRUE if ptr points to a non-zero value.
+#define NZV(ptr) \
+	((ptr) && (ptr)[0])
+
+/* Free's ptr (if not NULL), then assigns result to it.
+ * result can be an expression using the 'old' value of ptr.
+ * It prevents a memory leak compared with: ptr = func(ptr); */
+#define setptr(ptr, result)\
+	{\
+		gpointer setptr_tmp = ptr;\
+		ptr = result;\
+		g_free(setptr_tmp);\
+	}
+
 
 void utils_start_browser(const gchar *uri);
 
 /* taken from anjuta, to determine the EOL mode of the file */
 gint utils_get_line_endings(gchar* buffer, glong size);
 
-gboolean utils_isbrace(gchar c);
+gboolean utils_isbrace(gchar c, gboolean include_angles);
 
-gboolean utils_is_opening_brace(gchar c);
-
-gint utils_get_local_tag(gint idx, const gchar *qual_name);
+gboolean utils_is_opening_brace(gchar c, gboolean include_angles);
 
 gboolean utils_goto_file_line(const gchar *file, gboolean is_tm_filename, gint line);
 
@@ -66,8 +78,6 @@ gchar *utils_btoa(gboolean sbool);
 
 gboolean utils_atob(const gchar *str);
 
-void utils_wordcount(gchar *text, guint *chars, guint *lines, guint *words);
-
 gboolean utils_is_absolute_path(const gchar *path);
 
 gdouble utils_scale_round(gdouble val, gdouble factor);
@@ -89,10 +99,14 @@ gchar *utils_get_hostname();
 
 gint utils_make_settings_dir(const gchar *dir, const gchar *data_dir, const gchar *doc_dir);
 
+
+gboolean utils_string_replace_all(GString *str, const gchar *needle, const gchar *replace);
+
 gchar *utils_str_replace(gchar *haystack, const gchar *needle, const gchar *replacement);
 
 gint utils_strpos(const gchar* haystack, const gchar * needle);
 
+
 gchar *utils_get_date_time();
 
 gchar *utils_get_date();
@@ -109,8 +123,6 @@ void utils_switch_document(gint direction);
 
 void utils_replace_filename(gint idx);
 
-gint utils_compare_symbol(const GeanySymbol *a, const GeanySymbol *b);
-
 gchar *utils_get_hex_from_color(GdkColor *color);
 
 gchar *utils_get_current_file_dir();
@@ -128,7 +140,8 @@ gint utils_strtod(const gchar *source, gchar **end, gboolean with_route);
 // returned string must be freed.
 gchar *utils_get_current_time_string();
 
-GIOChannel *utils_set_up_io_channel(gint fd, GIOCondition cond, GIOFunc func, gpointer data);
+GIOChannel *utils_set_up_io_channel(gint fd, GIOCondition cond, gboolean nblock,
+									GIOFunc func, gpointer data);
 
 gchar **utils_read_file_in_array(const gchar *filename);
 
@@ -146,8 +159,25 @@ gchar *utils_get_locale_from_utf8(const gchar *utf8_text);
 /* Simple wrapper for g_locale_to_utf8; returns a copy of locale_text on failure. */
 gchar *utils_get_utf8_from_locale(const gchar *locale_text);
 
-/* frees all passed pointers if they are non-NULL, the first argument is nothing special,
- * it will also be freed, the list should be ended with NULL */
-void utils_free_pointers(gpointer first, ...);
+
+/* Frees all passed pointers if they are *ALL* non-NULL.
+ * Do not use if any pointers may be NULL.
+ * The first argument is nothing special, it will also be freed.
+ * The list must be ended with NULL. */
+void utils_free_pointers(gpointer first, ...) G_GNUC_NULL_TERMINATED;
+
+/* Creates a string array deep copy of a series of non-NULL strings.
+ * The first argument is nothing special.
+ * The list must be ended with NULL.
+ * If first is NULL, NULL is returned. */
+gchar **utils_strv_new(gchar *first, ...) G_GNUC_NULL_TERMINATED;
+
+
+gint utils_mkdir(const gchar *path, gboolean create_parent_dirs);
+
+/* Gets a sorted list of files in the specified directory. */
+GSList *utils_get_file_list(const gchar *path, guint *length, GError **error);
+
+gboolean utils_str_has_upper(const gchar *str);
 
 #endif
