@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: vte.c 1464 2007-04-19 13:44:42Z eht16 $
+ * $Id: vte.c 1929 2007-10-03 11:25:28Z ntrel $
  */
 
 /*
@@ -39,6 +39,8 @@
 #include "prefs.h"
 #include "ui_utils.h"
 #include "utils.h"
+#include "document.h"
+#include "callbacks.h"
 
 
 VteInfo vte_info;
@@ -58,6 +60,7 @@ static const gchar VTE_WORDCHARS[] = "-A-Za-z0-9,./?%&#:_";
 #define VTE_TERMINAL(obj) (GTK_CHECK_CAST((obj), VTE_TYPE_TERMINAL, VteTerminal))
 #define VTE_TYPE_TERMINAL (vf->vte_terminal_get_type())
 
+static void create_vte();
 static void vte_start(GtkWidget *widget);
 static gboolean vte_button_pressed(GtkWidget *widget, GdkEventButton *event, gpointer user_data);
 static gboolean vte_keypress(GtkWidget *widget, GdkEventKey *event, gpointer data);
@@ -66,6 +69,15 @@ static void vte_popup_menu_clicked(GtkMenuItem *menuitem, gpointer user_data);
 static GtkWidget *vte_create_popup_menu(void);
 
 
+enum
+{
+	POPUP_COPY,
+	POPUP_PASTE,
+	POPUP_CHANGEPATH,
+	POPUP_PREFERENCES
+};
+
+
 /* taken from anjuta, thanks */
 static gchar **vte_get_child_environment(void)
 {
@@ -123,9 +135,6 @@ static void override_menu_key()
 
 void vte_init(void)
 {
-
-	GtkWidget *vte, *scrollbar, *hbox, *frame;
-
 	if (vte_info.have_vte == FALSE)
 	{	// app->have_vte can be false, even if VTE is compiled in, think of command line option
 		geany_debug("Disabling terminal support");
@@ -161,6 +170,17 @@ void vte_init(void)
 		vte_register_symbols(module);
 	}
 
+	create_vte();
+
+	// setup the f10 menu override (so it works before the widget is first realized).
+	override_menu_key();
+}
+
+
+static void create_vte()
+{
+	GtkWidget *vte, *scrollbar, *hbox, *frame;
+
 	vte = vf->vte_terminal_new();
 	vc->vte = vte;
 	scrollbar = gtk_vscrollbar_new(GTK_ADJUSTMENT(VTE_TERMINAL(vte)->adjustment));
@@ -185,6 +205,7 @@ void vte_init(void)
 	g_signal_connect(G_OBJECT(vte), "child-exited", G_CALLBACK(vte_start), NULL);
 	g_signal_connect(G_OBJECT(vte), "button-press-event", G_CALLBACK(vte_button_pressed), NULL);
 	g_signal_connect(G_OBJECT(vte), "event", G_CALLBACK(vte_keypress), NULL);
+	g_signal_connect(G_OBJECT(vte), "motion-notify-event", G_CALLBACK(on_motion_event), NULL);
 	//g_signal_connect(G_OBJECT(vte), "drag-data-received", G_CALLBACK(vte_drag_data_received), NULL);
 	//g_signal_connect(G_OBJECT(vte), "drag-drop", G_CALLBACK(vte_drag_drop), NULL);
 
@@ -195,9 +216,6 @@ void vte_init(void)
 
 	// the vte widget has to be realised before color changes take effect
 	g_signal_connect(G_OBJECT(vte), "realize", G_CALLBACK(vte_apply_user_settings), NULL);
-
-	// setup the f10 menu override (so it works before the widget is first realized).
-	override_menu_key();
 }
 
 
@@ -222,14 +240,15 @@ void vte_close(void)
 
 static gboolean vte_keypress(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
-	if (event->type != GDK_KEY_RELEASE)
+	if (event->type != GDK_KEY_RELEASE || vc->enable_bash_keys)
 		return FALSE;
 
 	if ((event->keyval == GDK_c ||
 		event->keyval == GDK_d ||
 		event->keyval == GDK_C ||
 		event->keyval == GDK_D) &&
-		event->state & GDK_CONTROL_MASK)
+		event->state & GDK_CONTROL_MASK &&
+		! (event->state & GDK_SHIFT_MASK) && ! (event->state & GDK_MOD1_MASK))
 	{
 		vte_get_working_directory(); // try to keep the working directory when restarting the VTE
 
@@ -243,7 +262,6 @@ static gboolean vte_keypress(GtkWidget *widget, GdkEventKey *event, gpointer dat
 
 		return TRUE;
 	}
-
 	return FALSE;
 }
 
@@ -316,7 +334,7 @@ static void vte_register_symbols(GModule *mod)
 
 void vte_apply_user_settings(void)
 {
-	if (! app->msgwindow_visible) return;
+	if (! ui_prefs.msgwindow_visible) return;
 	//if (! GTK_WIDGET_REALIZED(vc->vte)) gtk_widget_realize(vc->vte);
 	vf->vte_terminal_set_scrollback_lines(VTE_TERMINAL(vc->vte), vc->scrollback_lines);
 	vf->vte_terminal_set_scroll_on_keystroke(VTE_TERMINAL(vc->vte), vc->scroll_on_key);
@@ -334,24 +352,31 @@ static void vte_popup_menu_clicked(GtkMenuItem *menuitem, gpointer user_data)
 {
 	switch (GPOINTER_TO_INT(user_data))
 	{
-		case 0:
+		case POPUP_COPY:
 		{
 			if (vf->vte_terminal_get_has_selection(VTE_TERMINAL(vc->vte)))
 				vf->vte_terminal_copy_clipboard(VTE_TERMINAL(vc->vte));
 			break;
 		}
-		case 1:
+		case POPUP_PASTE:
 		{
 			vf->vte_terminal_paste_clipboard(VTE_TERMINAL(vc->vte));
 			break;
 		}
-		case 2:
+		case POPUP_CHANGEPATH:
+		{
+			gint idx = document_get_cur_idx();
+			if (DOC_IDX_VALID(idx))
+				vte_cwd(doc_list[idx].file_name, TRUE);
+			break;
+		}
+		case POPUP_PREFERENCES:
 		{
 			GtkWidget *notebook;
 
 			prefs_show_dialog();
 
-			notebook = lookup_widget(app->prefs_dialog, "notebook2");
+			notebook = lookup_widget(ui_widgets.prefs_dialog, "notebook2");
 
 			gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), vte_prefs_tab_num);
 			break;
@@ -369,12 +394,17 @@ static GtkWidget *vte_create_popup_menu(void)
 	item = gtk_image_menu_item_new_from_stock("gtk-copy", NULL);
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect((gpointer)item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(0));
+	g_signal_connect((gpointer)item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_COPY));
 
 	item = gtk_image_menu_item_new_from_stock("gtk-paste", NULL);
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect((gpointer)item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(1));
+	g_signal_connect((gpointer)item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_PASTE));
+
+	item = gtk_image_menu_item_new_with_label("Change current working directory");
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(menu), item);
+	g_signal_connect((gpointer)item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_CHANGEPATH));
 
 	item = gtk_separator_menu_item_new();
 	gtk_widget_show(item);
@@ -383,7 +413,7 @@ static GtkWidget *vte_create_popup_menu(void)
 	item = gtk_image_menu_item_new_from_stock("gtk-preferences", NULL);
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	g_signal_connect((gpointer)item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(2));
+	g_signal_connect((gpointer)item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_PREFERENCES));
 
 	item = gtk_separator_menu_item_new();
 	gtk_widget_show(item);
@@ -516,8 +546,8 @@ void vte_append_preferences_tab()
 		GtkTooltips *tooltips;
 		GtkObject *spin_scrollback_adj;
 
-		tooltips = GTK_TOOLTIPS(lookup_widget(app->prefs_dialog, "tooltips"));
-		notebook = lookup_widget(app->prefs_dialog, "notebook2");
+		tooltips = GTK_TOOLTIPS(lookup_widget(ui_widgets.prefs_dialog, "tooltips"));
+		notebook = lookup_widget(ui_widgets.prefs_dialog, "notebook2");
 
 		frame = ui_frame_new_with_alignment(_("Terminal plugin"), &alignment);
 		vbox = gtk_vbox_new(FALSE, 12);
@@ -653,27 +683,27 @@ void vte_append_preferences_tab()
 
 		gtk_box_pack_start(GTK_BOX(vbox), box, FALSE, FALSE, 0);
 
-		g_object_set_data_full(G_OBJECT(app->prefs_dialog), "font_term",
+		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "font_term",
 				gtk_widget_ref(font_term),	(GDestroyNotify) gtk_widget_unref);
-		g_object_set_data_full(G_OBJECT(app->prefs_dialog), "color_fore",
+		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "color_fore",
 				gtk_widget_ref(color_fore),	(GDestroyNotify) gtk_widget_unref);
-		g_object_set_data_full(G_OBJECT(app->prefs_dialog), "color_back",
+		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "color_back",
 				gtk_widget_ref(color_back),	(GDestroyNotify) gtk_widget_unref);
-		g_object_set_data_full(G_OBJECT(app->prefs_dialog), "spin_scrollback",
+		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "spin_scrollback",
 				gtk_widget_ref(spin_scrollback),	(GDestroyNotify) gtk_widget_unref);
-		g_object_set_data_full(G_OBJECT(app->prefs_dialog), "entry_emulation",
+		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "entry_emulation",
 				gtk_widget_ref(entry_emulation),	(GDestroyNotify) gtk_widget_unref);
-		g_object_set_data_full(G_OBJECT(app->prefs_dialog), "entry_shell",
+		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "entry_shell",
 				gtk_widget_ref(entry_shell),	(GDestroyNotify) gtk_widget_unref);
-		g_object_set_data_full(G_OBJECT(app->prefs_dialog), "check_scroll_key",
+		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "check_scroll_key",
 				gtk_widget_ref(check_scroll_key),	(GDestroyNotify) gtk_widget_unref);
-		g_object_set_data_full(G_OBJECT(app->prefs_dialog), "check_scroll_out",
+		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "check_scroll_out",
 				gtk_widget_ref(check_scroll_out),	(GDestroyNotify) gtk_widget_unref);
-		g_object_set_data_full(G_OBJECT(app->prefs_dialog), "check_ignore_menu_key",
+		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "check_ignore_menu_key",
 				gtk_widget_ref(check_ignore_menu_key),	(GDestroyNotify) gtk_widget_unref);
-		g_object_set_data_full(G_OBJECT(app->prefs_dialog), "check_follow_path",
+		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "check_follow_path",
 				gtk_widget_ref(check_follow_path),	(GDestroyNotify) gtk_widget_unref);
-		g_object_set_data_full(G_OBJECT(app->prefs_dialog), "check_run_in_vte",
+		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "check_run_in_vte",
 				gtk_widget_ref(check_run_in_vte),	(GDestroyNotify) gtk_widget_unref);
 
 		gtk_widget_show_all(frame);
