@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: win32.c 1369 2007-03-06 16:57:09Z ntrel $
+ *  $Id: win32.c 1878 2007-09-12 16:25:19Z ntrel $
  */
 
 /*
@@ -29,6 +29,8 @@
 
 #ifdef G_OS_WIN32
 
+#define VC_EXTRALEAN
+#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #include <commdlg.h>
 #include <shlobj.h>
@@ -48,6 +50,7 @@
 #include "ui_utils.h"
 #include "sciwrappers.h"
 #include "dialogs.h"
+#include "filetypes.h"
 
 
 
@@ -60,14 +63,14 @@ static gchar *win32_get_file_filters()
 	GString *all_patterns = g_string_sized_new(100);
 	gchar *tmp;
 
-	for (i = 0; filetypes[i] != NULL; i++)
+	for (i = 0; i < GEANY_MAX_FILE_TYPES; i++)
 	{
 		tmp = g_strjoinv(";", filetypes[i]->pattern);
 		g_string_append_printf(str, "%s\t%s\t", filetypes[i]->title, tmp);
 		g_free(tmp);
 	}
 	// create meta file filter "All Source"
-	for (i = 0; filetypes[i] != NULL; i++)
+	for (i = 0; i < GEANY_MAX_FILE_TYPES; i++)
 	{
 		for (j = 0; filetypes[i]->pattern[j] != NULL; j++)
 		{
@@ -234,7 +237,7 @@ gchar *win32_show_project_open_dialog(const gchar *title, const gchar *initial_d
 		{
 			gchar *error;
 			error = g_strdup_printf("File dialog box error (%x)", (int)CommDlgExtendedError());
-			win32_message_dialog(GTK_MESSAGE_ERROR, error);
+			win32_message_dialog(NULL, GTK_MESSAGE_ERROR, error);
 			g_free(error);
 		}
 		g_free(fname);
@@ -294,7 +297,7 @@ gboolean win32_show_file_dialog(gboolean file_open)
 		{
 			gchar error[100];
 			snprintf(error, sizeof error, "File dialog box error (%x)", (int)CommDlgExtendedError());
-			win32_message_dialog(GTK_MESSAGE_ERROR, error);
+			win32_message_dialog(NULL, GTK_MESSAGE_ERROR, error);
 		}
 		g_free(fname);
 		return FALSE;
@@ -308,7 +311,7 @@ gboolean win32_show_file_dialog(gboolean file_open)
 		x = of.nFileOffset - 1;
 		if (x != strlen(fname))
 		{	// open a single file
-			document_open_file(-1, fname, 0, of.Flags & OFN_READONLY, NULL, NULL);
+			document_open_file(fname, of.Flags & OFN_READONLY, NULL, NULL);
 		}
 		else
 		{	// open multiple files
@@ -321,7 +324,7 @@ gboolean win32_show_file_dialog(gboolean file_open)
 					if (! fname[x+1]) break;
 
 					g_snprintf(file_name, 254, "%s\\%s", fname, fname + x + 1);
-					document_open_file(-1, file_name, 0, of.Flags & OFN_READONLY, NULL, NULL);
+					document_open_file(file_name, of.Flags & OFN_READONLY, NULL, NULL);
 				}
 				x++;
 			}
@@ -392,12 +395,12 @@ void win32_show_color_dialog(const gchar *colour)
 	if (ChooseColor(&cc))
 	{
 		rgb_current = cc.rgbResult;
-		g_snprintf(hex, 11, "#%2X%2X%2X",
+		g_snprintf(hex, 11, "#%02X%02X%02X",
 	      (guint) (utils_scale_round(GetRValue(rgb_current), 255)),
 	      (guint) (utils_scale_round(GetGValue(rgb_current), 255)),
 	      (guint) (utils_scale_round(GetBValue(rgb_current), 255)));
 
-		sci_add_text(doc_list[idx].sci, hex);
+		document_insert_colour(idx, hex);
 	}
 	g_free(hex);
 }
@@ -453,7 +456,7 @@ void win32_show_pref_file_dialog(GtkEntry *item)
 		{
 			gchar error[100];
 			snprintf(error, sizeof error, "File dialog box error (%x)", (int)CommDlgExtendedError());
-			win32_message_dialog(GTK_MESSAGE_ERROR, error);
+			win32_message_dialog(NULL, GTK_MESSAGE_ERROR, error);
 		}
 		g_strfreev(field);
 		g_free(fname);
@@ -483,7 +486,7 @@ void win32_show_pref_file_dialog(GtkEntry *item)
 /* Creates a native Windows message box of the given type and returns always TRUE
  * or FALSE representing th pressed Yes or No button.
  * If type is not GTK_MESSAGE_QUESTION, it returns always TRUE. */
-gboolean win32_message_dialog(GtkMessageType type, const gchar *msg)
+gboolean win32_message_dialog(GtkWidget *parent, GtkMessageType type, const gchar *msg)
 {
 	gboolean ret = TRUE;
 	gint rc;
@@ -522,11 +525,14 @@ gboolean win32_message_dialog(GtkMessageType type, const gchar *msg)
 
 	// convert the Unicode chars to wide chars
 	/// TODO test if LANG == C then possibly skip conversion => g_win32_getlocale()
-	MultiByteToWideChar(CP_UTF8, 0, msg, -1, w_msg, sizeof(w_msg)/sizeof(w_msg[0]));
-	MultiByteToWideChar(CP_UTF8, 0, title, -1, w_title, sizeof(w_title)/sizeof(w_title[0]));
+	MultiByteToWideChar(CP_UTF8, 0, msg, -1, w_msg, G_N_ELEMENTS(w_msg));
+	MultiByteToWideChar(CP_UTF8, 0, title, -1, w_title, G_N_ELEMENTS(w_title));
 
 	// display the message box
-	rc = MessageBoxW(GDK_WINDOW_HWND(app->window->window), w_msg, w_title, t);
+	if (parent == NULL)
+		parent = app->window;
+
+	rc = MessageBoxW(GDK_WINDOW_HWND(parent->window), w_msg, w_title, t);
 
 	if (type == GTK_MESSAGE_QUESTION && rc != IDYES)
 		ret = FALSE;
@@ -543,8 +549,8 @@ gint win32_message_dialog_unsaved(const gchar *msg)
 	gint ret;
 
 	// convert the Unicode chars to wide chars
-	MultiByteToWideChar(CP_UTF8, 0, msg, -1, w_msg, sizeof(w_msg)/sizeof(w_msg[0]));
-	MultiByteToWideChar(CP_UTF8, 0, _("Question"), -1, w_title, sizeof(w_title)/sizeof(w_title[0]));
+	MultiByteToWideChar(CP_UTF8, 0, msg, -1, w_msg, G_N_ELEMENTS(w_msg));
+	MultiByteToWideChar(CP_UTF8, 0, _("Question"), -1, w_title, G_N_ELEMENTS(w_title));
 
 	ret = MessageBoxW(GDK_WINDOW_HWND(app->window->window), w_msg, w_title, MB_YESNOCANCEL | MB_ICONQUESTION);
 	switch(ret)
