@@ -44,7 +44,8 @@ enum
 	TA_ACCESS,
 	TA_IMPL,
 	TA_LANG,
-	TA_INACTIVE
+	TA_INACTIVE,
+	TA_POINTER
 };
 
 static guint *s_sort_attrs = NULL;
@@ -136,6 +137,7 @@ gboolean tm_tag_init(TMTag *tag, TMSourceFile *file, const tagEntryInfo *tag_ent
 		tag->name = g_strdup(tag_entry->name);
 		tag->type = get_tag_type(tag_entry->kindName);
 		tag->atts.entry.local = tag_entry->isFileScope;
+		tag->atts.entry.pointerOrder = tag_entry->pointerOrder;
 		tag->atts.entry.line = tag_entry->lineNumber;
 		if (NULL != tag_entry->extensionFields.arglist)
 			tag->atts.entry.arglist = g_strdup(tag_entry->extensionFields.arglist);
@@ -197,11 +199,10 @@ TMTag *tm_tag_new(TMSourceFile *file, const tagEntryInfo *tag_entry)
 		TAG_FREE(tag);
 		return NULL;
 	}
-
 	return tag;
 }
 
-gboolean tm_tag_init_from_file(TMTag *tag, TMSourceFile *file, FILE *fp, gint mode)
+gboolean tm_tag_init_from_file(TMTag *tag, TMSourceFile *file, FILE *fp)
 {
 	guchar buf[BUFSIZ];
 	guchar *start, *end;
@@ -210,96 +211,110 @@ gboolean tm_tag_init_from_file(TMTag *tag, TMSourceFile *file, FILE *fp, gint mo
 
 	if ((NULL == fgets((gchar*)buf, BUFSIZ, fp)) || ('\0' == *buf))
 		return FALSE;
-	if (mode == 0 || mode == 1)
+	for (start = end = buf, status = TRUE; (TRUE == status); start = end, ++ end)
 	{
-		for (start = end = buf, status = TRUE; (TRUE == status); start = end, ++ end)
+		while ((*end < TA_NAME) && (*end != '\0') && (*end != '\n'))
+			++ end;
+		if (('\0' == *end) || ('\n' == *end))
+			status = FALSE;
+		changed_char = *end;
+		*end = '\0';
+		if (NULL == tag->name)
 		{
-			while ((*end < TA_NAME) && (*end != '\0') && (*end != '\n'))
-				++ end;
-			if (('\0' == *end) || ('\n' == *end))
-				status = FALSE;
-			changed_char = *end;
-			*end = '\0';
-			if (NULL == tag->name)
-			{
-				if (!isprint(*start))
-					return FALSE;
-				else
-					tag->name = g_strdup((gchar*)start);
-			}
+			if (!isprint(*start))
+				return FALSE;
 			else
-			{
-				switch (*start)
-				{
-					case TA_LINE:
-						tag->atts.entry.line = atol((gchar*)start + 1);
-						break;
-					case TA_LOCAL:
-						tag->atts.entry.local = atoi((gchar*)start + 1);
-						break;
-					case TA_TYPE:
-						tag->type = (TMTagType) atoi((gchar*)start + 1);
-						break;
-					case TA_ARGLIST:
-						tag->atts.entry.arglist = g_strdup((gchar*)start + 1);
-						break;
-					case TA_SCOPE:
-						tag->atts.entry.scope = g_strdup((gchar*)start + 1);
-						break;
-					case TA_VARTYPE:
-						tag->atts.entry.var_type = g_strdup((gchar*)start + 1);
-						break;
-					case TA_INHERITS:
-						tag->atts.entry.inheritance = g_strdup((gchar*)start + 1);
-						break;
-					case TA_TIME:
-						if (tm_tag_file_t != tag->type)
-						{
-							g_warning("Got time attribute for non-file tag %s", tag->name);
-							return FALSE;
-						}
-						else
-							tag->atts.file.timestamp = atol((gchar*)start + 1);
-						break;
-					case TA_LANG:
-						if (tm_tag_file_t != tag->type)
-						{
-							g_warning("Got lang attribute for non-file tag %s", tag->name);
-							return FALSE;
-						}
-						else
-							tag->atts.file.lang = atoi((gchar*)start + 1);
-						break;
-					case TA_INACTIVE:
-						if (tm_tag_file_t != tag->type)
-						{
-							g_warning("Got inactive attribute for non-file tag %s", tag->name);
-							return FALSE;
-						}
-						else
-							tag->atts.file.inactive = (gboolean) atoi((gchar*)start + 1);
-						break;
-					case TA_ACCESS:
-						tag->atts.entry.access = *(start + 1);
-						break;
-					case TA_IMPL:
-						tag->atts.entry.impl = *(start + 1);
-						break;
-					default:
-	#ifdef TM_DEBUG
-						g_warning("Unknown attribute %s", start + 1);
-	#endif
-						break;
-				}
-				tag->atts.file.lang = mode;
-			}
-			*end = changed_char;
+				tag->name = g_strdup((gchar*)start);
 		}
+		else
+		{
+			switch (*start)
+			{
+				case TA_LINE:
+					tag->atts.entry.line = atol((gchar*)start + 1);
+					break;
+				case TA_LOCAL:
+					tag->atts.entry.local = atoi((gchar*)start + 1);
+					break;
+				case TA_TYPE:
+					tag->type = (TMTagType) atoi((gchar*)start + 1);
+					break;
+				case TA_ARGLIST:
+					tag->atts.entry.arglist = g_strdup((gchar*)start + 1);
+					break;
+				case TA_SCOPE:
+					tag->atts.entry.scope = g_strdup((gchar*)start + 1);
+					break;
+				case TA_POINTER:
+					tag->atts.entry.pointerOrder = atoi((gchar*)start + 1);
+					break;
+				case TA_VARTYPE:
+					tag->atts.entry.var_type = g_strdup((gchar*)start + 1);
+					break;
+				case TA_INHERITS:
+					tag->atts.entry.inheritance = g_strdup((gchar*)start + 1);
+					break;
+				case TA_TIME:
+					if (tm_tag_file_t != tag->type)
+					{
+						g_warning("Got time attribute for non-file tag %s", tag->name);
+						return FALSE;
+					}
+					else
+						tag->atts.file.timestamp = atol((gchar*)start + 1);
+					break;
+				case TA_LANG:
+					if (tm_tag_file_t != tag->type)
+					{
+						g_warning("Got lang attribute for non-file tag %s", tag->name);
+						return FALSE;
+					}
+					else
+						tag->atts.file.lang = atoi((gchar*)start + 1);
+					break;
+				case TA_INACTIVE:
+					if (tm_tag_file_t != tag->type)
+					{
+						g_warning("Got inactive attribute for non-file tag %s", tag->name);
+						return FALSE;
+					}
+					else
+						tag->atts.file.inactive = (gboolean) atoi((gchar*)start + 1);
+					break;
+				case TA_ACCESS:
+					tag->atts.entry.access = *(start + 1);
+					break;
+				case TA_IMPL:
+					tag->atts.entry.impl = *(start + 1);
+					break;
+				default:
+#ifdef TM_DEBUG
+					g_warning("Unknown attribute %s", start + 1);
+#endif
+					break;
+			}
+		}
+		*end = changed_char;
 	}
-	else
+	if (NULL == tag->name)
+		return FALSE;
+	if (tm_tag_file_t != tag->type)
+		tag->atts.entry.file = file;
+	return TRUE;
+}
+
+// alternative parser for PHP and LaTeX global tags files with the following format
+// tagname|return value|arglist|description\n
+gboolean tm_tag_init_from_file_alt(TMTag *tag, TMSourceFile *file, FILE *fp)
+{
+	guchar buf[BUFSIZ];
+	guchar *start, *end;
+	gboolean status;
+	//guchar changed_char = TA_NAME;
+
+	if ((NULL == fgets((gchar*)buf, BUFSIZ, fp)) || ('\0' == *buf))
+		return FALSE;
 	{
-		// alternative parser for PHP and LaTeX global tags files with the following format
-		// tagname|return value|arglist|description\n
 		gchar **fields;
 		guint field_len;
 		for (start = end = buf, status = TRUE; (TRUE == status); start = end, ++ end)
@@ -320,7 +335,6 @@ gboolean tm_tag_init_from_file(TMTag *tag, TMSourceFile *file, FILE *fp, gint mo
 			else tag->name = NULL;
 			if (field_len >= 2 && fields[1] != NULL) tag->atts.entry.var_type = g_strdup(fields[1]);
 			if (field_len >= 3 && fields[2] != NULL) tag->atts.entry.arglist = g_strdup(fields[2]);
-			tag->atts.file.lang = mode;
 			tag->type = tm_tag_prototype_t;
 			g_strfreev(fields);
 		}
@@ -336,13 +350,28 @@ gboolean tm_tag_init_from_file(TMTag *tag, TMSourceFile *file, FILE *fp, gint mo
 TMTag *tm_tag_new_from_file(TMSourceFile *file, FILE *fp, gint mode)
 {
 	TMTag *tag;
+	gboolean result;
 
 	TAG_NEW(tag);
-	if (FALSE == tm_tag_init_from_file(tag, file, fp, mode))
+
+	switch (mode)
+	{
+		case 4:	// pascal
+		case 6:	// php
+		case 8:	// latex
+			result = tm_tag_init_from_file_alt(tag, file, fp);
+			break;
+
+		default:
+			result = tm_tag_init_from_file(tag, file, fp);
+	}
+
+	if (! result)
 	{
 		TAG_FREE(tag);
 		return NULL;
 	}
+	tag->atts.file.lang = mode;
 	return tag;
 }
 
@@ -372,6 +401,8 @@ gboolean tm_tag_write(TMTag *tag, FILE *fp, guint attrs)
 			fprintf(fp, "%c%s", TA_SCOPE, tag->atts.entry.scope);
 		if ((attrs & tm_tag_attr_inheritance_t) && (NULL != tag->atts.entry.inheritance))
 			fprintf(fp, "%c%s", TA_INHERITS, tag->atts.entry.inheritance);
+		if (attrs & tm_tag_attr_pointer_t)
+			fprintf(fp, "%c%d", TA_POINTER, tag->atts.entry.pointerOrder);
 		if ((attrs & tm_tag_attr_vartype_t) && (NULL != tag->atts.entry.var_type))
 			fprintf(fp, "%c%s", TA_VARTYPE, tag->atts.entry.var_type);
 		if ((attrs & tm_tag_attr_access_t) && (TAG_ACCESS_UNKNOWN != tag->atts.entry.access))
@@ -385,7 +416,7 @@ gboolean tm_tag_write(TMTag *tag, FILE *fp, guint attrs)
 		return FALSE;
 }
 
-void tm_tag_destroy(TMTag *tag)
+static void tm_tag_destroy(TMTag *tag)
 {
 	g_free(tag->name);
 	if (tm_tag_file_t != tag->type)
@@ -450,6 +481,14 @@ int tm_tag_compare(const void *ptr1, const void *ptr2)
 				if (0 != (returnval = strcmp(NVL(t1->atts.entry.scope, ""), NVL(t2->atts.entry.scope, ""))))
 					return returnval;
 				break;
+			case tm_tag_attr_arglist_t:
+				if (0 != (returnval = strcmp(NVL(t1->atts.entry.arglist, ""), NVL(t2->atts.entry.arglist, ""))))
+				{
+					int line_diff = (t1->atts.entry.line - t2->atts.entry.line);
+
+					return line_diff ? line_diff : returnval;
+				}
+				break;
 			case tm_tag_attr_vartype_t:
 				if (0 != (returnval = strcmp(NVL(t1->atts.entry.var_type, ""), NVL(t2->atts.entry.var_type, ""))))
 					return returnval;
@@ -486,7 +525,9 @@ gboolean tm_tags_dedup(GPtrArray *tags_array, TMTagAttrType *sort_attributes)
 	for (i = 1; i < tags_array->len; ++i)
 	{
 		if (0 == tm_tag_compare(&(tags_array->pdata[i - 1]), &(tags_array->pdata[i])))
+		{
 			tags_array->pdata[i-1] = NULL;
+		}
 	}
 	tm_tags_prune(tags_array);
 	return TRUE;
@@ -562,7 +603,8 @@ void tm_tags_array_free(GPtrArray *tags_array, gboolean free_all)
 	}
 }
 
-TMTag **tm_tags_find(GPtrArray *sorted_tags_array, const char *name, gboolean partial, int * tagCount)
+TMTag **tm_tags_find(const GPtrArray *sorted_tags_array, const char *name,
+		gboolean partial, int * tagCount)
 {
 	static TMTag *tag = NULL;
 	TMTag **result;
@@ -578,15 +620,30 @@ TMTag **tm_tags_find(GPtrArray *sorted_tags_array, const char *name, gboolean pa
 	s_partial = partial;
 	result = (TMTag **) bsearch(&tag, sorted_tags_array->pdata, sorted_tags_array->len
 	  , sizeof(gpointer), tm_tag_compare);
+	// there can be matches on both sides of result
 	if (result)
 	{
-		for (; result >= (TMTag **) sorted_tags_array->pdata; -- result) {
+		TMTag **last = (TMTag **) &sorted_tags_array->pdata[sorted_tags_array->len - 1];
+		TMTag **adv;
+
+		// First look for any matches after result
+		adv = result;
+		adv++;
+		for (; *adv && adv <= last; ++ adv)
+		{
+			if (0 != tm_tag_compare(&tag, adv))
+				break;
+			++tagMatches;
+		}
+		// Now look for matches from result and below
+		for (; result >= (TMTag **) sorted_tags_array->pdata; -- result)
+		{
 			if (0 != tm_tag_compare(&tag, (TMTag **) result))
 				break;
 			++tagMatches;
 		}
 		*tagCount=tagMatches;
-		++ result;
+		++ result;	// correct address for the last successful match
 	}
 	s_partial = FALSE;
 	return (TMTag **) result;
