@@ -19,6 +19,7 @@
 #ifdef HAVE_GLOB_H
 # include <glob.h>
 #endif
+#include <glib/gstdio.h>
 // handling of P_tmpdir, should be something like /tmp, take the root directory under Win32,
 // and assume /tmp on non-Win32 systems where P_tmpdir is not set
 #ifndef P_tmpdir
@@ -114,7 +115,7 @@ gboolean tm_workspace_add_object(TMWorkObject *work_object)
 	return TRUE;
 }
 
-gboolean tm_workspace_remove_object(TMWorkObject *w, gboolean free)
+gboolean tm_workspace_remove_object(TMWorkObject *w, gboolean do_free)
 {
 	guint i;
 	if ((NULL == theWorkspace) || (NULL == theWorkspace->work_objects)
@@ -124,7 +125,7 @@ gboolean tm_workspace_remove_object(TMWorkObject *w, gboolean free)
 	{
 		if (theWorkspace->work_objects->pdata[i] == w)
 		{
-			if (free)
+			if (do_free)
 				tm_work_object_free(w);
 			g_ptr_array_remove_index_fast(theWorkspace->work_objects, i);
 			tm_workspace_update(TM_WORK_OBJECT(theWorkspace), TRUE, FALSE, FALSE);
@@ -145,7 +146,7 @@ gboolean tm_workspace_load_global_tags(const char *tags_file, gint mode)
 	FILE *fp;
 	TMTag *tag;
 
-	if (NULL == (fp = fopen(tags_file, "r")))
+	if (NULL == (fp = g_fopen(tags_file, "r")))
 		return FALSE;
 	if (NULL == theWorkspace)
 		tm_create_workspace();
@@ -166,7 +167,7 @@ static guint tm_file_inode_hash(gconstpointer key)
 {
 	struct stat file_stat;
 	const char *filename = (const char*)key;
-	if (stat(filename, &file_stat) == 0)
+	if (g_stat(filename, &file_stat) == 0)
 	{
 #ifdef TM_DEBUG
 		g_message ("Hash for '%s' is '%d'\n", filename, file_stat.st_ino);
@@ -270,7 +271,7 @@ gboolean tm_workspace_create_global_tags(const char *pre_process, const char **i
 	char *temp_file2 = g_strdup_printf("%s/%d_%ld_2.cpp", P_tmpdir, getpid(), time(NULL));
 #endif
 
-	if (NULL == (fp = fopen(temp_file, "w")))
+	if (NULL == (fp = g_fopen(temp_file, "w")))
 		return FALSE;
 
 	includes_files_hash = g_hash_table_new_full (tm_file_inode_hash,
@@ -284,7 +285,7 @@ gboolean tm_workspace_create_global_tags(const char *pre_process, const char **i
 	for(idx_inc = 0; idx_inc < includes_count; idx_inc++)
 	{
  		int dirty_len = strlen(includes[idx_inc]);
-		char *clean_path = malloc(dirty_len - 1);
+		char *clean_path = g_malloc(dirty_len - 1);
 		strncpy(clean_path, includes[idx_inc] + 1, dirty_len - 1);
 		clean_path[dirty_len - 2] = 0;
 
@@ -401,7 +402,7 @@ gboolean tm_workspace_create_global_tags(const char *pre_process, const char **i
 		tm_source_file_free(source_file);
 		return FALSE;
 	}
-	if (NULL == (fp = fopen(tags_file, "w")))
+	if (NULL == (fp = g_fopen(tags_file, "w")))
 	{
 		tm_source_file_free(source_file);
 		return FALSE;
